# Copyright (c) 2015-2016, The Authors and Contributors
# <see AUTHORS file>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os

from supplychainpy._helpers._config_file_paths import ABS_FILE_PATH_APPLICATION_CONFIG
from supplychainpy._helpers._pickle_config import deserialise_config


class Config(object):
    pass


class ProdConfig(Config):
    PWD = os.path.abspath(os.curdir)
    DEBUG = False
    conf = deserialise_config(ABS_FILE_PATH_APPLICATION_CONFIG)
    db_path = conf.get('database_path', None)
    if db_path is not None and os.name in ['posix', 'mac']:
        SQLALCHEMY_DATABASE_URI = 'sqlite:///{}/reporting.db'.format(db_path)
    elif os.name == 'nt':
        SQLALCHEMY_DATABASE_URI = 'sqlite:///{}\\reporting.db'.format(db_path)
    SECRET_KEY = '|~G\xde\xa7\x9b\x1aKaZ-\xabk8\x0b\x12\xee)\xe0\xe0\x8b\x0c\xd9\x1d'
    SESSION_PROTECTION = 'strong'
    SQLALCHEMY_ECHO = False
    SQLALCHEMY_TRACK_MODIFICATIONS = True


class DevConfig(Config):
    PWD = os.path.abspath(os.curdir)
    DEBUG = True
    conf = deserialise_config(ABS_FILE_PATH_APPLICATION_CONFIG)
    db_path = conf.get('database_path', None)
    if os.name in ['posix', 'mac']:
        SQLALCHEMY_DATABASE_URI ='sqlite:///{}/reporting.db'.format(db_path)

    elif os.name == 'nt':
        SQLALCHEMY_DATABASE_URI = 'sqlite:///{}\\reporting.db'.format(db_path)
    SECRET_KEY = '|~G\xde\xa7\x9b\x1aKaZ-\xabk8\x0b\x12\xee)\xe0\xe0\x8b\x0c\xd9\x1d'
    SESSION_PROTECTION = 'strong'
    SQLALCHEMY_ECHO = False
    SQLALCHEMY_TRACK_MODIFICATIONS = True

class IntegrationConfig(Config):
    DEBUG = True
    SQLALCHEMY_DATABASE_URI =''
    SECRET_KEY = '|~G\xde\xa7\x9b\x1aKaZ-\xabk8\x0b\x12\xee)\xe0\xe0\x8b\x0c\xd9\x1d'
    SESSION_PROTECTION = 'strong'
    SQLALCHEMY_ECHO = False
    SQLALCHEMY_TRACK_MODIFICATIONS = True