{% extends "base.html" %}
{% block title %}Page Not Found{% endblock %}

{% block body %}

<h2> Ooops There has been a mistake! </h2>

<img src="{{ url_for('static', filename='images/PY_logo_trans.png') }}">

  <h1>Page Not Found</h1>
  <p>What you were looking for is just not there.

{% endblock %}