{% extends "base.html" %}

{% block body%}

<div>

    <label id="user-name">Kevin</label>

 {{form.first_name}}
    <label>Database Location:</label>
    {{form.database_location}}
</div>



{% endblock %}