{% extends "base.html" %}

{% block body%}
<div class="container-fluid">
    <h2>Dash Lexicon:</h2>
    <p><strong class="dash-notes"> Below are a list of the keywords that can be used to prompt Dash to
        provide a particular response .</strong></p><br/>
    <div class="row">
        <div class="col-md-11 ">
            <table class="table keyword-table">
                <tr>
                    <th>Keyword</th>
                    <th class="">Description</th>
                </tr>

                <tr>
                    <td>Shortage</td>
                    <td>The difference between the quantity on hand and the reorder level.</td>
                </tr>
                <tr>
                    <td>Excess</td>
                    <td>The quantity of stock above the reorder level and lead-time demand at cost</td>
                </tr>
                <tr>
                    <td>Revenue</td>
                    <td>The income generated by the sale of total stock sold over historical period in the analysis.
                    </td>
                </tr>
                <tr>
                    <td>Average Orders</td>
                    <td>The average quantity of orders placed for a SKU over the total period of the.</td>
                </tr>
                <tr>
                    <td>Inventory Turns</td>
                    <td>The number of times the current quantity on hand would be consumed based on historic demand.</td>
                </tr>
            </table>

        </div>
        <div class="row">
            <div class="col-md-11 dash-examples">
                <table class="table ">
                    <tr>
                        <th>Example</th>
                        <th>Result</th>
                    </tr>

                    <tr>
                        <td>"Show {SKU}", "Describe {SKU}", "Explain {SKU}" ...</td>
                        <td>Replacing {SKU} with a SKU ID from your inventory profile, returns a hyperlink to the
                            analysis of that SKU.
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <ul>
                                <li>"What is the {largest, smallest, highest, lowest...} {KEYWORD}?"
                                </li>
                                <li>"Which SKU has the {largest, smallest, highest, lowest...} {KEYWORD}?"
                                </li>
                            </ul>
                        </td>
                        <td>Replacing the braces with an appropriate adjective and keyword will return the SKU and value
                            with
                            the corresponding description.
                        </td>
                    </tr>
                    <tr>
                        <td>"What is the {KEYWORD} for {SKU}?</td>
                        <td>Replacing braces with the appropriate keyword and SKU will return the values for the
                            corresponding description.
                        </td>
                    </tr>
                </table>

            </div>
        </div>
    </div>
    {% endblock %}
