@import url(https://fonts.googleapis.com/css?family=Titillium+Web:400,600,700|Roboto+Slab);

a, abbr, acronym, address, applet, article, aside, audio, b, big, blockquote, body, canvas, caption, center, cite, code, dd, del, details, dfn, div, dl, dt, em, embed, fieldset, figcaption, figure, footer, form, h1, h2, h3, h4, h5, h6, header, hgroup, html, i, iframe, img, ins, kbd, label, legend, li, mark, menu, nav, object, ol, output, p, pre, q, ruby, s, samp, section, small, span, strike, strong, sub, summary, sup, table, tbody, td, tfoot, th, thead, time, tr, tt, u, ul, var, video {
    margin: 0;
    padding: 0;
    border: 0;
    font: inherit;
    vertical-align: baseline
}

article, aside, details, figcaption, figure, footer, header, hgroup, menu, nav, section {
    display: block
}

body {
    line-height: 1
}

ol, ul {
    list-style: none
}

blockquote, q {
    quotes: none
}

blockquote:after, blockquote:before, q:after, q:before {
    content: '';
    content: none
}

table {
    border-collapse: collapse;
    border-spacing: 0
}

/* Solarized Palette - http://ethanschoonover.com/solarized ---------
  lightgray      : #819090;
  gray           : #708284;
  mediumgray     : #536870;
  darkgray       : #475B62;
  darkblue       : #0A2933;
  darkerblue     : #042029;
  paleryellow    : #FCF4DC;
  paleyellow     : #EAE3CB;
  yellow         : #A57706;
  orange         : #BD3613;
  red            : #D11C29;
  pink           : #C61C6F;
  purple         : #595AB7;
  blue           : #2176C7;
  cyan           : #259286;
  green          : #0A2933; */

body {
    background-color: #0A2933;

    color: #708284;
    font-family: "Roboto Slab";
    font-size: 1em;
}

.content {
    padding-top: 40px;
    padding-left: 40px;
    padding-right: 40px;
    height: 2000px;
    background-color: #0A2933;
    transition: transform3d 0.8s ease;

}

.active-nav .content {
    transform: translate3d(100px, 0, 0);

}

.card {
    padding-left: 20px;
    margin-left: -40px;
    margin-top: 10px;
    margin-bottom: 30px;
    min-width: 100pc;
    float: left;
    color: #042029;
    opacity: .8;
}

table, th, td {
    color: white;
    border: 1px solid black;
    border-collapse: collapse;
    border-spacing: 5px;
}

th, td {
    padding: 1px;
    font-family: "Roboto Slab", "ff-tisa-web-pro", "Georgia", Arial, sans-serif;
    text-align: center;
}

th {
    color: #259286;
    font-size: 1.4em;
}

.ia {
    margin-left: 20px;
    margin-top: 20px;
    margin-bottom: 20px;
    float: left;
    box-shadow: 10px 5px 5px black;
}

table.ia tr:nth-child(even) {
    background-color: #042029;

}

table.ia tr:nth-child(odd) {
    background-color: #0A2933;;
}

table.ia th {

    background-color: transparent;
    font-family: "Roboto Slab";
    font-weight: 600;
}

h1, h2, h3, h4, h5, h6 {
    font-family: 'Roboto Slab', "Helvetica Neue", Helvetica, sans-serif;
    font-weight: 700;

    color: #EAE3CB;
    margin-bottom: .4em;
}

h2 {
    font-size: 5em;
    font-family: 'Roboto Slab';

}

.highlight {
    color: #D11C24;
    font-weight: 600;
}

.left {
    position: fixed;
    padding: 10px;
    top: 0;
    left: 0;
    height: 100%;
    width: 150px;
    transition: width ease .8s;
    margin-right: 300px;

}

body {
    overflow-x: hidden;
}

nav {
    position: fixed;
    z-index: 1000;
    top: 0;
    bottom: 0;
    width: 100px;
    background-color: #042029;
    transform: translate3d(-100px, 0, 0);
    transition: transform 0.2s ease;

}

nav ul {
    list-style: none;
    margin-top: 100px;

}

nav ul li a {
    text-decoration: none;
    display: block;
    text-align: center;
    color: white;
    padding: 20px 0;
    font-size: 1.5em;
    margin-bottom: 20px;

}

.nav-toggle-btn {
    display: block;
    position: absolute;
    left: 100px;
    width: 170px;
    height: 35px;
    background: #042029;
}

* {
    margin: 0;
    padding: 0;
}

.active-nav nav {
    transform: translate3d(0, 0, 0);

}

#pie-shortage, #excess-pie {
    padding-top: 20px;
    padding-left: 76px;
    width: 500px;

}

#shortage-chart {
    height: auto;
    padding-left: 20px;

}

.notes {
    padding-left: 60px;
}

.slate {
    margin-top: 30px;
    margin-bottom: 30px;
    margin-right: 10px;
    padding-top: 10px;
    padding-bottom: 10px;
    background-color: #092029;
    max-width: 300px;
    height: 150px;
    font-size: 1.3em;
    text-align: center;
    margin-left: 20px;
    box-shadow: 10px 5px 5px black;
}

#shortage-table, #excess {

    width: auto;
}

#short-table {
    margin: 30px;

}

.pad-chart-col {
    padding-left: 110px
}

.traffic-light {
    width: 20px;
    height: 20px;
    border-radius: 10px;
    float: left;
}

.green {
    background-color: lawngreen;
}

.amber {
    background-color: #BD3613;;
}

.red {
    background-color: red;
}

.white {
    background-color: white;
}

.highlight-event {
    background-color: #0A2933;

}

.sub-row {
    margin-right: 15px;
    padding-left: 15px;

}

#analysis_table {

}

.dataTables_wrapper .dataTables_paginate {
    color: white;
}

.strip {
    background-color: #708284
}

.pebble2 {
    margin-top: 30px;
    margin-bottom: 30px;
    margin-right: 10px;
    padding-top: 10px;
    padding-bottom: 10px;
    background-color: #092029;

    font-size: 1.3em;
    text-align: center;
    margin-left: 20px;
    box-shadow: 10px 5px 5px black;
}

.scrollable-menu {
    height: auto;
    max-height: 130px;
    overflow-x: hidden;
}

.scribble-panel, .recommendation-panel {
    background-color: #475B62;
    min-width: 500px;
    min-height: 200px;
    margin-top: 20px;
    margin-left: 20px;
    margin-right: 50px;
    font-size: 1em;
    box-shadow: 10px 5px 10px black;
    padding: 10px;
    padding-left: 60px;
    color: white;

}

.scribble-panel {
    min-width: 600px;
}

#response-panel {
    background-color: #708284;
    min-width: 450px;
    height: 250px;
    padding: 20px;
    overflow: auto;
    margin-top: 20px;
    box-shadow: 10px 5px 10px black;
}

.scribble-card, .scribble-card > h3 {
    min-width: 50%;
    background-color: white;
    min-height: 300px;
    color: #708284;
    margin-bottom: 0;
    margin-right: 0;
    padding-right: 0;
}

.scribble-card-alt, .scribble-card-alt > h3 {
    min-width: 50%;
    background-color: #708284;
    min-height: 300px;
    color: white;
    margin-bottom: 0;
    margin-right: 0;
    padding-right: 0;

}

#total-shortage {
    font-size: 2.0em;
}

.slate-text-lg {
    font-size: 1.5em;
}

.slate-text-sm {
    font-size: 0.6em;
}

.currency-code {
    margin-left: 30px;
}

#demand-chart {
    width: auto;
    height: 400px;
    box-shadow: 10px 5px 10px black;
    background-color: #042029;
}

#oldchap {
    float: right;
    width: 40px;
    height: 40px;
    border-radius: 20px;
    box-shadow: 2px 2px 2px black;
    background-color: #0A2933;

}

#inventory-summary-table th {
    text-align: left;
}

.slate-object {
    width: 100%;
    height: 400px;
    box-shadow: 10px 5px 10px black;
    background-color: #042029;
}

.success {
    color: #00A000;
}

.fail {
    color: red;
}

.banner {
    background-color: black;
    border-left-style: groove;
}

#forecast-chart, #ses-chart {
    width: auto;
    height: 400px;
    box-shadow: 10px 5px 10px black;
    background-color: #042029;
}

#chat-btn {
    float: right;
}

#chat-label {
    margin-top: 30px;
}

#chat-input {
    color: white;
}

#quick-guide {
    margin-top: 40px;
    color: white;
    font-size: 1.3em;
    line-height: 1.3em;
}

.dash-examples {
    margin-top: 40px;
    color: black;
    font-size: 1.3em;
}

.dash-notes {
    color: white;
    font-size: 1.3em;
    line-height: 1.3em;
}

#nav-list-settings {
    margin-top: 5px;
}

.keyword-table > thead > tr > th,
.keyword-table > tbody > tr > th,
.keyword-table > tfoot > tr > th,
.keyword-table > thead > tr > td,
.keyword-table > tbody > tr > td,
.keyword-table > tfoot > tr > td {
    text-align: left;
}

#sku-icon, #warehouse-icon, #search-btn, #clear-btn {
    float: right;
}

#clear-btn, #search-btn {
    margin-top: 10px;
    margin-left: 10px;

}

.recommendation-panel {
    overflow-x: hidden;
}

.rec-panel {
    max-height: 240px;
    background-color: #092029;
}

.recommendation-panel {
    padding: 25px;
    font-size: 1.3em;
    line-height: 1.3em;

}

#rec-btns {
    margin-right: 80px;
}

.contact {
    padding: 20px;
    height: 150px;
    width: 300px;
    margin-left: 20px;
    margin-top: 20px;
    float: left;
    box-shadow: 10px 5px 5px black;
}

.forecast-btn {
    margin: 20px;
}

.slate-values {
    color: white;
}

.slate-hint {
    color: #819090;
}

#sim-summary {
    font-size: 0.8em;
}


#loader {
  position: absolute;
  left: 50%;
  top: 50%;
  z-index: 1;
  width: 150px;
  height: 150px;
  margin: -75px 0 0 -75px;
  border: 16px solid #f3f3f3;
  border-radius: 50%;
  border-top: 16px solid #819090;
  width: 120px;
  height: 120px;
  -webkit-animation: spin 2s linear infinite;
  animation: spin 2s linear infinite;
}
@keyframes spin {
    0% {
        transform: rotate(0deg);
    }
    100% {
        transform: rotate(360deg);
    }
}

.summary-titles{

    color: white;
}