/* Generated by Cython 0.25.2 */

/* BEGIN: Cython Metadata
{
    "distutils": {},
    "module_name": "supplychainpy.demand._evo_algo"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_25_2"
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x03030000 || (PY_MAJOR_VERSION == 2 && PY_VERSION_HEX >= 0x02070000)
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_FASTCALL
  #define METH_FASTCALL 0x80
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args,
                                              Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__supplychainpy__demand___evo_algo
#define __PYX_HAVE_API__supplychainpy__demand___evo_algo
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "supplychainpy/demand/_evo_algo.pyx",
};

/*--- Type declarations ---*/
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome;
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome;

/* "supplychainpy/demand/_evo_algo.pyx":64
 *         #print("initialising {}".format(id(self)))
 * 
 *     def reproduce(self, recombination_type = 'single_point'):             # <<<<<<<<<<<<<<
 *         """ Coordinates the reproduction of two individuals, using one of three recombination methods 'single_point,
 *         two_point or uniform'.
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce {
  PyObject_HEAD
  PyObject *__pyx_v_i;
  PyObject *__pyx_v_recombination_type;
  PyObject *__pyx_v_self;
};


/* "supplychainpy/demand/_evo_algo.pyx":85
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 *     def _single_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on one point along the chromosome.
 * 
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination {
  PyObject_HEAD
  int __pyx_v_genome_count;
  PyObject *__pyx_v_index;
  PyObject *__pyx_v_individual;
  PyObject *__pyx_v_key;
  int __pyx_v_mutation_count;
  int __pyx_v_mutation_index;
  PyObject *__pyx_v_new_individual;
  PyObject *__pyx_v_new_individual_two;
  PyObject *__pyx_v_number_of_mutations_allowed;
  PyObject *__pyx_v_population_allele_count;
  PyObject *__pyx_v_self;
  PyObject *__pyx_v_value;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
  PyObject *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
};


/* "supplychainpy/demand/_evo_algo.pyx":152
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')
 * 
 *     def _two_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on two points along the chromosome.
 * 
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination {
  PyObject_HEAD
  PyObject *__pyx_v_e;
  int __pyx_v_genome_count;
  PyObject *__pyx_v_index;
  PyObject *__pyx_v_individual;
  PyObject *__pyx_v_key;
  int __pyx_v_mutation_count;
  int __pyx_v_mutation_index;
  PyObject *__pyx_v_new_individual;
  PyObject *__pyx_v_new_individual_two;
  PyObject *__pyx_v_number_of_mutations_allowed;
  PyObject *__pyx_v_population_allele_count;
  PyObject *__pyx_v_self;
  PyObject *__pyx_v_value;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
  PyObject *__pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
};


/* "supplychainpy/demand/_evo_algo.pyx":221
 * 
 *     @staticmethod
 *     def _mutation(gene: dict):             # <<<<<<<<<<<<<<
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 *         by having the bit representing it flipped.
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation {
  PyObject_HEAD
  PyObject *__pyx_v_original_gene;
};


/* "supplychainpy/demand/_evo_algo.pyx":350
 * 
 *     @staticmethod
 *     def create_individuals(list new_population):             # <<<<<<<<<<<<<<
 *         """Create individuals using class from genomes striped during processing fitness.
 *         Args:
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals {
  PyObject_HEAD
  PyObject *__pyx_v_genome;
  PyObject *__pyx_v_new_individual;
  PyObject *__pyx_v_new_population;
  PyObject *__pyx_v_parent_offspring_population;
  PyObject *__pyx_v_pke;
  PyObject *__pyx_v_po;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
};


/* "supplychainpy/demand/_evo_algo.pyx":367
 * 
 *     @staticmethod
 *     def _population_fitness(list population, individual_type='ses'):             # <<<<<<<<<<<<<<
 *         """ Assess the population for fitness before crossover and creating next generation. Positive traits
 *         should be reflected by more than 70% of the genes in the genome.
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness {
  PyObject_HEAD
  PyObject *__pyx_v_individual;
  PyObject *__pyx_v_individual_type;
  PyObject *__pyx_v_population;
  double __pyx_v_procreation_probability;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
};


/* "supplychainpy/demand/_evo_algo.pyx":389
 *                     yield individual
 * 
 *     def generate_smoothing_level_genome(self, list population, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome {
  PyObject_HEAD
  PyObject *__pyx_v_individual_type;
  PyObject *__pyx_v_individuals_genome;
  PyObject *__pyx_v_parent;
  PyObject *__pyx_v_population;
  PyObject *__pyx_v_self;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
};


/* "supplychainpy/demand/_evo_algo.pyx":445
 *         return appraised_individual
 * 
 *     def express_smoothing_level_genome(self, individuals_genome: list, standard_error):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */
struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome {
  PyObject_HEAD
  PyObject *__pyx_v_genome;
  PyObject *__pyx_v_individuals_genome;
  PyObject *__pyx_v_individuals_traits;
  PyObject *__pyx_v_self;
  PyObject *__pyx_v_standard_error;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ArgTypeTest.proto */
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_GET();
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyObject *);
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static int __Pyx_Coroutine_clear(PyObject *self);
#if 1 || PY_VERSION_HEX < 0x030300B0
static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue);
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue) PyGen_FetchStopIterationValue(pvalue)
#endif

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'supplychainpy.demand._evo_algo' */
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome = 0;
static PyTypeObject *__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome = 0;
#define __Pyx_MODULE_NAME "supplychainpy.demand._evo_algo"
int __pyx_module_is_main_supplychainpy__demand___evo_algo = 0;

/* Implementation of 'supplychainpy.demand._evo_algo' */
static PyObject *__pyx_builtin_property;
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_OSError;
static PyObject *__pyx_builtin_min;
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_sum;
static const char __pyx_k_[] = "{}: {}";
static const char __pyx_k_e[] = "e";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_p[] = "p";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_v[] = "v";
static const char __pyx_k_po[] = "po";
static const char __pyx_k_ape[] = "ape";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_min[] = "min";
static const char __pyx_k_pke[] = "pke";
static const char __pyx_k_ses[] = "ses";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_val[] = "val";
static const char __pyx_k_zip[] = "zip";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_gene[] = "gene";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mape[] = "mape";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_repr[] = "__repr__";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_alpha[] = "alpha";
static const char __pyx_k_clear[] = "clear";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_gamma[] = "gamma";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_order[] = "order";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_slope[] = "slope";
static const char __pyx_k_slots[] = "__slots__";
static const char __pyx_k_stats[] = "stats";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_demand[] = "demand";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_genome[] = "_genome";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_mutate[] = "mutate";
static const char __pyx_k_name_2[] = "_name";
static const char __pyx_k_orders[] = "orders";
static const char __pyx_k_parent[] = "parent";
static const char __pyx_k_random[] = "random";
static const char __pyx_k_setter[] = "setter";
static const char __pyx_k_sm_lvl[] = "sm_lvl";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_OSError[] = "OSError";
static const char __pyx_k_evo_mod[] = "evo_mod";
static const char __pyx_k_overide[] = "overide";
static const char __pyx_k_parents[] = "parents";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_uniform[] = "uniform";
static const char __pyx_k_Forecast[] = "Forecast";
static const char __pyx_k_forecast[] = "forecast";
static const char __pyx_k_genome_2[] = "genome";
static const char __pyx_k_mutation[] = "_mutation";
static const char __pyx_k_property[] = "property";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_htce_para[] = "htce_para";
static const char __pyx_k_intercept[] = "intercept";
static const char __pyx_k_log_stats[] = "log_stats";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_offspring[] = "offspring";
static const char __pyx_k_reproduce[] = "reproduce";
static const char __pyx_k_slice_end[] = "slice_end";
static const char __pyx_k_two_point[] = "two_point";
static const char __pyx_k_Individual[] = "Individual";
static const char __pyx_k_Population[] = "Population";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_avg_orders[] = "avg_orders";
static const char __pyx_k_gene_count[] = "gene_count";
static const char __pyx_k_individual[] = "individual";
static const char __pyx_k_population[] = "population";
static const char __pyx_k_regression[] = "regression";
static const char __pyx_k_statistics[] = "statistics";
static const char __pyx_k_sum_orders[] = "sum_orders";
static const char __pyx_k_final_error[] = "final_error";
static const char __pyx_k_individuals[] = "individuals";
static const char __pyx_k_gene_count_2[] = "_gene_count";
static const char __pyx_k_genome_count[] = "genome_count";
static const char __pyx_k_ses_forecast[] = "ses_forecast";
static const char __pyx_k_single_point[] = "single_point";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_average_order[] = "average_order";
static const char __pyx_k_express_trait[] = "_express_trait";
static const char __pyx_k_forecast_type[] = "forecast_type";
static const char __pyx_k_optimal_alpha[] = "optimal_alpha";
static const char __pyx_k_original_gene[] = "original_gene";
static const char __pyx_k_squared_error[] = "squared_error";
static const char __pyx_k_fit_population[] = "fit_population";
static const char __pyx_k_mutation_count[] = "mutation_count";
static const char __pyx_k_mutation_index[] = "mutation_index";
static const char __pyx_k_new_individual[] = "new_individual";
static const char __pyx_k_new_population[] = "new_population";
static const char __pyx_k_standard_error[] = "standard_error";
static const char __pyx_k_Individual_name[] = "Individual.name";
static const char __pyx_k_forecast_demand[] = "forecast_demand";
static const char __pyx_k_forecast_length[] = "forecast_length";
static const char __pyx_k_forecast_type_2[] = "_forecast_type";
static const char __pyx_k_individual_type[] = "individual_type";
static const char __pyx_k_new_individuals[] = "new_individuals";
static const char __pyx_k_population_size[] = "population_size";
static const char __pyx_k_simple_forecast[] = "simple_forecast";
static const char __pyx_k_smoothing_level[] = "smoothing_level";
static const char __pyx_k_LinearRegression[] = "LinearRegression";
static const char __pyx_k_create_offspring[] = "create_offspring";
static const char __pyx_k_genome_generator[] = "__genome_generator";
static const char __pyx_k_Individual___init[] = "Individual.__init__";
static const char __pyx_k_Individual___repr[] = "Individual.__repr__";
static const char __pyx_k_Individual_genome[] = "Individual.genome";
static const char __pyx_k_Population___init[] = "Population.__init__";
static const char __pyx_k_sum_squared_error[] = "sum_squared_error";
static const char __pyx_k_create_individuals[] = "create_individuals";
static const char __pyx_k_forecast_breakdown[] = "forecast_breakdown";
static const char __pyx_k_individuals_genome[] = "individuals_genome";
static const char __pyx_k_individuals_traits[] = "individuals_traits";
static const char __pyx_k_initial_population[] = "initial_population";
static const char __pyx_k_new_fit_population[] = "new_fit_population";
static const char __pyx_k_new_individual_two[] = "new_individual_two";
static const char __pyx_k_parents_population[] = "parents_population";
static const char __pyx_k_population_fitness[] = "_population_fitness";
static const char __pyx_k_populations_genome[] = "populations_genome";
static const char __pyx_k_populations_traits[] = "populations_traits";
static const char __pyx_k_recombination_type[] = "recombination_type";
static const char __pyx_k_sum_squared_errors[] = "sum_squared_errors";
static const char __pyx_k_individuals_fitness[] = "individuals_fitness";
static const char __pyx_k_least_squared_error[] = "least_squared_error";
static const char __pyx_k_Population__mutation[] = "Population._mutation";
static const char __pyx_k_Population_reproduce[] = "Population.reproduce";
static const char __pyx_k_appraised_individual[] = "appraised_individual";
static const char __pyx_k_mutation_probability[] = "mutation_probability";
static const char __pyx_k_optimal_ses_forecast[] = "optimal_ses_forecast";
static const char __pyx_k_recombination_type_2[] = "_recombination_type";
static const char __pyx_k_selection_population[] = "_selection_population";
static const char __pyx_k_Individual_gene_count[] = "Individual.gene_count";
static const char __pyx_k_new_population_genome[] = "new_population_genome";
static const char __pyx_k_simple_expo_smoothing[] = "simple_expo_smoothing";
static const char __pyx_k_new_populations_traits[] = "new_populations_traits";
static const char __pyx_k_sum_squared_error_para[] = "sum_squared_error_para";
static const char __pyx_k_initial_estimate_period[] = "initial_estimate_period";
static const char __pyx_k_minimum_smoothing_level[] = "minimum_smoothing_level";
static const char __pyx_k_original_standard_error[] = "original_standard_error";
static const char __pyx_k_population_allele_count[] = "population_allele_count";
static const char __pyx_k_procreation_probability[] = "procreation_probability";
static const char __pyx_k_sum_squared_errors_indi[] = "sum_squared_errors_indi";
static const char __pyx_k_smoothing_level_constant[] = "smoothing_level_constant";
static const char __pyx_k_number_of_mutations_allowed[] = "number_of_mutations_allowed";
static const char __pyx_k_parent_offspring_population[] = "parent_offspring_population";
static const char __pyx_k_Individual__genome_generator[] = "_Individual__genome_generator";
static const char __pyx_k_simple_exponential_smoothing[] = "simple_exponential_smoothing";
static const char __pyx_k_Individual___genome_generator[] = "Individual.__genome_generator";
static const char __pyx_k_sum_squared_errors_indi_htces[] = "sum_squared_errors_indi_htces";
static const char __pyx_k_express_smoothing_level_genome[] = "express_smoothing_level_genome";
static const char __pyx_k_supplychainpy_demand__evo_algo[] = "supplychainpy.demand._evo_algo";
static const char __pyx_k_Create_a_population_of_individu[] = " Create a population of individuals to reproduce. ";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAl[] = "_OptimiseSmoothingLevelGeneticAlgorithm__orders";
static const char __pyx_k_Population__two_point_crossover[] = "Population._two_point_crossover_recombination";
static const char __pyx_k_generate_smoothing_level_genome[] = "generate_smoothing_level_genome";
static const char __pyx_k_holts_trend_corrected_smoothing[] = "holts_trend_corrected_smoothing";
static const char __pyx_k_home_kevin_Projects_Python_supp[] = "/home/kevin/Projects/Python/supplychainpy/supplychainpy/demand/_evo_algo.pyx";
static const char __pyx_k_initialise_smoothing_level_evol[] = "_initialise_smoothing_level_evolutionary_algorithm_population";
static const char __pyx_k_run_exponential_smoothing_forec[] = "_run_exponential_smoothing_forecast";
static const char __pyx_k_run_holts_trend_corrected_expon[] = "_run_holts_trend_corrected_exponential_smoothing";
static const char __pyx_k_single_point_crossover_recombin[] = "_single_point_crossover_recombination";
static const char __pyx_k_supplychainpy_demand_regression[] = "supplychainpy.demand.regression";
static const char __pyx_k_two_point_crossover_recombinati[] = "_two_point_crossover_recombination";
static const char __pyx_k_uniform_crossover_recombination[] = "_uniform_crossover_recombination";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg[] = "OptimiseSmoothingLevelGeneticAlgorithm.create_individuals";
static const char __pyx_k_Population_Size_is_too_small_for[] = "Population Size is too small for effective genetic recombination and reproduction of offspring";
static const char __pyx_k_Population__mutation_locals_muta[] = "Population._mutation.<locals>.mutate";
static const char __pyx_k_Population__single_point_crossov[] = "Population._single_point_crossover_recombination";
static const char __pyx_k_Population__uniform_crossover_re[] = "Population._uniform_crossover_recombination";
static const char __pyx_k_holts_trend_corrected_exponentia[] = "holts_trend_corrected_exponential_smoothing";
static const char __pyx_k_mean_aboslute_percentage_error_o[] = "mean_aboslute_percentage_error_opt";
static const char __pyx_k_simple_exponential_smoothing_evo[] = "simple_exponential_smoothing_evo";
static const char __pyx_k_simple_exponential_smoothing_for[] = "simple_exponential_smoothing_forecast";
static const char __pyx_k_supplychainpy_demand__forecast_d[] = "supplychainpy.demand._forecast_demand";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAl_2[] = "_OptimiseSmoothingLevelGeneticAlgorithm__average_order";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAl_3[] = "_OptimiseSmoothingLevelGeneticAlgorithm__population_size";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAl_4[] = "_OptimiseSmoothingLevelGeneticAlgorithm__standard_error";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAl_5[] = "_OptimiseSmoothingLevelGeneticAlgorithm__recombination_type";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAl_6[] = "_OptimiseSmoothingLevelGeneticAlgorithm__generation";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_2[] = "OptimiseSmoothingLevelGeneticAlgorithm._population_fitness";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_3[] = "OptimiseSmoothingLevelGeneticAlgorithm.generate_smoothing_level_genome";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_4[] = "OptimiseSmoothingLevelGeneticAlgorithm.express_smoothing_level_genome";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_5[] = "OptimiseSmoothingLevelGeneticAlgorithm";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_6[] = "OptimiseSmoothingLevelGeneticAlgorithm.__init__";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_7[] = "OptimiseSmoothingLevelGeneticAlgorithm.initial_population";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_8[] = "OptimiseSmoothingLevelGeneticAlgorithm.population_size";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_9[] = "OptimiseSmoothingLevelGeneticAlgorithm._initialise_smoothing_level_evolutionary_algorithm_population";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_10[] = "OptimiseSmoothingLevelGeneticAlgorithm.htce_para";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_11[] = "OptimiseSmoothingLevelGeneticAlgorithm.sum_squared_error_para";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_12[] = "OptimiseSmoothingLevelGeneticAlgorithm._run_holts_trend_corrected_exponential_smoothing";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_13[] = "OptimiseSmoothingLevelGeneticAlgorithm._run_exponential_smoothing_forecast";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_14[] = "OptimiseSmoothingLevelGeneticAlgorithm._express_trait";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_15[] = "OptimiseSmoothingLevelGeneticAlgorithm._selection_population";
static const char __pyx_k_OptimiseSmoothingLevelGeneticAlg_16[] = "OptimiseSmoothingLevelGeneticAlgorithm.simple_exponential_smoothing_evo";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_Create_a_population_of_individu;
static PyObject *__pyx_n_s_Forecast;
static PyObject *__pyx_n_s_Individual;
static PyObject *__pyx_n_s_Individual___genome_generator;
static PyObject *__pyx_n_s_Individual___init;
static PyObject *__pyx_n_s_Individual___repr;
static PyObject *__pyx_n_s_Individual__genome_generator;
static PyObject *__pyx_n_s_Individual_gene_count;
static PyObject *__pyx_n_s_Individual_genome;
static PyObject *__pyx_n_s_Individual_name;
static PyObject *__pyx_n_s_LinearRegression;
static PyObject *__pyx_n_s_OSError;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAl;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAl_2;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAl_3;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAl_4;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAl_5;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAl_6;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_10;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_11;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_12;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_13;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_14;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_15;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_16;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_2;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_3;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_4;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_6;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_7;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_8;
static PyObject *__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_9;
static PyObject *__pyx_n_s_Population;
static PyObject *__pyx_kp_s_Population_Size_is_too_small_for;
static PyObject *__pyx_n_s_Population___init;
static PyObject *__pyx_n_s_Population__mutation;
static PyObject *__pyx_n_s_Population__mutation_locals_muta;
static PyObject *__pyx_n_s_Population__single_point_crossov;
static PyObject *__pyx_n_s_Population__two_point_crossover;
static PyObject *__pyx_n_s_Population__uniform_crossover_re;
static PyObject *__pyx_n_s_Population_reproduce;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_alpha;
static PyObject *__pyx_n_s_ape;
static PyObject *__pyx_n_s_appraised_individual;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_average_order;
static PyObject *__pyx_n_s_avg_orders;
static PyObject *__pyx_n_s_clear;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_create_individuals;
static PyObject *__pyx_n_s_create_offspring;
static PyObject *__pyx_n_s_demand;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_e;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_evo_mod;
static PyObject *__pyx_n_s_express_smoothing_level_genome;
static PyObject *__pyx_n_s_express_trait;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_final_error;
static PyObject *__pyx_n_s_fit_population;
static PyObject *__pyx_n_s_forecast;
static PyObject *__pyx_n_s_forecast_breakdown;
static PyObject *__pyx_n_s_forecast_demand;
static PyObject *__pyx_n_s_forecast_length;
static PyObject *__pyx_n_s_forecast_type;
static PyObject *__pyx_n_s_forecast_type_2;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_gamma;
static PyObject *__pyx_n_s_gene;
static PyObject *__pyx_n_s_gene_count;
static PyObject *__pyx_n_s_gene_count_2;
static PyObject *__pyx_n_s_generate_smoothing_level_genome;
static PyObject *__pyx_n_s_genome;
static PyObject *__pyx_n_s_genome_2;
static PyObject *__pyx_n_s_genome_count;
static PyObject *__pyx_n_s_genome_generator;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_holts_trend_corrected_exponentia;
static PyObject *__pyx_n_s_holts_trend_corrected_smoothing;
static PyObject *__pyx_kp_s_home_kevin_Projects_Python_supp;
static PyObject *__pyx_n_s_htce_para;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_individual;
static PyObject *__pyx_n_s_individual_type;
static PyObject *__pyx_n_s_individuals;
static PyObject *__pyx_n_s_individuals_fitness;
static PyObject *__pyx_n_s_individuals_genome;
static PyObject *__pyx_n_s_individuals_traits;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_initial_estimate_period;
static PyObject *__pyx_n_s_initial_population;
static PyObject *__pyx_n_s_initialise_smoothing_level_evol;
static PyObject *__pyx_n_s_intercept;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_keys;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_least_squared_error;
static PyObject *__pyx_n_s_log_stats;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mape;
static PyObject *__pyx_n_s_mean_aboslute_percentage_error_o;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_min;
static PyObject *__pyx_n_s_minimum_smoothing_level;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_mutate;
static PyObject *__pyx_n_s_mutation;
static PyObject *__pyx_n_s_mutation_count;
static PyObject *__pyx_n_s_mutation_index;
static PyObject *__pyx_n_s_mutation_probability;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new_fit_population;
static PyObject *__pyx_n_s_new_individual;
static PyObject *__pyx_n_s_new_individual_two;
static PyObject *__pyx_n_s_new_individuals;
static PyObject *__pyx_n_s_new_population;
static PyObject *__pyx_n_s_new_population_genome;
static PyObject *__pyx_n_s_new_populations_traits;
static PyObject *__pyx_n_s_number_of_mutations_allowed;
static PyObject *__pyx_n_s_offspring;
static PyObject *__pyx_n_s_optimal_alpha;
static PyObject *__pyx_n_s_optimal_ses_forecast;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_s_orders;
static PyObject *__pyx_n_s_original_gene;
static PyObject *__pyx_n_s_original_standard_error;
static PyObject *__pyx_n_s_overide;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_parent;
static PyObject *__pyx_n_s_parent_offspring_population;
static PyObject *__pyx_n_s_parents;
static PyObject *__pyx_n_s_parents_population;
static PyObject *__pyx_n_s_pke;
static PyObject *__pyx_n_s_po;
static PyObject *__pyx_n_s_population;
static PyObject *__pyx_n_s_population_allele_count;
static PyObject *__pyx_n_s_population_fitness;
static PyObject *__pyx_n_s_population_size;
static PyObject *__pyx_n_s_populations_genome;
static PyObject *__pyx_n_s_populations_traits;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_procreation_probability;
static PyObject *__pyx_n_s_property;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_random;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_recombination_type;
static PyObject *__pyx_n_s_recombination_type_2;
static PyObject *__pyx_n_s_regression;
static PyObject *__pyx_n_s_repr;
static PyObject *__pyx_n_s_reproduce;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_run_exponential_smoothing_forec;
static PyObject *__pyx_n_s_run_holts_trend_corrected_expon;
static PyObject *__pyx_n_s_selection_population;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_ses;
static PyObject *__pyx_n_s_ses_forecast;
static PyObject *__pyx_n_s_setter;
static PyObject *__pyx_n_s_simple_expo_smoothing;
static PyObject *__pyx_n_s_simple_exponential_smoothing;
static PyObject *__pyx_n_s_simple_exponential_smoothing_evo;
static PyObject *__pyx_n_s_simple_exponential_smoothing_for;
static PyObject *__pyx_n_s_simple_forecast;
static PyObject *__pyx_n_s_single_point;
static PyObject *__pyx_n_s_single_point_crossover_recombin;
static PyObject *__pyx_n_s_slice_end;
static PyObject *__pyx_n_s_slope;
static PyObject *__pyx_n_s_slots;
static PyObject *__pyx_n_s_sm_lvl;
static PyObject *__pyx_n_s_smoothing_level;
static PyObject *__pyx_n_s_smoothing_level_constant;
static PyObject *__pyx_n_s_squared_error;
static PyObject *__pyx_n_s_standard_error;
static PyObject *__pyx_n_s_staticmethod;
static PyObject *__pyx_n_s_statistics;
static PyObject *__pyx_n_s_stats;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_sum_orders;
static PyObject *__pyx_n_s_sum_squared_error;
static PyObject *__pyx_n_s_sum_squared_error_para;
static PyObject *__pyx_n_s_sum_squared_errors;
static PyObject *__pyx_n_s_sum_squared_errors_indi;
static PyObject *__pyx_n_s_sum_squared_errors_indi_htces;
static PyObject *__pyx_n_s_supplychainpy_demand__evo_algo;
static PyObject *__pyx_n_s_supplychainpy_demand__forecast_d;
static PyObject *__pyx_n_s_supplychainpy_demand_regression;
static PyObject *__pyx_n_s_t;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_two_point;
static PyObject *__pyx_n_s_two_point_crossover_recombinati;
static PyObject *__pyx_n_s_uniform;
static PyObject *__pyx_n_s_uniform_crossover_recombination;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_n_s_val;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_values;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_overide, int __pyx_v_gene_count, PyObject *__pyx_v_forecast_type); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_2__repr__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_4gene_count(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_6gene_count(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_8genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_10genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_val); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_12name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_14__genome_generator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individuals, double __pyx_v_mutation_probability); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_2reproduce(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_recombination_type); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_5_single_point_crossover_recombination(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_8_two_point_crossover_recombination(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_11_uniform_crossover_recombination(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_mutate(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_13_mutation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_gene); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_orders, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_2initial_population(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual_type); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_4population_size(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_6population_size(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_population_size); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_8_initialise_smoothing_level_evolutionary_algorithm_population(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual_type); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_10create_individuals(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_new_population); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_13_population_fitness(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_population, PyObject *__pyx_v_individual_type); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_16generate_smoothing_level_genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_population, PyObject *__pyx_v_individual_type); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_19htce_para(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_forecast, PyObject *__pyx_v_alpha, PyObject *__pyx_v_gamma, PyObject *__pyx_v_intercept, PyObject *__pyx_v_slope); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_21sum_squared_error_para(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_forecast, PyObject *__pyx_v_holts_trend_corrected_smoothing, PyObject *__pyx_v_alpha, PyObject *__pyx_v_gamma, PyObject *__pyx_v_smoothing_level); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_23_run_holts_trend_corrected_exponential_smoothing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_25express_smoothing_level_genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individuals_genome, PyObject *__pyx_v_standard_error); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_28_run_exponential_smoothing_forecast(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_30_express_trait(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_original_standard_error, PyObject *__pyx_v_appraised_individual); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_32_selection_population(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_individuals_fitness, CYTHON_UNUSED PyObject *__pyx_v_appraised_individual); /* proto */
static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_34simple_exponential_smoothing_evo(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_smoothing_level_constant, PyObject *__pyx_v_initial_estimate_period, PyObject *__pyx_v_recombination_type, int __pyx_v_population_size, int __pyx_v_forecast_length); /* proto */
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_10;
static PyObject *__pyx_int_12;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__29;
static PyObject *__pyx_codeobj__31;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__47;
static PyObject *__pyx_codeobj__49;
static PyObject *__pyx_codeobj__51;
static PyObject *__pyx_codeobj__53;
static PyObject *__pyx_codeobj__55;
static PyObject *__pyx_codeobj__58;
static PyObject *__pyx_codeobj__61;
static PyObject *__pyx_codeobj__63;
static PyObject *__pyx_codeobj__65;
static PyObject *__pyx_codeobj__67;
static PyObject *__pyx_codeobj__69;
static PyObject *__pyx_codeobj__71;
static PyObject *__pyx_codeobj__73;
static PyObject *__pyx_codeobj__75;

/* "supplychainpy/demand/_evo_algo.pyx":9
 *     _genome = 0
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, name ='offspring', overide=False, int gene_count =12,
 *             forecast_type ='ses'
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_1__init__ = {"__init__", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_overide = 0;
  int __pyx_v_gene_count;
  PyObject *__pyx_v_forecast_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_name,&__pyx_n_s_overide,&__pyx_n_s_gene_count,&__pyx_n_s_forecast_type,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_n_s_offspring));

    /* "supplychainpy/demand/_evo_algo.pyx":10
 * 
 *     def __init__(
 *             self, name ='offspring', overide=False, int gene_count =12,             # <<<<<<<<<<<<<<
 *             forecast_type ='ses'
 *         ):
 */
    values[2] = ((PyObject *)((PyObject *)Py_False));
    values[4] = ((PyObject *)((PyObject*)__pyx_n_s_ses));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_overide);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_gene_count);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_forecast_type);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 9, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_name = values[1];
    __pyx_v_overide = values[2];
    if (values[3]) {
      __pyx_v_gene_count = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_gene_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 10, __pyx_L3_error)
    } else {
      __pyx_v_gene_count = ((int)((int)12));
    }
    __pyx_v_forecast_type = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 9, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual___init__(__pyx_self, __pyx_v_self, __pyx_v_name, __pyx_v_overide, __pyx_v_gene_count, __pyx_v_forecast_type);

  /* "supplychainpy/demand/_evo_algo.pyx":9
 *     _genome = 0
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, name ='offspring', overide=False, int gene_count =12,
 *             forecast_type ='ses'
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_overide, int __pyx_v_gene_count, PyObject *__pyx_v_forecast_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":14
 *         ):
 * 
 *         self._name = name             # <<<<<<<<<<<<<<
 *         self._gene_count = gene_count
 *         self._forecast_type = forecast_type
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_name_2, __pyx_v_name) < 0) __PYX_ERR(0, 14, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":15
 * 
 *         self._name = name
 *         self._gene_count = gene_count             # <<<<<<<<<<<<<<
 *         self._forecast_type = forecast_type
 *         if not overide:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_gene_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_gene_count_2, __pyx_t_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":16
 *         self._name = name
 *         self._gene_count = gene_count
 *         self._forecast_type = forecast_type             # <<<<<<<<<<<<<<
 *         if not overide:
 *             self._genome = self.__genome_generator()
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_forecast_type_2, __pyx_v_forecast_type) < 0) __PYX_ERR(0, 16, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":17
 *         self._gene_count = gene_count
 *         self._forecast_type = forecast_type
 *         if not overide:             # <<<<<<<<<<<<<<
 *             self._genome = self.__genome_generator()
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_overide); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 17, __pyx_L1_error)
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":18
 *         self._forecast_type = forecast_type
 *         if not overide:
 *             self._genome = self.__genome_generator()             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Individual__genome_generator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 18, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_genome, __pyx_t_1) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":17
 *         self._gene_count = gene_count
 *         self._forecast_type = forecast_type
 *         if not overide:             # <<<<<<<<<<<<<<
 *             self._genome = self.__genome_generator()
 * 
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":9
 *     _genome = 0
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, name ='offspring', overide=False, int gene_count =12,
 *             forecast_type ='ses'
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":20
 *             self._genome = self.__genome_generator()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '{}: {}'.format(self._name, self._genome)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_3__repr__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_3__repr__ = {"__repr__", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_3__repr__, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_3__repr__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_2__repr__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_2__repr__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":21
 * 
 *     def __repr__(self):
 *         return '{}: {}'.format(self._name, self._genome)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_genome); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":20
 *             self._genome = self.__genome_generator()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '{}: {}'.format(self._name, self._genome)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":24
 * 
 *     @property
 *     def gene_count(self):             # <<<<<<<<<<<<<<
 *         return self.gene_count
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_5gene_count(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_5gene_count = {"gene_count", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_5gene_count, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_5gene_count(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gene_count (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_4gene_count(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_4gene_count(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("gene_count", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":25
 *     @property
 *     def gene_count(self):
 *         return self.gene_count             # <<<<<<<<<<<<<<
 * 
 *     @gene_count.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_gene_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":24
 * 
 *     @property
 *     def gene_count(self):             # <<<<<<<<<<<<<<
 *         return self.gene_count
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.gene_count", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":28
 * 
 *     @gene_count.setter
 *     def gene_count(self, int value):             # <<<<<<<<<<<<<<
 *         self._gene_count = value
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_7gene_count(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_7gene_count = {"gene_count", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_7gene_count, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_7gene_count(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  int __pyx_v_value;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gene_count (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gene_count", 1, 2, 2, 1); __PYX_ERR(0, 28, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gene_count") < 0)) __PYX_ERR(0, 28, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 28, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gene_count", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 28, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.gene_count", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_6gene_count(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_6gene_count(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, int __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("gene_count", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":29
 *     @gene_count.setter
 *     def gene_count(self, int value):
 *         self._gene_count = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_gene_count_2, __pyx_t_1) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":28
 * 
 *     @gene_count.setter
 *     def gene_count(self, int value):             # <<<<<<<<<<<<<<
 *         self._gene_count = value
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.gene_count", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":32
 * 
 *     @property
 *     def genome(self):             # <<<<<<<<<<<<<<
 *         return self._genome
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_9genome(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_9genome = {"genome", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_9genome, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_9genome(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genome (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_8genome(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_8genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("genome", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":33
 *     @property
 *     def genome(self):
 *         return self._genome             # <<<<<<<<<<<<<<
 * 
 *     @genome.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_genome); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":32
 * 
 *     @property
 *     def genome(self):             # <<<<<<<<<<<<<<
 *         return self._genome
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":36
 * 
 *     @genome.setter
 *     def genome(self, tuple val):             # <<<<<<<<<<<<<<
 *         self._genome = val
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_11genome(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_11genome = {"genome", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_11genome, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_11genome(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_val = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genome (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_val,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("genome", 1, 2, 2, 1); __PYX_ERR(0, 36, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "genome") < 0)) __PYX_ERR(0, 36, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_val = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("genome", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 36, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_val), (&PyTuple_Type), 1, "val", 1))) __PYX_ERR(0, 36, __pyx_L1_error)
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_10genome(__pyx_self, __pyx_v_self, __pyx_v_val);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_10genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_val) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genome", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":37
 *     @genome.setter
 *     def genome(self, tuple val):
 *         self._genome = val             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_genome, __pyx_v_val) < 0) __PYX_ERR(0, 37, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":36
 * 
 *     @genome.setter
 *     def genome(self, tuple val):             # <<<<<<<<<<<<<<
 *         self._genome = val
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":40
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_13name(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_13name = {"name", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_13name, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_13name(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("name (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_12name(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_12name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("name", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":41
 *     @property
 *     def name(self):
 *         return self._name             # <<<<<<<<<<<<<<
 * 
 *     def __genome_generator(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":40
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":43
 *         return self._name
 * 
 *     def __genome_generator(self):             # <<<<<<<<<<<<<<
 *         genome = ()
 *         if self._forecast_type == 'ses':
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_15__genome_generator(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_15__genome_generator = {"__genome_generator", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_15__genome_generator, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Individual_15__genome_generator(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__genome_generator (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_14__genome_generator(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Individual_14__genome_generator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_v_genome = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("__genome_generator", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":44
 * 
 *     def __genome_generator(self):
 *         genome = ()             # <<<<<<<<<<<<<<
 *         if self._forecast_type == 'ses':
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])
 */
  __Pyx_INCREF(__pyx_empty_tuple);
  __pyx_v_genome = __pyx_empty_tuple;

  /* "supplychainpy/demand/_evo_algo.pyx":45
 *     def __genome_generator(self):
 *         genome = ()
 *         if self._forecast_type == 'ses':             # <<<<<<<<<<<<<<
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_forecast_type_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_n_s_ses, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "supplychainpy/demand/_evo_algo.pyx":46
 *         genome = ()
 *         if self._forecast_type == 'ses':
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])             # <<<<<<<<<<<<<<
 *         else:
 *             genome = ([(uniform(0, 1), uniform(0, 1)) for i in range(0, self._gene_count)])
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_gene_count_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_int_0);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 46, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 46, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 46, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 46, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 46, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 46, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_6(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 46, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_uniform); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 46, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 46, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 46, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_genome, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":45
 *     def __genome_generator(self):
 *         genome = ()
 *         if self._forecast_type == 'ses':             # <<<<<<<<<<<<<<
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])
 *         else:
 */
    goto __pyx_L3;
  }

  /* "supplychainpy/demand/_evo_algo.pyx":48
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])
 *         else:
 *             genome = ([(uniform(0, 1), uniform(0, 1)) for i in range(0, self._gene_count)])             # <<<<<<<<<<<<<<
 * 
 *         return genome
 */
  /*else*/ {
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_gene_count_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 48, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_int_0);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_7 = __pyx_t_4; __Pyx_INCREF(__pyx_t_7); __pyx_t_5 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 48, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_7))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 48, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_7, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 48, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_7, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_6(__pyx_t_7);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 48, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_uniform); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_uniform); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_8);
      __pyx_t_3 = 0;
      __pyx_t_8 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF_SET(__pyx_v_genome, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "supplychainpy/demand/_evo_algo.pyx":50
 *             genome = ([(uniform(0, 1), uniform(0, 1)) for i in range(0, self._gene_count)])
 * 
 *         return genome             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_genome);
  __pyx_r = __pyx_v_genome;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":43
 *         return self._name
 * 
 *     def __genome_generator(self):             # <<<<<<<<<<<<<<
 *         genome = ()
 *         if self._forecast_type == 'ses':
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Individual.__genome_generator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_genome);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":59
 *     _recombination_type = ('single_point', 'two_point', 'uniform')
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):             # <<<<<<<<<<<<<<
 *         self.individuals = individuals
 *         self.mutation_probability = mutation_probability
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_1__init__ = {"__init__", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_individuals = 0;
  double __pyx_v_mutation_probability;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individuals,&__pyx_n_s_mutation_probability,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individuals)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 59, __pyx_L3_error)
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mutation_probability);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 59, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_individuals = ((PyObject*)values[1]);
    if (values[2]) {
      __pyx_v_mutation_probability = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_mutation_probability == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 59, __pyx_L3_error)
    } else {
      __pyx_v_mutation_probability = ((double)((double)0.2));
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 59, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_individuals), (&PyList_Type), 1, "individuals", 1))) __PYX_ERR(0, 59, __pyx_L1_error)
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population___init__(__pyx_self, __pyx_v_self, __pyx_v_individuals, __pyx_v_mutation_probability);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individuals, double __pyx_v_mutation_probability) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":60
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):
 *         self.individuals = individuals             # <<<<<<<<<<<<<<
 *         self.mutation_probability = mutation_probability
 *         #print("initialising {}".format(id(self)))
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_individuals, __pyx_v_individuals) < 0) __PYX_ERR(0, 60, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":61
 *     def __init__(self, list individuals, double mutation_probability = 0.2):
 *         self.individuals = individuals
 *         self.mutation_probability = mutation_probability             # <<<<<<<<<<<<<<
 *         #print("initialising {}".format(id(self)))
 * 
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_mutation_probability); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_mutation_probability, __pyx_t_1) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":59
 *     _recombination_type = ('single_point', 'two_point', 'uniform')
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):             # <<<<<<<<<<<<<<
 *         self.individuals = individuals
 *         self.mutation_probability = mutation_probability
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_4generator(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":64
 *         #print("initialising {}".format(id(self)))
 * 
 *     def reproduce(self, recombination_type = 'single_point'):             # <<<<<<<<<<<<<<
 *         """ Coordinates the reproduction of two individuals, using one of three recombination methods 'single_point,
 *         two_point or uniform'.
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_3reproduce(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_2reproduce[] = " Coordinates the reproduction of two individuals, using one of three recombination methods 'single_point,\n        two_point or uniform'.\n\n        Args:\n            recombination_type: The method of recombination used to\n\n        Yields:\n            (dict):     The result of recombination and mutation.\n\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_3reproduce = {"reproduce", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_3reproduce, METH_VARARGS|METH_KEYWORDS, __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_2reproduce};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_3reproduce(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_recombination_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reproduce (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_recombination_type,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_n_s_single_point));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_recombination_type);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "reproduce") < 0)) __PYX_ERR(0, 64, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_recombination_type = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("reproduce", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 64, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population.reproduce", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_2reproduce(__pyx_self, __pyx_v_self, __pyx_v_recombination_type);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_2reproduce(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_recombination_type) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reproduce", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 64, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_recombination_type = __pyx_v_recombination_type;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_recombination_type);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_recombination_type);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_4generator, (PyObject *) __pyx_cur_scope, __pyx_n_s_reproduce, __pyx_n_s_Population_reproduce, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 64, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population.reproduce", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_4generator(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    case 2: goto __pyx_L11_resume_from_yield;
    case 3: goto __pyx_L15_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 64, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":77
 * 
 * 
 *         if recombination_type == self._recombination_type[0]:             # <<<<<<<<<<<<<<
 * 
 *             yield [i for i in self._single_point_crossover_recombination()][0]
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_recombination_type_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_recombination_type, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":79
 *         if recombination_type == self._recombination_type[0]:
 * 
 *             yield [i for i in self._single_point_crossover_recombination()][0]             # <<<<<<<<<<<<<<
 *         if recombination_type == self._recombination_type[1]:
 *             yield [i for i in self._two_point_crossover_recombination()][0]
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_single_point_crossover_recombin); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 79, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 79, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 79, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_7(__pyx_t_4);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 79, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_2);
      __pyx_t_2 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_cur_scope->__pyx_v_i))) __PYX_ERR(0, 79, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L7_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 79, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":77
 * 
 * 
 *         if recombination_type == self._recombination_type[0]:             # <<<<<<<<<<<<<<
 * 
 *             yield [i for i in self._single_point_crossover_recombination()][0]
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":80
 * 
 *             yield [i for i in self._single_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[1]:             # <<<<<<<<<<<<<<
 *             yield [i for i in self._two_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[2]:
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_recombination_type_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_4, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_recombination_type, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":81
 *             yield [i for i in self._single_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[1]:
 *             yield [i for i in self._two_point_crossover_recombination()][0]             # <<<<<<<<<<<<<<
 *         if recombination_type == self._recombination_type[2]:
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 */
    __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 81, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_two_point_crossover_recombinati); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 81, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 81, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 81, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_7(__pyx_t_2);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 81, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_4, (PyObject*)__pyx_cur_scope->__pyx_v_i))) __PYX_ERR(0, 81, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 2;
    return __pyx_r;
    __pyx_L11_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 81, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":80
 * 
 *             yield [i for i in self._single_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[1]:             # <<<<<<<<<<<<<<
 *             yield [i for i in self._two_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[2]:
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":82
 *         if recombination_type == self._recombination_type[1]:
 *             yield [i for i in self._two_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[2]:             # <<<<<<<<<<<<<<
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_recombination_type_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_2, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_cur_scope->__pyx_v_recombination_type, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":83
 *             yield [i for i in self._two_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[2]:
 *             yield [i for i in self._uniform_crossover_recombination()][0]             # <<<<<<<<<<<<<<
 * 
 *     def _single_point_crossover_recombination(self):
 */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_uniform_crossover_recombination); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 83, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 83, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 83, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_7(__pyx_t_1);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 83, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_i);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_i, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_cur_scope->__pyx_v_i))) __PYX_ERR(0, 83, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 3;
    return __pyx_r;
    __pyx_L15_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 83, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":82
 *         if recombination_type == self._recombination_type[1]:
 *             yield [i for i in self._two_point_crossover_recombination()][0]
 *         if recombination_type == self._recombination_type[2]:             # <<<<<<<<<<<<<<
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 */
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":64
 *         #print("initialising {}".format(id(self)))
 * 
 *     def reproduce(self, recombination_type = 'single_point'):             # <<<<<<<<<<<<<<
 *         """ Coordinates the reproduction of two individuals, using one of three recombination methods 'single_point,
 *         two_point or uniform'.
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("reproduce", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_7generator1(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":85
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 *     def _single_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on one point along the chromosome.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_6_single_point_crossover_recombination(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_5_single_point_crossover_recombination[] = " Selects genes from the parent to crossover based on one point along the chromosome.\n\n        Yields:\n            individual (dict):          Parents p1-xxxxxxxxxxxx and p2-oooooooooooo will produce offspring c1-xxxxxxooooo\n                                        and c2-ooooooxxxxxx. Individual yields the first offspring (c1).\n            individual_two (dict):      Parents p1-xxxxxxxxxxxxx and p2-oooooooooooo will produce offspring c1-xxxxxxooooo\n                                        and c2-ooooooxxxxxx. Individual_two yields the second offspring (c2).\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_6_single_point_crossover_recombination = {"_single_point_crossover_recombination", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_6_single_point_crossover_recombination, METH_O, __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_5_single_point_crossover_recombination};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_6_single_point_crossover_recombination(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_single_point_crossover_recombination (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_5_single_point_crossover_recombination(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_5_single_point_crossover_recombination(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_single_point_crossover_recombination", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 85, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_7generator1, (PyObject *) __pyx_cur_scope, __pyx_n_s_single_point_crossover_recombin, __pyx_n_s_Population__single_point_crossov, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population._single_point_crossover_recombination", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_7generator1(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *(*__pyx_t_16)(PyObject *);
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_t_19;
  int __pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  int __pyx_t_24;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L30_resume_from_yield;
    case 2: goto __pyx_L31_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 85, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":95
 *         """
 * 
 *         cdef int genome_count = 0             # <<<<<<<<<<<<<<
 *         cdef dict new_individual = {}
 *         cdef dict new_individual_two = {}
 */
  __pyx_cur_scope->__pyx_v_genome_count = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":96
 * 
 *         cdef int genome_count = 0
 *         cdef dict new_individual = {}             # <<<<<<<<<<<<<<
 *         cdef dict new_individual_two = {}
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_new_individual = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":97
 *         cdef int genome_count = 0
 *         cdef dict new_individual = {}
 *         cdef dict new_individual_two = {}             # <<<<<<<<<<<<<<
 * 
 *         cdef int mutation_count = 0
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_new_individual_two = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":99
 *         cdef dict new_individual_two = {}
 * 
 *         cdef int mutation_count = 0             # <<<<<<<<<<<<<<
 *         cdef int mutation_index = 0
 * 
 */
  __pyx_cur_scope->__pyx_v_mutation_count = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":100
 * 
 *         cdef int mutation_count = 0
 *         cdef int mutation_index = 0             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_cur_scope->__pyx_v_mutation_index = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":102
 *         cdef int mutation_index = 0
 * 
 *         try:             # <<<<<<<<<<<<<<
 * 
 *             if len(self.individuals) > 1:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "supplychainpy/demand/_evo_algo.pyx":104
 *         try:
 * 
 *             if len(self.individuals) > 1:             # <<<<<<<<<<<<<<
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_individuals); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 104, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = ((__pyx_t_5 > 1) != 0);
      if (__pyx_t_6) {

        /* "supplychainpy/demand/_evo_algo.pyx":105
 * 
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):             # <<<<<<<<<<<<<<
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 */
        __Pyx_INCREF(__pyx_int_0);
        __pyx_t_1 = __pyx_int_0;
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_individuals); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 105, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
          __pyx_t_8 = __pyx_t_7; __Pyx_INCREF(__pyx_t_8); __pyx_t_5 = 0;
          __pyx_t_9 = NULL;
        } else {
          __pyx_t_5 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 105, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 105, __pyx_L4_error)
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        for (;;) {
          if (likely(!__pyx_t_9)) {
            if (likely(PyList_CheckExact(__pyx_t_8))) {
              if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_8)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_7 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 105, __pyx_L4_error)
              #else
              __pyx_t_7 = PySequence_ITEM(__pyx_t_8, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 105, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_7);
              #endif
            } else {
              if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 105, __pyx_L4_error)
              #else
              __pyx_t_7 = PySequence_ITEM(__pyx_t_8, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 105, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_7);
              #endif
            }
          } else {
            __pyx_t_7 = __pyx_t_9(__pyx_t_8);
            if (unlikely(!__pyx_t_7)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 105, __pyx_L4_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_7);
          }
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_individual);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_individual, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_7);
          __pyx_t_7 = 0;
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_index);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_index, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 105, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_1);
          __pyx_t_1 = __pyx_t_7;
          __pyx_t_7 = 0;

          /* "supplychainpy/demand/_evo_algo.pyx":106
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():             # <<<<<<<<<<<<<<
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 */
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_individual, __pyx_n_s_items); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 106, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
            }
          }
          if (__pyx_t_11) {
            __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 106, __pyx_L4_error)
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          } else {
            __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 106, __pyx_L4_error)
          }
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
            __pyx_t_10 = __pyx_t_7; __Pyx_INCREF(__pyx_t_10); __pyx_t_12 = 0;
            __pyx_t_13 = NULL;
          } else {
            __pyx_t_12 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 106, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_13 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 106, __pyx_L4_error)
          }
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          for (;;) {
            if (likely(!__pyx_t_13)) {
              if (likely(PyList_CheckExact(__pyx_t_10))) {
                if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_10)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_7 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_12); __Pyx_INCREF(__pyx_t_7); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 106, __pyx_L4_error)
                #else
                __pyx_t_7 = PySequence_ITEM(__pyx_t_10, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 106, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                #endif
              } else {
                if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_12); __Pyx_INCREF(__pyx_t_7); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 106, __pyx_L4_error)
                #else
                __pyx_t_7 = PySequence_ITEM(__pyx_t_10, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 106, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                #endif
              }
            } else {
              __pyx_t_7 = __pyx_t_13(__pyx_t_10);
              if (unlikely(!__pyx_t_7)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 106, __pyx_L4_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_7);
            }
            if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
              PyObject* sequence = __pyx_t_7;
              #if !CYTHON_COMPILING_IN_PYPY
              Py_ssize_t size = Py_SIZE(sequence);
              #else
              Py_ssize_t size = PySequence_Size(sequence);
              #endif
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(0, 106, __pyx_L4_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              if (likely(PyTuple_CheckExact(sequence))) {
                __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
                __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
              } else {
                __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
                __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
              }
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(__pyx_t_14);
              #else
              __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 106, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_11);
              __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 106, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_14);
              #endif
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            } else {
              Py_ssize_t index = -1;
              __pyx_t_15 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 106, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_16 = Py_TYPE(__pyx_t_15)->tp_iternext;
              index = 0; __pyx_t_11 = __pyx_t_16(__pyx_t_15); if (unlikely(!__pyx_t_11)) goto __pyx_L17_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_11);
              index = 1; __pyx_t_14 = __pyx_t_16(__pyx_t_15); if (unlikely(!__pyx_t_14)) goto __pyx_L17_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_14);
              if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_15), 2) < 0) __PYX_ERR(0, 106, __pyx_L4_error)
              __pyx_t_16 = NULL;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              goto __pyx_L18_unpacking_done;
              __pyx_L17_unpacking_failed:;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              __pyx_t_16 = NULL;
              if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
              __PYX_ERR(0, 106, __pyx_L4_error)
              __pyx_L18_unpacking_done:;
            }
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_key);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_11);
            __Pyx_GIVEREF(__pyx_t_11);
            __pyx_t_11 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_value);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_value, __pyx_t_14);
            __Pyx_GIVEREF(__pyx_t_14);
            __pyx_t_14 = 0;

            /* "supplychainpy/demand/_evo_algo.pyx":107
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())             # <<<<<<<<<<<<<<
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 */
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_individuals); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 107, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_17 = PyObject_Length(__pyx_t_7); if (unlikely(__pyx_t_17 == -1)) __PYX_ERR(0, 107, __pyx_L4_error)
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_individual, __pyx_n_s_items); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 107, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_11 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_11)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_11);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
              }
            }
            if (__pyx_t_11) {
              __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 107, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            } else {
              __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 107, __pyx_L4_error)
            }
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_18 = PyObject_Length(__pyx_t_7); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 107, __pyx_L4_error)
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_7 = PyInt_FromSsize_t((__pyx_t_17 * __pyx_t_18)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 107, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_population_allele_count);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_population_allele_count, __pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_7);
            __pyx_t_7 = 0;

            /* "supplychainpy/demand/_evo_algo.pyx":108
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)             # <<<<<<<<<<<<<<
 *                         mutation_index += 1
 *                         if (genome_count <= 5 and len(new_individual) < 12) or (
 */
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_mutation_probability); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 108, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_14 = PyNumber_Multiply(__pyx_cur_scope->__pyx_v_population_allele_count, __pyx_t_7); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 108, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 108, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_14);
            PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_14);
            __pyx_t_14 = 0;
            __pyx_t_14 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_7, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 108, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_number_of_mutations_allowed);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_number_of_mutations_allowed, __pyx_t_14);
            __Pyx_GIVEREF(__pyx_t_14);
            __pyx_t_14 = 0;

            /* "supplychainpy/demand/_evo_algo.pyx":109
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1             # <<<<<<<<<<<<<<
 *                         if (genome_count <= 5 and len(new_individual) < 12) or (
 *                                             18 >= genome_count > 12 > len(new_individual)):
 */
            __pyx_cur_scope->__pyx_v_mutation_index = (__pyx_cur_scope->__pyx_v_mutation_index + 1);

            /* "supplychainpy/demand/_evo_algo.pyx":110
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 *                         if (genome_count <= 5 and len(new_individual) < 12) or (             # <<<<<<<<<<<<<<
 *                                             18 >= genome_count > 12 > len(new_individual)):
 * 
 */
            __pyx_t_19 = ((__pyx_cur_scope->__pyx_v_genome_count <= 5) != 0);
            if (!__pyx_t_19) {
              goto __pyx_L21_next_or;
            } else {
            }
            __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 110, __pyx_L4_error)
            __pyx_t_19 = ((__pyx_t_18 < 12) != 0);
            if (!__pyx_t_19) {
            } else {
              __pyx_t_6 = __pyx_t_19;
              goto __pyx_L20_bool_binop_done;
            }
            __pyx_L21_next_or:;

            /* "supplychainpy/demand/_evo_algo.pyx":111
 *                         mutation_index += 1
 *                         if (genome_count <= 5 and len(new_individual) < 12) or (
 *                                             18 >= genome_count > 12 > len(new_individual)):             # <<<<<<<<<<<<<<
 * 
 *                             if mutation_index == number_of_mutations_allowed:
 */
            __pyx_t_19 = (18 >= __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_19) {
              __pyx_t_19 = (__pyx_cur_scope->__pyx_v_genome_count > 12);
              if (__pyx_t_19) {
                __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 111, __pyx_L4_error)
                __pyx_t_19 = (12 > __pyx_t_18);
              }
            }
            __pyx_t_20 = (__pyx_t_19 != 0);
            __pyx_t_6 = __pyx_t_20;
            __pyx_L20_bool_binop_done:;

            /* "supplychainpy/demand/_evo_algo.pyx":110
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 *                         if (genome_count <= 5 and len(new_individual) < 12) or (             # <<<<<<<<<<<<<<
 *                                             18 >= genome_count > 12 > len(new_individual)):
 * 
 */
            if (__pyx_t_6) {

              /* "supplychainpy/demand/_evo_algo.pyx":113
 *                                             18 >= genome_count > 12 > len(new_individual)):
 * 
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
              __pyx_t_14 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_mutation_index); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 113, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_14);
              __pyx_t_7 = PyObject_RichCompare(__pyx_t_14, __pyx_cur_scope->__pyx_v_number_of_mutations_allowed, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 113, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 113, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (__pyx_t_6) {

                /* "supplychainpy/demand/_evo_algo.pyx":114
 * 
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 *                                 mutation_index = 0
 */
                __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 114, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_mutation); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 114, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_21 = PyDict_New(); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 114, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_21);
                if (PyDict_SetItem(__pyx_t_21, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 114, __pyx_L4_error)
                __pyx_t_22 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
                  __pyx_t_22 = PyMethod_GET_SELF(__pyx_t_15);
                  if (likely(__pyx_t_22)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
                    __Pyx_INCREF(__pyx_t_22);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_15, function);
                  }
                }
                if (!__pyx_t_22) {
                  __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_t_21); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 114, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                  __Pyx_GOTREF(__pyx_t_11);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_15)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_22, __pyx_t_21};
                    __pyx_t_11 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                    __Pyx_GOTREF(__pyx_t_11);
                    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_22, __pyx_t_21};
                    __pyx_t_11 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                    __Pyx_GOTREF(__pyx_t_11);
                    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_23 = PyTuple_New(1+1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_23);
                    __Pyx_GIVEREF(__pyx_t_22); PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_t_22); __pyx_t_22 = NULL;
                    __Pyx_GIVEREF(__pyx_t_21);
                    PyTuple_SET_ITEM(__pyx_t_23, 0+1, __pyx_t_21);
                    __pyx_t_21 = 0;
                    __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_23, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_11);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_14, function);
                  }
                }
                if (!__pyx_t_15) {
                  __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 114, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  __Pyx_GOTREF(__pyx_t_7);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_11};
                    __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_11};
                    __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_23 = PyTuple_New(1+1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_23);
                    __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_t_15); __pyx_t_15 = NULL;
                    __Pyx_GIVEREF(__pyx_t_11);
                    PyTuple_SET_ITEM(__pyx_t_23, 0+1, __pyx_t_11);
                    __pyx_t_11 = 0;
                    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_23, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 114, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":115
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 *                                 mutation_index = 0
 *                                 mutation_count += 1
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":116
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 *                                 mutation_index = 0             # <<<<<<<<<<<<<<
 *                                 mutation_count += 1
 *                             else:
 */
                __pyx_cur_scope->__pyx_v_mutation_index = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":117
 *                                 genome_count += 1
 *                                 mutation_index = 0
 *                                 mutation_count += 1             # <<<<<<<<<<<<<<
 *                             else:
 *                                 new_individual.update({key: value})
 */
                __pyx_cur_scope->__pyx_v_mutation_count = (__pyx_cur_scope->__pyx_v_mutation_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":113
 *                                             18 >= genome_count > 12 > len(new_individual)):
 * 
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
                goto __pyx_L23;
              }

              /* "supplychainpy/demand/_evo_algo.pyx":119
 *                                 mutation_count += 1
 *                             else:
 *                                 new_individual.update({key: value})             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 * 
 */
              /*else*/ {
                __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 119, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_23 = PyDict_New(); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 119, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_23);
                if (PyDict_SetItem(__pyx_t_23, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 119, __pyx_L4_error)
                __pyx_t_11 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
                  __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_14);
                  if (likely(__pyx_t_11)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                    __Pyx_INCREF(__pyx_t_11);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_14, function);
                  }
                }
                if (!__pyx_t_11) {
                  __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_23); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 119, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  __Pyx_GOTREF(__pyx_t_7);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_23};
                    __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 119, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_23};
                    __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 119, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 119, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_11); __pyx_t_11 = NULL;
                    __Pyx_GIVEREF(__pyx_t_23);
                    PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_t_23);
                    __pyx_t_23 = 0;
                    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_15, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 119, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":120
 *                             else:
 *                                 new_individual.update({key: value})
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 * 
 *                         elif (genome_count >= 6 and len(new_individual_two) < 12) or (18 < genome_count <= 24 and len(
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);
              }
              __pyx_L23:;

              /* "supplychainpy/demand/_evo_algo.pyx":110
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 *                         if (genome_count <= 5 and len(new_individual) < 12) or (             # <<<<<<<<<<<<<<
 *                                             18 >= genome_count > 12 > len(new_individual)):
 * 
 */
              goto __pyx_L19;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":122
 *                                 genome_count += 1
 * 
 *                         elif (genome_count >= 6 and len(new_individual_two) < 12) or (18 < genome_count <= 24 and len(             # <<<<<<<<<<<<<<
 *                                 new_individual) < 12):
 * 
 */
            __pyx_t_20 = ((__pyx_cur_scope->__pyx_v_genome_count >= 6) != 0);
            if (!__pyx_t_20) {
              goto __pyx_L25_next_or;
            } else {
            }
            __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual_two); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 122, __pyx_L4_error)
            __pyx_t_20 = ((__pyx_t_18 < 12) != 0);
            if (!__pyx_t_20) {
            } else {
              __pyx_t_6 = __pyx_t_20;
              goto __pyx_L24_bool_binop_done;
            }
            __pyx_L25_next_or:;
            __pyx_t_20 = (18 < __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_20) {
              __pyx_t_20 = (__pyx_cur_scope->__pyx_v_genome_count <= 24);
            }
            __pyx_t_19 = (__pyx_t_20 != 0);
            if (__pyx_t_19) {
            } else {
              __pyx_t_6 = __pyx_t_19;
              goto __pyx_L24_bool_binop_done;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":123
 * 
 *                         elif (genome_count >= 6 and len(new_individual_two) < 12) or (18 < genome_count <= 24 and len(
 *                                 new_individual) < 12):             # <<<<<<<<<<<<<<
 * 
 *                             if mutation_index == number_of_mutations_allowed:
 */
            __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 122, __pyx_L4_error)
            __pyx_t_19 = ((__pyx_t_18 < 12) != 0);
            __pyx_t_6 = __pyx_t_19;
            __pyx_L24_bool_binop_done:;

            /* "supplychainpy/demand/_evo_algo.pyx":122
 *                                 genome_count += 1
 * 
 *                         elif (genome_count >= 6 and len(new_individual_two) < 12) or (18 < genome_count <= 24 and len(             # <<<<<<<<<<<<<<
 *                                 new_individual) < 12):
 * 
 */
            if (__pyx_t_6) {

              /* "supplychainpy/demand/_evo_algo.pyx":125
 *                                 new_individual) < 12):
 * 
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual_two.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
              __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_mutation_index); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 125, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_14 = PyObject_RichCompare(__pyx_t_7, __pyx_cur_scope->__pyx_v_number_of_mutations_allowed, Py_EQ); __Pyx_XGOTREF(__pyx_t_14); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 125, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_14); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 125, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (__pyx_t_6) {

                /* "supplychainpy/demand/_evo_algo.pyx":126
 * 
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual_two.update(self._mutation({key: value}))             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 *                                 mutation_index = 0
 */
                __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual_two, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 126, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                __pyx_t_23 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_mutation); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 126, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_23);
                __pyx_t_11 = PyDict_New(); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 126, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_11);
                if (PyDict_SetItem(__pyx_t_11, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 126, __pyx_L4_error)
                __pyx_t_21 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_23))) {
                  __pyx_t_21 = PyMethod_GET_SELF(__pyx_t_23);
                  if (likely(__pyx_t_21)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_23);
                    __Pyx_INCREF(__pyx_t_21);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_23, function);
                  }
                }
                if (!__pyx_t_21) {
                  __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_t_23, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 126, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  __Pyx_GOTREF(__pyx_t_15);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_23)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_21, __pyx_t_11};
                    __pyx_t_15 = __Pyx_PyFunction_FastCall(__pyx_t_23, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_23)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_21, __pyx_t_11};
                    __pyx_t_15 = __Pyx_PyCFunction_FastCall(__pyx_t_23, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_22 = PyTuple_New(1+1); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_22);
                    __Pyx_GIVEREF(__pyx_t_21); PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_t_21); __pyx_t_21 = NULL;
                    __Pyx_GIVEREF(__pyx_t_11);
                    PyTuple_SET_ITEM(__pyx_t_22, 0+1, __pyx_t_11);
                    __pyx_t_11 = 0;
                    __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_23, __pyx_t_22, NULL); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                __pyx_t_23 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                  __pyx_t_23 = PyMethod_GET_SELF(__pyx_t_7);
                  if (likely(__pyx_t_23)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                    __Pyx_INCREF(__pyx_t_23);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_7, function);
                  }
                }
                if (!__pyx_t_23) {
                  __pyx_t_14 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 126, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  __Pyx_GOTREF(__pyx_t_14);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_23, __pyx_t_15};
                    __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_23, __pyx_t_15};
                    __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_22 = PyTuple_New(1+1); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_22);
                    __Pyx_GIVEREF(__pyx_t_23); PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_t_23); __pyx_t_23 = NULL;
                    __Pyx_GIVEREF(__pyx_t_15);
                    PyTuple_SET_ITEM(__pyx_t_22, 0+1, __pyx_t_15);
                    __pyx_t_15 = 0;
                    __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_22, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 126, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":127
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual_two.update(self._mutation({key: value}))
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 *                                 mutation_index = 0
 *                                 mutation_count += 1
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":128
 *                                 new_individual_two.update(self._mutation({key: value}))
 *                                 genome_count += 1
 *                                 mutation_index = 0             # <<<<<<<<<<<<<<
 *                                 mutation_count += 1
 *                             else:
 */
                __pyx_cur_scope->__pyx_v_mutation_index = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":129
 *                                 genome_count += 1
 *                                 mutation_index = 0
 *                                 mutation_count += 1             # <<<<<<<<<<<<<<
 *                             else:
 *                                 new_individual.update({key: value})
 */
                __pyx_cur_scope->__pyx_v_mutation_count = (__pyx_cur_scope->__pyx_v_mutation_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":125
 *                                 new_individual) < 12):
 * 
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual_two.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
                goto __pyx_L28;
              }

              /* "supplychainpy/demand/_evo_algo.pyx":131
 *                                 mutation_count += 1
 *                             else:
 *                                 new_individual.update({key: value})             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 * 
 */
              /*else*/ {
                __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 131, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                __pyx_t_22 = PyDict_New(); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 131, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_22);
                if (PyDict_SetItem(__pyx_t_22, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 131, __pyx_L4_error)
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_7, function);
                  }
                }
                if (!__pyx_t_15) {
                  __pyx_t_14 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_22); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 131, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  __Pyx_GOTREF(__pyx_t_14);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_22};
                    __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 131, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_22};
                    __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 131, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_23 = PyTuple_New(1+1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 131, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_23);
                    __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_t_15); __pyx_t_15 = NULL;
                    __Pyx_GIVEREF(__pyx_t_22);
                    PyTuple_SET_ITEM(__pyx_t_23, 0+1, __pyx_t_22);
                    __pyx_t_22 = 0;
                    __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_23, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 131, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":132
 *                             else:
 *                                 new_individual.update({key: value})
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 * 
 *                         else:
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);
              }
              __pyx_L28:;

              /* "supplychainpy/demand/_evo_algo.pyx":122
 *                                 genome_count += 1
 * 
 *                         elif (genome_count >= 6 and len(new_individual_two) < 12) or (18 < genome_count <= 24 and len(             # <<<<<<<<<<<<<<
 *                                 new_individual) < 12):
 * 
 */
              goto __pyx_L19;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":135
 * 
 *                         else:
 *                             genome_count += 1             # <<<<<<<<<<<<<<
 * 
 *                         if genome_count == 24:
 */
            /*else*/ {
              __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);
            }
            __pyx_L19:;

            /* "supplychainpy/demand/_evo_algo.pyx":137
 *                             genome_count += 1
 * 
 *                         if genome_count == 24:             # <<<<<<<<<<<<<<
 *                             genome_count = 0
 *                             # if new_individual != new_individual_two:
 */
            __pyx_t_6 = ((__pyx_cur_scope->__pyx_v_genome_count == 24) != 0);
            if (__pyx_t_6) {

              /* "supplychainpy/demand/_evo_algo.pyx":138
 * 
 *                         if genome_count == 24:
 *                             genome_count = 0             # <<<<<<<<<<<<<<
 *                             # if new_individual != new_individual_two:
 *                             # log.debug('single point crossover was successful')
 */
              __pyx_cur_scope->__pyx_v_genome_count = 0;

              /* "supplychainpy/demand/_evo_algo.pyx":143
 *                             # else:
 *                             # log.debug('single point crossover resulted in clone of parent')
 *                             yield (new_individual)             # <<<<<<<<<<<<<<
 *                             yield (new_individual_two)
 * 
 */
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_new_individual);
              __pyx_r = __pyx_cur_scope->__pyx_v_new_individual;
              __Pyx_XGIVEREF(__pyx_t_1);
              __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
              __Pyx_XGIVEREF(__pyx_t_2);
              __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
              __Pyx_XGIVEREF(__pyx_t_3);
              __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
              __Pyx_XGIVEREF(__pyx_t_4);
              __pyx_cur_scope->__pyx_t_3 = __pyx_t_4;
              __pyx_cur_scope->__pyx_t_4 = __pyx_t_5;
              __Pyx_XGIVEREF(__pyx_t_8);
              __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
              __pyx_cur_scope->__pyx_t_6 = __pyx_t_9;
              __Pyx_XGIVEREF(__pyx_t_10);
              __pyx_cur_scope->__pyx_t_7 = __pyx_t_10;
              __pyx_cur_scope->__pyx_t_8 = __pyx_t_12;
              __pyx_cur_scope->__pyx_t_9 = __pyx_t_13;
              __Pyx_XGIVEREF(__pyx_r);
              __Pyx_RefNannyFinishContext();
              /* return from generator, yielding value */
              __pyx_generator->resume_label = 1;
              return __pyx_r;
              __pyx_L30_resume_from_yield:;
              __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
              __pyx_cur_scope->__pyx_t_0 = 0;
              __Pyx_XGOTREF(__pyx_t_1);
              __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
              __pyx_cur_scope->__pyx_t_1 = 0;
              __Pyx_XGOTREF(__pyx_t_2);
              __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
              __pyx_cur_scope->__pyx_t_2 = 0;
              __Pyx_XGOTREF(__pyx_t_3);
              __pyx_t_4 = __pyx_cur_scope->__pyx_t_3;
              __pyx_cur_scope->__pyx_t_3 = 0;
              __Pyx_XGOTREF(__pyx_t_4);
              __pyx_t_5 = __pyx_cur_scope->__pyx_t_4;
              __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
              __pyx_cur_scope->__pyx_t_5 = 0;
              __Pyx_XGOTREF(__pyx_t_8);
              __pyx_t_9 = __pyx_cur_scope->__pyx_t_6;
              __pyx_t_10 = __pyx_cur_scope->__pyx_t_7;
              __pyx_cur_scope->__pyx_t_7 = 0;
              __Pyx_XGOTREF(__pyx_t_10);
              __pyx_t_12 = __pyx_cur_scope->__pyx_t_8;
              __pyx_t_13 = __pyx_cur_scope->__pyx_t_9;
              if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 143, __pyx_L4_error)

              /* "supplychainpy/demand/_evo_algo.pyx":144
 *                             # log.debug('single point crossover resulted in clone of parent')
 *                             yield (new_individual)
 *                             yield (new_individual_two)             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_new_individual_two);
              __pyx_r = __pyx_cur_scope->__pyx_v_new_individual_two;
              __Pyx_XGIVEREF(__pyx_t_1);
              __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
              __Pyx_XGIVEREF(__pyx_t_2);
              __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
              __Pyx_XGIVEREF(__pyx_t_3);
              __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
              __Pyx_XGIVEREF(__pyx_t_4);
              __pyx_cur_scope->__pyx_t_3 = __pyx_t_4;
              __pyx_cur_scope->__pyx_t_4 = __pyx_t_5;
              __Pyx_XGIVEREF(__pyx_t_8);
              __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
              __pyx_cur_scope->__pyx_t_6 = __pyx_t_9;
              __Pyx_XGIVEREF(__pyx_t_10);
              __pyx_cur_scope->__pyx_t_7 = __pyx_t_10;
              __pyx_cur_scope->__pyx_t_8 = __pyx_t_12;
              __pyx_cur_scope->__pyx_t_9 = __pyx_t_13;
              __Pyx_XGIVEREF(__pyx_r);
              __Pyx_RefNannyFinishContext();
              /* return from generator, yielding value */
              __pyx_generator->resume_label = 2;
              return __pyx_r;
              __pyx_L31_resume_from_yield:;
              __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
              __pyx_cur_scope->__pyx_t_0 = 0;
              __Pyx_XGOTREF(__pyx_t_1);
              __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
              __pyx_cur_scope->__pyx_t_1 = 0;
              __Pyx_XGOTREF(__pyx_t_2);
              __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
              __pyx_cur_scope->__pyx_t_2 = 0;
              __Pyx_XGOTREF(__pyx_t_3);
              __pyx_t_4 = __pyx_cur_scope->__pyx_t_3;
              __pyx_cur_scope->__pyx_t_3 = 0;
              __Pyx_XGOTREF(__pyx_t_4);
              __pyx_t_5 = __pyx_cur_scope->__pyx_t_4;
              __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
              __pyx_cur_scope->__pyx_t_5 = 0;
              __Pyx_XGOTREF(__pyx_t_8);
              __pyx_t_9 = __pyx_cur_scope->__pyx_t_6;
              __pyx_t_10 = __pyx_cur_scope->__pyx_t_7;
              __pyx_cur_scope->__pyx_t_7 = 0;
              __Pyx_XGOTREF(__pyx_t_10);
              __pyx_t_12 = __pyx_cur_scope->__pyx_t_8;
              __pyx_t_13 = __pyx_cur_scope->__pyx_t_9;
              if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 144, __pyx_L4_error)

              /* "supplychainpy/demand/_evo_algo.pyx":137
 *                             genome_count += 1
 * 
 *                         if genome_count == 24:             # <<<<<<<<<<<<<<
 *                             genome_count = 0
 *                             # if new_individual != new_individual_two:
 */
            }

            /* "supplychainpy/demand/_evo_algo.pyx":106
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():             # <<<<<<<<<<<<<<
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 */
          }
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "supplychainpy/demand/_evo_algo.pyx":105
 * 
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):             # <<<<<<<<<<<<<<
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 */
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "supplychainpy/demand/_evo_algo.pyx":104
 *         try:
 * 
 *             if len(self.individuals) > 1:             # <<<<<<<<<<<<<<
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():
 */
        goto __pyx_L12;
      }

      /* "supplychainpy/demand/_evo_algo.pyx":147
 * 
 *             else:
 *                 raise ValueError()             # <<<<<<<<<<<<<<
 * 
 *         except ValueError:
 */
      /*else*/ {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_ValueError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 147, __pyx_L4_error)
      }
      __pyx_L12:;

      /* "supplychainpy/demand/_evo_algo.pyx":102
 *         cdef int mutation_index = 0
 * 
 *         try:             # <<<<<<<<<<<<<<
 * 
 *             if len(self.individuals) > 1:
 */
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_PyThreadState_assign
    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
    __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":149
 *                 raise ValueError()
 * 
 *         except ValueError:             # <<<<<<<<<<<<<<
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')
 * 
 */
    __pyx_t_24 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError);
    if (__pyx_t_24) {
      __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population._single_point_crossover_recombination", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_10) < 0) __PYX_ERR(0, 149, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_10);

      /* "supplychainpy/demand/_evo_algo.pyx":150
 * 
 *         except ValueError:
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')             # <<<<<<<<<<<<<<
 * 
 *     def _two_point_crossover_recombination(self):
 */
      if (__Pyx_PrintOne(0, __pyx_kp_s_Population_Size_is_too_small_for) < 0) __PYX_ERR(0, 150, __pyx_L6_except_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "supplychainpy/demand/_evo_algo.pyx":102
 *         cdef int mutation_index = 0
 * 
 *         try:             # <<<<<<<<<<<<<<
 * 
 *             if len(self.individuals) > 1:
 */
    __Pyx_PyThreadState_assign
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_PyThreadState_assign
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    __pyx_L11_try_end:;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":85
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 *     def _single_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on one point along the chromosome.
 * 
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_AddTraceback("_single_point_crossover_recombination", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_10generator2(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":152
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')
 * 
 *     def _two_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on two points along the chromosome.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_9_two_point_crossover_recombination(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_8_two_point_crossover_recombination[] = " Selects genes from the parent to crossover based on two points along the chromosome.\n\n        Yields:\n            individual (dict):          Parents p1-xxxxxxxxxxxxx and p2-oooooooooooo will produce offspring c1-xxxooooooxxx\n                                        and c2-oooxxxxxxooo. Individual yields the first offspring (c1).\n            individual_two (dict):      Parents p1-xxxxxxxxxxxxx and p2-oooooooooooo will produce offspring c1-xxxooooooxxx\n                                        and c2-oooxxxxxxooo. Individual_two yields the second offspring (c2).\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_9_two_point_crossover_recombination = {"_two_point_crossover_recombination", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_9_two_point_crossover_recombination, METH_O, __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_8_two_point_crossover_recombination};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_9_two_point_crossover_recombination(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_two_point_crossover_recombination (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_8_two_point_crossover_recombination(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_8_two_point_crossover_recombination(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_two_point_crossover_recombination", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 152, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_10generator2, (PyObject *) __pyx_cur_scope, __pyx_n_s_two_point_crossover_recombinati, __pyx_n_s_Population__two_point_crossover, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population._two_point_crossover_recombination", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_10Population_10generator2(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *(*__pyx_t_16)(PyObject *);
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_t_19;
  int __pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  int __pyx_t_24;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L32_resume_from_yield;
    case 2: goto __pyx_L33_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 152, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":162
 *         """
 * 
 *         cdef int genome_count = 0             # <<<<<<<<<<<<<<
 *         cdef dict new_individual = {}
 * 
 */
  __pyx_cur_scope->__pyx_v_genome_count = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":163
 * 
 *         cdef int genome_count = 0
 *         cdef dict new_individual = {}             # <<<<<<<<<<<<<<
 * 
 *         cdef int mutation_count = 0
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_new_individual = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":165
 *         cdef dict new_individual = {}
 * 
 *         cdef int mutation_count = 0             # <<<<<<<<<<<<<<
 *         cdef int mutation_index = 0
 *         cdef dict new_individual_two = {}
 */
  __pyx_cur_scope->__pyx_v_mutation_count = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":166
 * 
 *         cdef int mutation_count = 0
 *         cdef int mutation_index = 0             # <<<<<<<<<<<<<<
 *         cdef dict new_individual_two = {}
 * 
 */
  __pyx_cur_scope->__pyx_v_mutation_index = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":167
 *         cdef int mutation_count = 0
 *         cdef int mutation_index = 0
 *         cdef dict new_individual_two = {}             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_new_individual_two = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":169
 *         cdef dict new_individual_two = {}
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "supplychainpy/demand/_evo_algo.pyx":170
 * 
 *         try:
 *             if len(self.individuals) > 1:             # <<<<<<<<<<<<<<
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_individuals); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 170, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = ((__pyx_t_5 > 1) != 0);
      if (__pyx_t_6) {

        /* "supplychainpy/demand/_evo_algo.pyx":171
 *         try:
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):             # <<<<<<<<<<<<<<
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 */
        __Pyx_INCREF(__pyx_int_0);
        __pyx_t_1 = __pyx_int_0;
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_individuals); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 171, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
          __pyx_t_8 = __pyx_t_7; __Pyx_INCREF(__pyx_t_8); __pyx_t_5 = 0;
          __pyx_t_9 = NULL;
        } else {
          __pyx_t_5 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 171, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 171, __pyx_L4_error)
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        for (;;) {
          if (likely(!__pyx_t_9)) {
            if (likely(PyList_CheckExact(__pyx_t_8))) {
              if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_8)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_7 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 171, __pyx_L4_error)
              #else
              __pyx_t_7 = PySequence_ITEM(__pyx_t_8, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 171, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_7);
              #endif
            } else {
              if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 171, __pyx_L4_error)
              #else
              __pyx_t_7 = PySequence_ITEM(__pyx_t_8, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 171, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_7);
              #endif
            }
          } else {
            __pyx_t_7 = __pyx_t_9(__pyx_t_8);
            if (unlikely(!__pyx_t_7)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 171, __pyx_L4_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_7);
          }
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_individual);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_individual, __pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_7);
          __pyx_t_7 = 0;
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_index);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_index, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 171, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_1);
          __pyx_t_1 = __pyx_t_7;
          __pyx_t_7 = 0;

          /* "supplychainpy/demand/_evo_algo.pyx":172
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():             # <<<<<<<<<<<<<<
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 */
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_individual, __pyx_n_s_items); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 172, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
            }
          }
          if (__pyx_t_11) {
            __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 172, __pyx_L4_error)
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          } else {
            __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 172, __pyx_L4_error)
          }
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
            __pyx_t_10 = __pyx_t_7; __Pyx_INCREF(__pyx_t_10); __pyx_t_12 = 0;
            __pyx_t_13 = NULL;
          } else {
            __pyx_t_12 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 172, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_13 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 172, __pyx_L4_error)
          }
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          for (;;) {
            if (likely(!__pyx_t_13)) {
              if (likely(PyList_CheckExact(__pyx_t_10))) {
                if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_10)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_7 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_12); __Pyx_INCREF(__pyx_t_7); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 172, __pyx_L4_error)
                #else
                __pyx_t_7 = PySequence_ITEM(__pyx_t_10, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 172, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                #endif
              } else {
                if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_12); __Pyx_INCREF(__pyx_t_7); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 172, __pyx_L4_error)
                #else
                __pyx_t_7 = PySequence_ITEM(__pyx_t_10, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 172, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                #endif
              }
            } else {
              __pyx_t_7 = __pyx_t_13(__pyx_t_10);
              if (unlikely(!__pyx_t_7)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 172, __pyx_L4_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_7);
            }
            if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
              PyObject* sequence = __pyx_t_7;
              #if !CYTHON_COMPILING_IN_PYPY
              Py_ssize_t size = Py_SIZE(sequence);
              #else
              Py_ssize_t size = PySequence_Size(sequence);
              #endif
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(0, 172, __pyx_L4_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              if (likely(PyTuple_CheckExact(sequence))) {
                __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
                __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
              } else {
                __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
                __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
              }
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(__pyx_t_14);
              #else
              __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 172, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_11);
              __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 172, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_14);
              #endif
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            } else {
              Py_ssize_t index = -1;
              __pyx_t_15 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 172, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_16 = Py_TYPE(__pyx_t_15)->tp_iternext;
              index = 0; __pyx_t_11 = __pyx_t_16(__pyx_t_15); if (unlikely(!__pyx_t_11)) goto __pyx_L17_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_11);
              index = 1; __pyx_t_14 = __pyx_t_16(__pyx_t_15); if (unlikely(!__pyx_t_14)) goto __pyx_L17_unpacking_failed;
              __Pyx_GOTREF(__pyx_t_14);
              if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_15), 2) < 0) __PYX_ERR(0, 172, __pyx_L4_error)
              __pyx_t_16 = NULL;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              goto __pyx_L18_unpacking_done;
              __pyx_L17_unpacking_failed:;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              __pyx_t_16 = NULL;
              if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
              __PYX_ERR(0, 172, __pyx_L4_error)
              __pyx_L18_unpacking_done:;
            }
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_key);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_11);
            __Pyx_GIVEREF(__pyx_t_11);
            __pyx_t_11 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_value);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_value, __pyx_t_14);
            __Pyx_GIVEREF(__pyx_t_14);
            __pyx_t_14 = 0;

            /* "supplychainpy/demand/_evo_algo.pyx":173
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())             # <<<<<<<<<<<<<<
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 */
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_individuals); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 173, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_17 = PyObject_Length(__pyx_t_7); if (unlikely(__pyx_t_17 == -1)) __PYX_ERR(0, 173, __pyx_L4_error)
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_individual, __pyx_n_s_items); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 173, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_11 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_11)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_11);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
              }
            }
            if (__pyx_t_11) {
              __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 173, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            } else {
              __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 173, __pyx_L4_error)
            }
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_18 = PyObject_Length(__pyx_t_7); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 173, __pyx_L4_error)
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_7 = PyInt_FromSsize_t((__pyx_t_17 * __pyx_t_18)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 173, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_population_allele_count);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_population_allele_count, __pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_7);
            __pyx_t_7 = 0;

            /* "supplychainpy/demand/_evo_algo.pyx":174
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)             # <<<<<<<<<<<<<<
 *                         mutation_index += 1
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (
 */
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_mutation_probability); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 174, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_14 = PyNumber_Multiply(__pyx_cur_scope->__pyx_v_population_allele_count, __pyx_t_7); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 174, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 174, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_14);
            PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_14);
            __pyx_t_14 = 0;
            __pyx_t_14 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_7, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 174, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_number_of_mutations_allowed);
            __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_number_of_mutations_allowed, __pyx_t_14);
            __Pyx_GIVEREF(__pyx_t_14);
            __pyx_t_14 = 0;

            /* "supplychainpy/demand/_evo_algo.pyx":175
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1             # <<<<<<<<<<<<<<
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (
 *                                         9 <= genome_count <= 11):
 */
            __pyx_cur_scope->__pyx_v_mutation_index = (__pyx_cur_scope->__pyx_v_mutation_index + 1);

            /* "supplychainpy/demand/_evo_algo.pyx":176
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (             # <<<<<<<<<<<<<<
 *                                         9 <= genome_count <= 11):
 *                             if mutation_index == number_of_mutations_allowed:
 */
            __pyx_t_19 = ((2 >= __pyx_cur_scope->__pyx_v_genome_count) != 0);
            if (!__pyx_t_19) {
              goto __pyx_L21_next_or;
            } else {
            }
            __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 176, __pyx_L4_error)
            __pyx_t_19 = ((__pyx_t_18 < 12) != 0);
            if (!__pyx_t_19) {
            } else {
              __pyx_t_6 = __pyx_t_19;
              goto __pyx_L20_bool_binop_done;
            }
            __pyx_L21_next_or:;
            __pyx_t_19 = (15 <= __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_19) {
              __pyx_t_19 = (__pyx_cur_scope->__pyx_v_genome_count <= 20);
            }
            __pyx_t_20 = (__pyx_t_19 != 0);
            if (!__pyx_t_20) {
            } else {
              __pyx_t_6 = __pyx_t_20;
              goto __pyx_L20_bool_binop_done;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":177
 *                         mutation_index += 1
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (
 *                                         9 <= genome_count <= 11):             # <<<<<<<<<<<<<<
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))
 */
            __pyx_t_20 = (9 <= __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_20) {
              __pyx_t_20 = (__pyx_cur_scope->__pyx_v_genome_count <= 11);
            }
            __pyx_t_19 = (__pyx_t_20 != 0);
            __pyx_t_6 = __pyx_t_19;
            __pyx_L20_bool_binop_done:;

            /* "supplychainpy/demand/_evo_algo.pyx":176
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (             # <<<<<<<<<<<<<<
 *                                         9 <= genome_count <= 11):
 *                             if mutation_index == number_of_mutations_allowed:
 */
            if (__pyx_t_6) {

              /* "supplychainpy/demand/_evo_algo.pyx":178
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (
 *                                         9 <= genome_count <= 11):
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
              __pyx_t_14 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_mutation_index); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 178, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_14);
              __pyx_t_7 = PyObject_RichCompare(__pyx_t_14, __pyx_cur_scope->__pyx_v_number_of_mutations_allowed, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 178, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 178, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (__pyx_t_6) {

                /* "supplychainpy/demand/_evo_algo.pyx":179
 *                                         9 <= genome_count <= 11):
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 *                                 mutation_index = 0
 */
                __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 179, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_mutation); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 179, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_21 = PyDict_New(); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 179, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_21);
                if (PyDict_SetItem(__pyx_t_21, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 179, __pyx_L4_error)
                __pyx_t_22 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
                  __pyx_t_22 = PyMethod_GET_SELF(__pyx_t_15);
                  if (likely(__pyx_t_22)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
                    __Pyx_INCREF(__pyx_t_22);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_15, function);
                  }
                }
                if (!__pyx_t_22) {
                  __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_t_21); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 179, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                  __Pyx_GOTREF(__pyx_t_11);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_15)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_22, __pyx_t_21};
                    __pyx_t_11 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                    __Pyx_GOTREF(__pyx_t_11);
                    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_22, __pyx_t_21};
                    __pyx_t_11 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                    __Pyx_GOTREF(__pyx_t_11);
                    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_23 = PyTuple_New(1+1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_23);
                    __Pyx_GIVEREF(__pyx_t_22); PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_t_22); __pyx_t_22 = NULL;
                    __Pyx_GIVEREF(__pyx_t_21);
                    PyTuple_SET_ITEM(__pyx_t_23, 0+1, __pyx_t_21);
                    __pyx_t_21 = 0;
                    __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_23, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_11);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_14, function);
                  }
                }
                if (!__pyx_t_15) {
                  __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 179, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  __Pyx_GOTREF(__pyx_t_7);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_11};
                    __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_11};
                    __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_23 = PyTuple_New(1+1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_23);
                    __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_t_15); __pyx_t_15 = NULL;
                    __Pyx_GIVEREF(__pyx_t_11);
                    PyTuple_SET_ITEM(__pyx_t_23, 0+1, __pyx_t_11);
                    __pyx_t_11 = 0;
                    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_23, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 179, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":180
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 *                                 mutation_index = 0
 *                                 mutation_count += 1
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":181
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 *                                 mutation_index = 0             # <<<<<<<<<<<<<<
 *                                 mutation_count += 1
 *                             else:
 */
                __pyx_cur_scope->__pyx_v_mutation_index = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":182
 *                                 genome_count += 1
 *                                 mutation_index = 0
 *                                 mutation_count += 1             # <<<<<<<<<<<<<<
 *                             else:
 *                                 new_individual.update({key: value})
 */
                __pyx_cur_scope->__pyx_v_mutation_count = (__pyx_cur_scope->__pyx_v_mutation_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":178
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (
 *                                         9 <= genome_count <= 11):
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
                goto __pyx_L24;
              }

              /* "supplychainpy/demand/_evo_algo.pyx":184
 *                                 mutation_count += 1
 *                             else:
 *                                 new_individual.update({key: value})             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 * 
 */
              /*else*/ {
                __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 184, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_23 = PyDict_New(); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 184, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_23);
                if (PyDict_SetItem(__pyx_t_23, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 184, __pyx_L4_error)
                __pyx_t_11 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
                  __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_14);
                  if (likely(__pyx_t_11)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                    __Pyx_INCREF(__pyx_t_11);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_14, function);
                  }
                }
                if (!__pyx_t_11) {
                  __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_23); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 184, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  __Pyx_GOTREF(__pyx_t_7);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_23};
                    __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 184, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_23};
                    __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 184, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 184, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_11); __pyx_t_11 = NULL;
                    __Pyx_GIVEREF(__pyx_t_23);
                    PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_t_23);
                    __pyx_t_23 = 0;
                    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_15, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 184, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":185
 *                             else:
 *                                 new_individual.update({key: value})
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 * 
 *                         elif (12 <= genome_count <= 14 and len(new_individual_two) < 12) or (
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);
              }
              __pyx_L24:;

              /* "supplychainpy/demand/_evo_algo.pyx":176
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 *                         mutation_index += 1
 *                         if (2 >= genome_count and len(new_individual) < 12) or (15 <= genome_count <= 20) or (             # <<<<<<<<<<<<<<
 *                                         9 <= genome_count <= 11):
 *                             if mutation_index == number_of_mutations_allowed:
 */
              goto __pyx_L19;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":187
 *                                 genome_count += 1
 * 
 *                         elif (12 <= genome_count <= 14 and len(new_individual_two) < 12) or (             # <<<<<<<<<<<<<<
 *                                         3 <= genome_count <= 8) and (len(
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 */
            __pyx_t_19 = (12 <= __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_19) {
              __pyx_t_19 = (__pyx_cur_scope->__pyx_v_genome_count <= 14);
            }
            __pyx_t_20 = (__pyx_t_19 != 0);
            if (!__pyx_t_20) {
              goto __pyx_L26_next_or;
            } else {
            }
            __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual_two); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 187, __pyx_L4_error)
            __pyx_t_20 = ((__pyx_t_18 < 12) != 0);
            if (!__pyx_t_20) {
            } else {
              __pyx_t_6 = __pyx_t_20;
              goto __pyx_L25_bool_binop_done;
            }
            __pyx_L26_next_or:;

            /* "supplychainpy/demand/_evo_algo.pyx":188
 * 
 *                         elif (12 <= genome_count <= 14 and len(new_individual_two) < 12) or (
 *                                         3 <= genome_count <= 8) and (len(             # <<<<<<<<<<<<<<
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 *                             if mutation_index == number_of_mutations_allowed:
 */
            __pyx_t_20 = (3 <= __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_20) {
              __pyx_t_20 = (__pyx_cur_scope->__pyx_v_genome_count <= 8);
            }
            __pyx_t_19 = (__pyx_t_20 != 0);
            if (__pyx_t_19) {
            } else {
              __pyx_t_6 = __pyx_t_19;
              goto __pyx_L25_bool_binop_done;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":189
 *                         elif (12 <= genome_count <= 14 and len(new_individual_two) < 12) or (
 *                                         3 <= genome_count <= 8) and (len(
 *                             new_individual) < 12 or 21 <= genome_count <= 23):             # <<<<<<<<<<<<<<
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))
 */
            __pyx_t_18 = PyDict_Size(__pyx_cur_scope->__pyx_v_new_individual); if (unlikely(__pyx_t_18 == -1)) __PYX_ERR(0, 188, __pyx_L4_error)
            __pyx_t_19 = ((__pyx_t_18 < 12) != 0);
            if (!__pyx_t_19) {
            } else {
              __pyx_t_6 = __pyx_t_19;
              goto __pyx_L25_bool_binop_done;
            }
            __pyx_t_19 = (21 <= __pyx_cur_scope->__pyx_v_genome_count);
            if (__pyx_t_19) {
              __pyx_t_19 = (__pyx_cur_scope->__pyx_v_genome_count <= 23);
            }
            __pyx_t_20 = (__pyx_t_19 != 0);
            __pyx_t_6 = __pyx_t_20;
            __pyx_L25_bool_binop_done:;

            /* "supplychainpy/demand/_evo_algo.pyx":187
 *                                 genome_count += 1
 * 
 *                         elif (12 <= genome_count <= 14 and len(new_individual_two) < 12) or (             # <<<<<<<<<<<<<<
 *                                         3 <= genome_count <= 8) and (len(
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 */
            if (__pyx_t_6) {

              /* "supplychainpy/demand/_evo_algo.pyx":190
 *                                         3 <= genome_count <= 8) and (len(
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
              __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_cur_scope->__pyx_v_mutation_index); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 190, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_14 = PyObject_RichCompare(__pyx_t_7, __pyx_cur_scope->__pyx_v_number_of_mutations_allowed, Py_EQ); __Pyx_XGOTREF(__pyx_t_14); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 190, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_14); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 190, __pyx_L4_error)
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (__pyx_t_6) {

                /* "supplychainpy/demand/_evo_algo.pyx":191
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 *                                 mutation_index = 0
 */
                __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 191, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                __pyx_t_23 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_mutation); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 191, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_23);
                __pyx_t_11 = PyDict_New(); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 191, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_11);
                if (PyDict_SetItem(__pyx_t_11, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 191, __pyx_L4_error)
                __pyx_t_21 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_23))) {
                  __pyx_t_21 = PyMethod_GET_SELF(__pyx_t_23);
                  if (likely(__pyx_t_21)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_23);
                    __Pyx_INCREF(__pyx_t_21);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_23, function);
                  }
                }
                if (!__pyx_t_21) {
                  __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_t_23, __pyx_t_11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 191, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  __Pyx_GOTREF(__pyx_t_15);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_23)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_21, __pyx_t_11};
                    __pyx_t_15 = __Pyx_PyFunction_FastCall(__pyx_t_23, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_23)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_21, __pyx_t_11};
                    __pyx_t_15 = __Pyx_PyCFunction_FastCall(__pyx_t_23, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_22 = PyTuple_New(1+1); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_22);
                    __Pyx_GIVEREF(__pyx_t_21); PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_t_21); __pyx_t_21 = NULL;
                    __Pyx_GIVEREF(__pyx_t_11);
                    PyTuple_SET_ITEM(__pyx_t_22, 0+1, __pyx_t_11);
                    __pyx_t_11 = 0;
                    __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_23, __pyx_t_22, NULL); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                __pyx_t_23 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                  __pyx_t_23 = PyMethod_GET_SELF(__pyx_t_7);
                  if (likely(__pyx_t_23)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                    __Pyx_INCREF(__pyx_t_23);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_7, function);
                  }
                }
                if (!__pyx_t_23) {
                  __pyx_t_14 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 191, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  __Pyx_GOTREF(__pyx_t_14);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_23, __pyx_t_15};
                    __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_23, __pyx_t_15};
                    __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_22 = PyTuple_New(1+1); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_22);
                    __Pyx_GIVEREF(__pyx_t_23); PyTuple_SET_ITEM(__pyx_t_22, 0, __pyx_t_23); __pyx_t_23 = NULL;
                    __Pyx_GIVEREF(__pyx_t_15);
                    PyTuple_SET_ITEM(__pyx_t_22, 0+1, __pyx_t_15);
                    __pyx_t_15 = 0;
                    __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_22, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 191, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":192
 *                             if mutation_index == number_of_mutations_allowed:
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 *                                 mutation_index = 0
 *                                 mutation_count += 1
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":193
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 *                                 mutation_index = 0             # <<<<<<<<<<<<<<
 *                                 mutation_count += 1
 *                             else:
 */
                __pyx_cur_scope->__pyx_v_mutation_index = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":194
 *                                 genome_count += 1
 *                                 mutation_index = 0
 *                                 mutation_count += 1             # <<<<<<<<<<<<<<
 *                             else:
 *                                 new_individual.update({key: value})
 */
                __pyx_cur_scope->__pyx_v_mutation_count = (__pyx_cur_scope->__pyx_v_mutation_count + 1);

                /* "supplychainpy/demand/_evo_algo.pyx":190
 *                                         3 <= genome_count <= 8) and (len(
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 *                             if mutation_index == number_of_mutations_allowed:             # <<<<<<<<<<<<<<
 *                                 new_individual.update(self._mutation({key: value}))
 *                                 genome_count += 1
 */
                goto __pyx_L30;
              }

              /* "supplychainpy/demand/_evo_algo.pyx":196
 *                                 mutation_count += 1
 *                             else:
 *                                 new_individual.update({key: value})             # <<<<<<<<<<<<<<
 *                                 genome_count += 1
 *                         else:
 */
              /*else*/ {
                __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 196, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_7);
                __pyx_t_22 = PyDict_New(); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 196, __pyx_L4_error)
                __Pyx_GOTREF(__pyx_t_22);
                if (PyDict_SetItem(__pyx_t_22, __pyx_cur_scope->__pyx_v_key, __pyx_cur_scope->__pyx_v_value) < 0) __PYX_ERR(0, 196, __pyx_L4_error)
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_7, function);
                  }
                }
                if (!__pyx_t_15) {
                  __pyx_t_14 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_22); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 196, __pyx_L4_error)
                  __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  __Pyx_GOTREF(__pyx_t_14);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_22};
                    __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 196, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_t_22};
                    __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 196, __pyx_L4_error)
                    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  } else
                  #endif
                  {
                    __pyx_t_23 = PyTuple_New(1+1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 196, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_23);
                    __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_23, 0, __pyx_t_15); __pyx_t_15 = NULL;
                    __Pyx_GIVEREF(__pyx_t_22);
                    PyTuple_SET_ITEM(__pyx_t_23, 0+1, __pyx_t_22);
                    __pyx_t_22 = 0;
                    __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_23, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 196, __pyx_L4_error)
                    __Pyx_GOTREF(__pyx_t_14);
                    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

                /* "supplychainpy/demand/_evo_algo.pyx":197
 *                             else:
 *                                 new_individual.update({key: value})
 *                                 genome_count += 1             # <<<<<<<<<<<<<<
 *                         else:
 *                             genome_count += 1
 */
                __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);
              }
              __pyx_L30:;

              /* "supplychainpy/demand/_evo_algo.pyx":187
 *                                 genome_count += 1
 * 
 *                         elif (12 <= genome_count <= 14 and len(new_individual_two) < 12) or (             # <<<<<<<<<<<<<<
 *                                         3 <= genome_count <= 8) and (len(
 *                             new_individual) < 12 or 21 <= genome_count <= 23):
 */
              goto __pyx_L19;
            }

            /* "supplychainpy/demand/_evo_algo.pyx":199
 *                                 genome_count += 1
 *                         else:
 *                             genome_count += 1             # <<<<<<<<<<<<<<
 * 
 *                         if genome_count == 24:
 */
            /*else*/ {
              __pyx_cur_scope->__pyx_v_genome_count = (__pyx_cur_scope->__pyx_v_genome_count + 1);
            }
            __pyx_L19:;

            /* "supplychainpy/demand/_evo_algo.pyx":201
 *                             genome_count += 1
 * 
 *                         if genome_count == 24:             # <<<<<<<<<<<<<<
 *                             genome_count = 0
 *                             # if new_individual != new_individual_two:
 */
            __pyx_t_6 = ((__pyx_cur_scope->__pyx_v_genome_count == 24) != 0);
            if (__pyx_t_6) {

              /* "supplychainpy/demand/_evo_algo.pyx":202
 * 
 *                         if genome_count == 24:
 *                             genome_count = 0             # <<<<<<<<<<<<<<
 *                             # if new_individual != new_individual_two:
 *                             # log.debug('single point crossover was successful')
 */
              __pyx_cur_scope->__pyx_v_genome_count = 0;

              /* "supplychainpy/demand/_evo_algo.pyx":207
 *                             # else:
 *                             # log.debug('single point crossover resulted in clone of parent')
 *                             yield (new_individual)             # <<<<<<<<<<<<<<
 *                             yield (new_individual_two)
 * 
 */
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_new_individual);
              __pyx_r = __pyx_cur_scope->__pyx_v_new_individual;
              __Pyx_XGIVEREF(__pyx_t_1);
              __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
              __Pyx_XGIVEREF(__pyx_t_2);
              __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
              __Pyx_XGIVEREF(__pyx_t_3);
              __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
              __Pyx_XGIVEREF(__pyx_t_4);
              __pyx_cur_scope->__pyx_t_3 = __pyx_t_4;
              __pyx_cur_scope->__pyx_t_4 = __pyx_t_5;
              __Pyx_XGIVEREF(__pyx_t_8);
              __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
              __pyx_cur_scope->__pyx_t_6 = __pyx_t_9;
              __Pyx_XGIVEREF(__pyx_t_10);
              __pyx_cur_scope->__pyx_t_7 = __pyx_t_10;
              __pyx_cur_scope->__pyx_t_8 = __pyx_t_12;
              __pyx_cur_scope->__pyx_t_9 = __pyx_t_13;
              __Pyx_XGIVEREF(__pyx_r);
              __Pyx_RefNannyFinishContext();
              /* return from generator, yielding value */
              __pyx_generator->resume_label = 1;
              return __pyx_r;
              __pyx_L32_resume_from_yield:;
              __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
              __pyx_cur_scope->__pyx_t_0 = 0;
              __Pyx_XGOTREF(__pyx_t_1);
              __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
              __pyx_cur_scope->__pyx_t_1 = 0;
              __Pyx_XGOTREF(__pyx_t_2);
              __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
              __pyx_cur_scope->__pyx_t_2 = 0;
              __Pyx_XGOTREF(__pyx_t_3);
              __pyx_t_4 = __pyx_cur_scope->__pyx_t_3;
              __pyx_cur_scope->__pyx_t_3 = 0;
              __Pyx_XGOTREF(__pyx_t_4);
              __pyx_t_5 = __pyx_cur_scope->__pyx_t_4;
              __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
              __pyx_cur_scope->__pyx_t_5 = 0;
              __Pyx_XGOTREF(__pyx_t_8);
              __pyx_t_9 = __pyx_cur_scope->__pyx_t_6;
              __pyx_t_10 = __pyx_cur_scope->__pyx_t_7;
              __pyx_cur_scope->__pyx_t_7 = 0;
              __Pyx_XGOTREF(__pyx_t_10);
              __pyx_t_12 = __pyx_cur_scope->__pyx_t_8;
              __pyx_t_13 = __pyx_cur_scope->__pyx_t_9;
              if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 207, __pyx_L4_error)

              /* "supplychainpy/demand/_evo_algo.pyx":208
 *                             # log.debug('single point crossover resulted in clone of parent')
 *                             yield (new_individual)
 *                             yield (new_individual_two)             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __Pyx_INCREF(__pyx_cur_scope->__pyx_v_new_individual_two);
              __pyx_r = __pyx_cur_scope->__pyx_v_new_individual_two;
              __Pyx_XGIVEREF(__pyx_t_1);
              __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
              __Pyx_XGIVEREF(__pyx_t_2);
              __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
              __Pyx_XGIVEREF(__pyx_t_3);
              __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
              __Pyx_XGIVEREF(__pyx_t_4);
              __pyx_cur_scope->__pyx_t_3 = __pyx_t_4;
              __pyx_cur_scope->__pyx_t_4 = __pyx_t_5;
              __Pyx_XGIVEREF(__pyx_t_8);
              __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
              __pyx_cur_scope->__pyx_t_6 = __pyx_t_9;
              __Pyx_XGIVEREF(__pyx_t_10);
              __pyx_cur_scope->__pyx_t_7 = __pyx_t_10;
              __pyx_cur_scope->__pyx_t_8 = __pyx_t_12;
              __pyx_cur_scope->__pyx_t_9 = __pyx_t_13;
              __Pyx_XGIVEREF(__pyx_r);
              __Pyx_RefNannyFinishContext();
              /* return from generator, yielding value */
              __pyx_generator->resume_label = 2;
              return __pyx_r;
              __pyx_L33_resume_from_yield:;
              __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
              __pyx_cur_scope->__pyx_t_0 = 0;
              __Pyx_XGOTREF(__pyx_t_1);
              __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
              __pyx_cur_scope->__pyx_t_1 = 0;
              __Pyx_XGOTREF(__pyx_t_2);
              __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
              __pyx_cur_scope->__pyx_t_2 = 0;
              __Pyx_XGOTREF(__pyx_t_3);
              __pyx_t_4 = __pyx_cur_scope->__pyx_t_3;
              __pyx_cur_scope->__pyx_t_3 = 0;
              __Pyx_XGOTREF(__pyx_t_4);
              __pyx_t_5 = __pyx_cur_scope->__pyx_t_4;
              __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
              __pyx_cur_scope->__pyx_t_5 = 0;
              __Pyx_XGOTREF(__pyx_t_8);
              __pyx_t_9 = __pyx_cur_scope->__pyx_t_6;
              __pyx_t_10 = __pyx_cur_scope->__pyx_t_7;
              __pyx_cur_scope->__pyx_t_7 = 0;
              __Pyx_XGOTREF(__pyx_t_10);
              __pyx_t_12 = __pyx_cur_scope->__pyx_t_8;
              __pyx_t_13 = __pyx_cur_scope->__pyx_t_9;
              if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 208, __pyx_L4_error)

              /* "supplychainpy/demand/_evo_algo.pyx":201
 *                             genome_count += 1
 * 
 *                         if genome_count == 24:             # <<<<<<<<<<<<<<
 *                             genome_count = 0
 *                             # if new_individual != new_individual_two:
 */
            }

            /* "supplychainpy/demand/_evo_algo.pyx":172
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():             # <<<<<<<<<<<<<<
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 *                         number_of_mutations_allowed = round(population_allele_count * self.mutation_probability)
 */
          }
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "supplychainpy/demand/_evo_algo.pyx":171
 *         try:
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):             # <<<<<<<<<<<<<<
 *                     for key, value in individual.items():
 *                         population_allele_count = len(self.individuals) * len(individual.items())
 */
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "supplychainpy/demand/_evo_algo.pyx":170
 * 
 *         try:
 *             if len(self.individuals) > 1:             # <<<<<<<<<<<<<<
 *                 for index, individual in enumerate(self.individuals):
 *                     for key, value in individual.items():
 */
      }

      /* "supplychainpy/demand/_evo_algo.pyx":169
 *         cdef dict new_individual_two = {}
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 */
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_PyThreadState_assign
    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
    __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":212
 * 
 * 
 *         except OSError as e:             # <<<<<<<<<<<<<<
 * 
 *             print(e)
 */
    __pyx_t_24 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_OSError);
    if (__pyx_t_24) {
      __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population._two_point_crossover_recombination", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_10) < 0) __PYX_ERR(0, 212, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      __pyx_cur_scope->__pyx_v_e = __pyx_t_8;

      /* "supplychainpy/demand/_evo_algo.pyx":214
 *         except OSError as e:
 * 
 *             print(e)             # <<<<<<<<<<<<<<
 * 
 *     # TODO-feature uniform crossover recombination for reproduction
 */
      if (__Pyx_PrintOne(0, __pyx_cur_scope->__pyx_v_e) < 0) __PYX_ERR(0, 214, __pyx_L6_except_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "supplychainpy/demand/_evo_algo.pyx":169
 *         cdef dict new_individual_two = {}
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if len(self.individuals) > 1:
 *                 for index, individual in enumerate(self.individuals):
 */
    __Pyx_PyThreadState_assign
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_PyThreadState_assign
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    __pyx_L11_try_end:;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":152
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')
 * 
 *     def _two_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on two points along the chromosome.
 * 
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_AddTraceback("_two_point_crossover_recombination", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":217
 * 
 *     # TODO-feature uniform crossover recombination for reproduction
 *     def _uniform_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_12_uniform_crossover_recombination(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_12_uniform_crossover_recombination = {"_uniform_crossover_recombination", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_12_uniform_crossover_recombination, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_12_uniform_crossover_recombination(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_uniform_crossover_recombination (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_11_uniform_crossover_recombination(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_11_uniform_crossover_recombination(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_uniform_crossover_recombination", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":221
 * 
 *     @staticmethod
 *     def _mutation(gene: dict):             # <<<<<<<<<<<<<<
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 *         by having the bit representing it flipped.
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_14_mutation(PyObject *__pyx_self, PyObject *__pyx_v_gene); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_13_mutation[] = "Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered\n        by having the bit representing it flipped.\n\n        Args:\n            gene:   The alpha value and bit representing the positive or negative fit of the gene.\n\n        Returns:\n            dict:   The new gene with an altered value for the it representing the positive or negative fit of the gene.\n\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_14_mutation = {"_mutation", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_14_mutation, METH_O, __pyx_doc_13supplychainpy_6demand_9_evo_algo_10Population_13_mutation};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_14_mutation(PyObject *__pyx_self, PyObject *__pyx_v_gene) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_mutation (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_13_mutation(__pyx_self, ((PyObject *)__pyx_v_gene));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":235
 *         original_gene = [[k, v] for k, v in gene.items()]
 * 
 *         def mutate():             # <<<<<<<<<<<<<<
 * 
 *             if original_gene[0][1] == 0:
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_1mutate(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_1mutate = {"mutate", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_1mutate, METH_NOARGS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_1mutate(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mutate (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_mutate(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_mutate(PyObject *__pyx_self) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *__pyx_cur_scope;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("mutate", 0);
  __pyx_outer_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "supplychainpy/demand/_evo_algo.pyx":237
 *         def mutate():
 * 
 *             if original_gene[0][1] == 0:             # <<<<<<<<<<<<<<
 *                 return {original_gene[0][0]: 1}
 *             elif original_gene[0][1] == 1:
 */
  if (unlikely(!__pyx_cur_scope->__pyx_v_original_gene)) { __Pyx_RaiseClosureNameError("original_gene"); __PYX_ERR(0, 237, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_original_gene == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 237, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_cur_scope->__pyx_v_original_gene, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_2, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":238
 * 
 *             if original_gene[0][1] == 0:
 *                 return {original_gene[0][0]: 1}             # <<<<<<<<<<<<<<
 *             elif original_gene[0][1] == 1:
 *                 return {original_gene[0][0]: 0}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(!__pyx_cur_scope->__pyx_v_original_gene)) { __Pyx_RaiseClosureNameError("original_gene"); __PYX_ERR(0, 238, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_original_gene == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 238, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_cur_scope->__pyx_v_original_gene, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_1, __pyx_t_4, __pyx_int_1) < 0) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "supplychainpy/demand/_evo_algo.pyx":237
 *         def mutate():
 * 
 *             if original_gene[0][1] == 0:             # <<<<<<<<<<<<<<
 *                 return {original_gene[0][0]: 1}
 *             elif original_gene[0][1] == 1:
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":239
 *             if original_gene[0][1] == 0:
 *                 return {original_gene[0][0]: 1}
 *             elif original_gene[0][1] == 1:             # <<<<<<<<<<<<<<
 *                 return {original_gene[0][0]: 0}
 *             else:
 */
  if (unlikely(!__pyx_cur_scope->__pyx_v_original_gene)) { __Pyx_RaiseClosureNameError("original_gene"); __PYX_ERR(0, 239, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_original_gene == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 239, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_cur_scope->__pyx_v_original_gene, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_4, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":240
 *                 return {original_gene[0][0]: 1}
 *             elif original_gene[0][1] == 1:
 *                 return {original_gene[0][0]: 0}             # <<<<<<<<<<<<<<
 *             else:
 *                 return {original_gene[0][0]: original_gene[0][0]}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(!__pyx_cur_scope->__pyx_v_original_gene)) { __Pyx_RaiseClosureNameError("original_gene"); __PYX_ERR(0, 240, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_original_gene == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 240, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_cur_scope->__pyx_v_original_gene, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (PyDict_SetItem(__pyx_t_1, __pyx_t_2, __pyx_int_0) < 0) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "supplychainpy/demand/_evo_algo.pyx":239
 *             if original_gene[0][1] == 0:
 *                 return {original_gene[0][0]: 1}
 *             elif original_gene[0][1] == 1:             # <<<<<<<<<<<<<<
 *                 return {original_gene[0][0]: 0}
 *             else:
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":242
 *                 return {original_gene[0][0]: 0}
 *             else:
 *                 return {original_gene[0][0]: original_gene[0][0]}             # <<<<<<<<<<<<<<
 * 
 *         return mutate()
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(!__pyx_cur_scope->__pyx_v_original_gene)) { __Pyx_RaiseClosureNameError("original_gene"); __PYX_ERR(0, 242, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_original_gene == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 242, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_cur_scope->__pyx_v_original_gene, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_cur_scope->__pyx_v_original_gene)) { __Pyx_RaiseClosureNameError("original_gene"); __PYX_ERR(0, 242, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_original_gene == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 242, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_cur_scope->__pyx_v_original_gene, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (PyDict_SetItem(__pyx_t_1, __pyx_t_4, __pyx_t_5) < 0) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "supplychainpy/demand/_evo_algo.pyx":235
 *         original_gene = [[k, v] for k, v in gene.items()]
 * 
 *         def mutate():             # <<<<<<<<<<<<<<
 * 
 *             if original_gene[0][1] == 0:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population._mutation.mutate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":221
 * 
 *     @staticmethod
 *     def _mutation(gene: dict):             # <<<<<<<<<<<<<<
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 *         by having the bit representing it flipped.
 */

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_13_mutation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_gene) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *__pyx_cur_scope;
  PyObject *__pyx_v_mutate = 0;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  __Pyx_RefNannySetupContext("_mutation", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 221, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }

  /* "supplychainpy/demand/_evo_algo.pyx":233
 *         """
 * 
 *         original_gene = [[k, v] for k, v in gene.items()]             # <<<<<<<<<<<<<<
 * 
 *         def mutate():
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_gene, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 233, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 233, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 233, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_6(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 233, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 233, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_7);
      #else
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 233, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 233, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 233, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
      index = 0; __pyx_t_4 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
      __pyx_t_9 = NULL;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 233, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_k);
    __Pyx_GIVEREF(__pyx_v_k);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_k);
    __Pyx_INCREF(__pyx_v_v);
    __Pyx_GIVEREF(__pyx_v_v);
    PyList_SET_ITEM(__pyx_t_2, 1, __pyx_v_v);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_original_gene = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":235
 *         original_gene = [[k, v] for k, v in gene.items()]
 * 
 *         def mutate():             # <<<<<<<<<<<<<<
 * 
 *             if original_gene[0][1] == 0:
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_1mutate, 0, __pyx_n_s_Population__mutation_locals_muta, ((PyObject*)__pyx_cur_scope), __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__5)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_mutate = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":244
 *                 return {original_gene[0][0]: original_gene[0][0]}
 * 
 *         return mutate()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_pf_13supplychainpy_6demand_9_evo_algo_10Population_9_mutation_mutate(__pyx_v_mutate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":221
 * 
 *     @staticmethod
 *     def _mutation(gene: dict):             # <<<<<<<<<<<<<<
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 *         by having the bit representing it flipped.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.Population._mutation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mutate);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":251
 *     __generation = 0
 * 
 *     def __init__(self, orders: list, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__orders = orders
 *         if len(kwargs) != 0:
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_1__init__ = {"__init__", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_orders = 0;
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_orders,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_orders)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 251, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 251, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_orders = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 251, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm___init__(__pyx_self, __pyx_v_self, __pyx_v_orders, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_orders, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":252
 * 
 *     def __init__(self, orders: list, **kwargs):
 *         self.__orders = orders             # <<<<<<<<<<<<<<
 *         if len(kwargs) != 0:
 *             self.__average_order = kwargs['average_order']
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl, __pyx_v_orders) < 0) __PYX_ERR(0, 252, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":253
 *     def __init__(self, orders: list, **kwargs):
 *         self.__orders = orders
 *         if len(kwargs) != 0:             # <<<<<<<<<<<<<<
 *             self.__average_order = kwargs['average_order']
 *             self.__population_size = kwargs['population_size']
 */
  __pyx_t_1 = PyDict_Size(__pyx_v_kwargs); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 253, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 != 0) != 0);
  if (__pyx_t_2) {

    /* "supplychainpy/demand/_evo_algo.pyx":254
 *         self.__orders = orders
 *         if len(kwargs) != 0:
 *             self.__average_order = kwargs['average_order']             # <<<<<<<<<<<<<<
 *             self.__population_size = kwargs['population_size']
 *             self.__standard_error = kwargs['standard_error']
 */
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_kwargs, __pyx_n_s_average_order); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_2, __pyx_t_3) < 0) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":255
 *         if len(kwargs) != 0:
 *             self.__average_order = kwargs['average_order']
 *             self.__population_size = kwargs['population_size']             # <<<<<<<<<<<<<<
 *             self.__standard_error = kwargs['standard_error']
 *             self.__recombination_type = kwargs['recombination_type']
 */
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_kwargs, __pyx_n_s_population_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_3, __pyx_t_3) < 0) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":256
 *             self.__average_order = kwargs['average_order']
 *             self.__population_size = kwargs['population_size']
 *             self.__standard_error = kwargs['standard_error']             # <<<<<<<<<<<<<<
 *             self.__recombination_type = kwargs['recombination_type']
 * 
 */
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_kwargs, __pyx_n_s_standard_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_4, __pyx_t_3) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":257
 *             self.__population_size = kwargs['population_size']
 *             self.__standard_error = kwargs['standard_error']
 *             self.__recombination_type = kwargs['recombination_type']             # <<<<<<<<<<<<<<
 * 
 *     def initial_population(self, individual_type = 'ses'):
 */
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_kwargs, __pyx_n_s_recombination_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_5, __pyx_t_3) < 0) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":253
 *     def __init__(self, orders: list, **kwargs):
 *         self.__orders = orders
 *         if len(kwargs) != 0:             # <<<<<<<<<<<<<<
 *             self.__average_order = kwargs['average_order']
 *             self.__population_size = kwargs['population_size']
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":251
 *     __generation = 0
 * 
 *     def __init__(self, orders: list, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__orders = orders
 *         if len(kwargs) != 0:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":259
 *             self.__recombination_type = kwargs['recombination_type']
 * 
 *     def initial_population(self, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 *         """Initialises population and initiates the optimisation of the standard error by searching for an optimum
 *         alpha value.
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_3initial_population(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_2initial_population[] = "Initialises population and initiates the optimisation of the standard error by searching for an optimum\n        alpha value.\n        Returns:\n\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_3initial_population = {"initial_population", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_3initial_population, METH_VARARGS|METH_KEYWORDS, __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_2initial_population};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_3initial_population(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_individual_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initial_population (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individual_type,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_n_s_ses));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individual_type);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "initial_population") < 0)) __PYX_ERR(0, 259, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_individual_type = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("initial_population", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 259, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.initial_population", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_2initial_population(__pyx_self, __pyx_v_self, __pyx_v_individual_type);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_2initial_population(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("initial_population", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":265
 * 
 *         """
 *         return self._initialise_smoothing_level_evolutionary_algorithm_population(individual_type=individual_type)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_initialise_smoothing_level_evol); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_individual_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 265, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":259
 *             self.__recombination_type = kwargs['recombination_type']
 * 
 *     def initial_population(self, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 *         """Initialises population and initiates the optimisation of the standard error by searching for an optimum
 *         alpha value.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.initial_population", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":268
 * 
 *     @property
 *     def population_size(self):             # <<<<<<<<<<<<<<
 *         return self.__population_size
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_5population_size(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_5population_size = {"population_size", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_5population_size, METH_O, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_5population_size(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("population_size (wrapper)", 0);
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_4population_size(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_4population_size(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("population_size", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":269
 *     @property
 *     def population_size(self):
 *         return self.__population_size             # <<<<<<<<<<<<<<
 * 
 *     @population_size.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":268
 * 
 *     @property
 *     def population_size(self):             # <<<<<<<<<<<<<<
 *         return self.__population_size
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.population_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":272
 * 
 *     @population_size.setter
 *     def population_size(self, population_size):             # <<<<<<<<<<<<<<
 *         self.__population_size = population_size
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_7population_size(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_7population_size = {"population_size", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_7population_size, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_7population_size(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_population_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("population_size (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_population_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_population_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("population_size", 1, 2, 2, 1); __PYX_ERR(0, 272, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "population_size") < 0)) __PYX_ERR(0, 272, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_population_size = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("population_size", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 272, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.population_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_6population_size(__pyx_self, __pyx_v_self, __pyx_v_population_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_6population_size(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_population_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("population_size", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":273
 *     @population_size.setter
 *     def population_size(self, population_size):
 *         self.__population_size = population_size             # <<<<<<<<<<<<<<
 * 
 *     def _initialise_smoothing_level_evolutionary_algorithm_population(self, individual_type: str):
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_3, __pyx_v_population_size) < 0) __PYX_ERR(0, 273, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":272
 * 
 *     @population_size.setter
 *     def population_size(self, population_size):             # <<<<<<<<<<<<<<
 *         self.__population_size = population_size
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.population_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":275
 *         self.__population_size = population_size
 * 
 *     def _initialise_smoothing_level_evolutionary_algorithm_population(self, individual_type: str):             # <<<<<<<<<<<<<<
 *         """ Starts the process for creating the population. The number of parents is specified during the
 *          initialisation of the class. """
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_9_initialise_smoothing_level_evolutionary_algorithm_population(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_8_initialise_smoothing_level_evolutionary_algorithm_population[] = " Starts the process for creating the population. The number of parents is specified during the\n         initialisation of the class. ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_9_initialise_smoothing_level_evolutionary_algorithm_population = {"_initialise_smoothing_level_evolutionary_algorithm_population", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_9_initialise_smoothing_level_evolutionary_algorithm_population, METH_VARARGS|METH_KEYWORDS, __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_8_initialise_smoothing_level_evolutionary_algorithm_population};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_9_initialise_smoothing_level_evolutionary_algorithm_population(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_individual_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_initialise_smoothing_level_evolutionary_algorithm_population (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individual_type,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individual_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_initialise_smoothing_level_evolutionary_algorithm_population", 1, 2, 2, 1); __PYX_ERR(0, 275, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_initialise_smoothing_level_evolutionary_algorithm_population") < 0)) __PYX_ERR(0, 275, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_individual_type = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_initialise_smoothing_level_evolutionary_algorithm_population", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 275, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._initialise_smoothing_level_evolutionary_algorithm_population", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_8_initialise_smoothing_level_evolutionary_algorithm_population(__pyx_self, __pyx_v_self, __pyx_v_individual_type);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_8_initialise_smoothing_level_evolutionary_algorithm_population(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual_type) {
  PyObject *__pyx_v_parents = 0;
  PyObject *__pyx_v_parents_population = 0;
  PyObject *__pyx_v_populations_genome = 0;
  PyObject *__pyx_v_populations_traits = 0;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_parent = NULL;
  PyObject *__pyx_v_fit_population = NULL;
  PyObject *__pyx_v_create_offspring = NULL;
  PyObject *__pyx_v_new_population = 0;
  PyObject *__pyx_v_parent_offspring_population = 0;
  PyObject *__pyx_v_new_individuals = 0;
  PyObject *__pyx_v_new_populations_traits = 0;
  PyObject *__pyx_v_new_fit_population = 0;
  PyObject *__pyx_v_final_error = 0;
  PyObject *__pyx_v_minimum_smoothing_level = 0;
  PyObject *__pyx_v_po = NULL;
  PyObject *__pyx_v_pke = NULL;
  PyObject *__pyx_v_genome = NULL;
  PyObject *__pyx_v_new_individual = NULL;
  PyObject *__pyx_v_new_population_genome = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("_initialise_smoothing_level_evolutionary_algorithm_population", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":279
 *          initialisation of the class. """
 * 
 *         cdef list parents = []             # <<<<<<<<<<<<<<
 *         cdef list parents_population = []
 *         cdef list populations_genome = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_parents = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":280
 * 
 *         cdef list parents = []
 *         cdef list parents_population = []             # <<<<<<<<<<<<<<
 *         cdef list populations_genome = []
 *         cdef list populations_traits = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_parents_population = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":281
 *         cdef list parents = []
 *         cdef list parents_population = []
 *         cdef list populations_genome = []             # <<<<<<<<<<<<<<
 *         cdef list populations_traits = []
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_populations_genome = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":282
 *         cdef list parents_population = []
 *         cdef list populations_genome = []
 *         cdef list populations_traits = []             # <<<<<<<<<<<<<<
 * 
 *         while len(parents_population) < self.__population_size:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_populations_traits = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":284
 *         cdef list populations_traits = []
 * 
 *         while len(parents_population) < self.__population_size:             # <<<<<<<<<<<<<<
 *             for i in range(0, self.__population_size):
 *                 parent = Individual(name='parent', forecast_type=individual_type)
 */
  while (1) {
    __pyx_t_2 = PyList_GET_SIZE(__pyx_v_parents_population); if (unlikely(__pyx_t_2 == -1)) __PYX_ERR(0, 284, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!__pyx_t_5) break;

    /* "supplychainpy/demand/_evo_algo.pyx":285
 * 
 *         while len(parents_population) < self.__population_size:
 *             for i in range(0, self.__population_size):             # <<<<<<<<<<<<<<
 *                 parent = Individual(name='parent', forecast_type=individual_type)
 *                 parents.append(parent)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_int_0);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 285, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 285, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 285, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_6(__pyx_t_3);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 285, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":286
 *         while len(parents_population) < self.__population_size:
 *             for i in range(0, self.__population_size):
 *                 parent = Individual(name='parent', forecast_type=individual_type)             # <<<<<<<<<<<<<<
 *                 parents.append(parent)
 * 
 */
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_Individual); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_name, __pyx_n_s_parent) < 0) __PYX_ERR(0, 286, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_forecast_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 286, __pyx_L1_error)
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_parent, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":287
 *             for i in range(0, self.__population_size):
 *                 parent = Individual(name='parent', forecast_type=individual_type)
 *                 parents.append(parent)             # <<<<<<<<<<<<<<
 * 
 *             populations_genome = [i for i in self.generate_smoothing_level_genome(population=parents,
 */
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_parents, __pyx_v_parent); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 287, __pyx_L1_error)

      /* "supplychainpy/demand/_evo_algo.pyx":285
 * 
 *         while len(parents_population) < self.__population_size:
 *             for i in range(0, self.__population_size):             # <<<<<<<<<<<<<<
 *                 parent = Individual(name='parent', forecast_type=individual_type)
 *                 parents.append(parent)
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":289
 *                 parents.append(parent)
 * 
 *             populations_genome = [i for i in self.generate_smoothing_level_genome(population=parents,             # <<<<<<<<<<<<<<
 *                                                                                   individual_type=individual_type)]
 *             populations_traits = [i for i in self.express_smoothing_level_genome(individuals_genome=populations_genome,
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_smoothing_level_genome); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_population, __pyx_v_parents) < 0) __PYX_ERR(0, 289, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":290
 * 
 *             populations_genome = [i for i in self.generate_smoothing_level_genome(population=parents,
 *                                                                                   individual_type=individual_type)]             # <<<<<<<<<<<<<<
 *             populations_traits = [i for i in self.express_smoothing_level_genome(individuals_genome=populations_genome,
 *                                                                                  standard_error=self.__standard_error)]
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_individual_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 289, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":289
 *                 parents.append(parent)
 * 
 *             populations_genome = [i for i in self.generate_smoothing_level_genome(population=parents,             # <<<<<<<<<<<<<<
 *                                                                                   individual_type=individual_type)]
 *             populations_traits = [i for i in self.express_smoothing_level_genome(individuals_genome=populations_genome,
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 289, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 289, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 289, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 289, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 289, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_6(__pyx_t_1);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 289, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 289, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_populations_genome, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":291
 *             populations_genome = [i for i in self.generate_smoothing_level_genome(population=parents,
 *                                                                                   individual_type=individual_type)]
 *             populations_traits = [i for i in self.express_smoothing_level_genome(individuals_genome=populations_genome,             # <<<<<<<<<<<<<<
 *                                                                                  standard_error=self.__standard_error)]
 * 
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_express_smoothing_level_genome); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_individuals_genome, __pyx_v_populations_genome) < 0) __PYX_ERR(0, 291, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":292
 *                                                                                   individual_type=individual_type)]
 *             populations_traits = [i for i in self.express_smoothing_level_genome(individuals_genome=populations_genome,
 *                                                                                  standard_error=self.__standard_error)]             # <<<<<<<<<<<<<<
 * 
 *             fit_population = [i for i in
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_standard_error, __pyx_t_7) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":291
 *             populations_genome = [i for i in self.generate_smoothing_level_genome(population=parents,
 *                                                                                   individual_type=individual_type)]
 *             populations_traits = [i for i in self.express_smoothing_level_genome(individuals_genome=populations_genome,             # <<<<<<<<<<<<<<
 *                                                                                  standard_error=self.__standard_error)]
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
      __pyx_t_4 = __pyx_t_7; __Pyx_INCREF(__pyx_t_4); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 291, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 291, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_2); __Pyx_INCREF(__pyx_t_7); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 291, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 291, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_2); __Pyx_INCREF(__pyx_t_7); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 291, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 291, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_6(__pyx_t_4);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 291, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
      __pyx_t_7 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 291, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_populations_traits, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":294
 *                                                                                  standard_error=self.__standard_error)]
 * 
 *             fit_population = [i for i in             # <<<<<<<<<<<<<<
 *                               self._population_fitness(population=populations_traits, individual_type=individual_type)]
 * 
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "supplychainpy/demand/_evo_algo.pyx":295
 * 
 *             fit_population = [i for i in
 *                               self._population_fitness(population=populations_traits, individual_type=individual_type)]             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_population_fitness); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_population, __pyx_v_populations_traits) < 0) __PYX_ERR(0, 295, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_individual_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 295, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_7 = __pyx_t_1; __Pyx_INCREF(__pyx_t_7); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 295, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 295, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_7))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 295, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 295, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_6(__pyx_t_7);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 295, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":294
 *                                                                                  standard_error=self.__standard_error)]
 * 
 *             fit_population = [i for i in             # <<<<<<<<<<<<<<
 *                               self._population_fitness(population=populations_traits, individual_type=individual_type)]
 * 
 */
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 294, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF_SET(__pyx_v_fit_population, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":298
 * 
 * 
 *             parents_population += fit_population             # <<<<<<<<<<<<<<
 * 
 *         create_offspring = Population(individuals=parents_population)
 */
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_parents_population, __pyx_v_fit_population); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 298, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_parents_population, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;
  }

  /* "supplychainpy/demand/_evo_algo.pyx":300
 *             parents_population += fit_population
 * 
 *         create_offspring = Population(individuals=parents_population)             # <<<<<<<<<<<<<<
 * 
 *         cdef list new_population = []
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_Population); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_individuals, __pyx_v_parents_population) < 0) __PYX_ERR(0, 300, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_create_offspring = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":302
 *         create_offspring = Population(individuals=parents_population)
 * 
 *         cdef list new_population = []             # <<<<<<<<<<<<<<
 *         # population reproduce
 *         new_population = [i for i in create_offspring.reproduce(recombination_type=self.__recombination_type)]
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_population = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":304
 *         cdef list new_population = []
 *         # population reproduce
 *         new_population = [i for i in create_offspring.reproduce(recombination_type=self.__recombination_type)]             # <<<<<<<<<<<<<<
 * 
 *         if new_population is None:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_create_offspring, __pyx_n_s_reproduce); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_recombination_type, __pyx_t_4) < 0) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
    __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 304, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 304, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 304, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_6(__pyx_t_3);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 304, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 304, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_new_population, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":306
 *         new_population = [i for i in create_offspring.reproduce(recombination_type=self.__recombination_type)]
 * 
 *         if new_population is None:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  __pyx_t_5 = (__pyx_v_new_population == ((PyObject*)Py_None));
  __pyx_t_9 = (__pyx_t_5 != 0);
  if (__pyx_t_9) {

    /* "supplychainpy/demand/_evo_algo.pyx":307
 * 
 *         if new_population is None:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *         cdef list parent_offspring_population = []
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_0);
    __pyx_r = __pyx_int_0;
    goto __pyx_L0;

    /* "supplychainpy/demand/_evo_algo.pyx":306
 *         new_population = [i for i in create_offspring.reproduce(recombination_type=self.__recombination_type)]
 * 
 *         if new_population is None:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":309
 *             return 0
 * 
 *         cdef list parent_offspring_population = []             # <<<<<<<<<<<<<<
 *         cdef list new_individuals = []
 *         cdef list new_populations_traits = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_parent_offspring_population = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":310
 * 
 *         cdef list parent_offspring_population = []
 *         cdef list new_individuals = []             # <<<<<<<<<<<<<<
 *         cdef list new_populations_traits = []
 *         cdef list new_fit_population = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_individuals = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":311
 *         cdef list parent_offspring_population = []
 *         cdef list new_individuals = []
 *         cdef list new_populations_traits = []             # <<<<<<<<<<<<<<
 *         cdef list new_fit_population = []
 *         cdef list final_error = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_populations_traits = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":312
 *         cdef list new_individuals = []
 *         cdef list new_populations_traits = []
 *         cdef list new_fit_population = []             # <<<<<<<<<<<<<<
 *         cdef list final_error = []
 *         cdef tuple minimum_smoothing_level
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_fit_population = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":313
 *         cdef list new_populations_traits = []
 *         cdef list new_fit_population = []
 *         cdef list final_error = []             # <<<<<<<<<<<<<<
 *         cdef tuple minimum_smoothing_level
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_final_error = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":316
 *         cdef tuple minimum_smoothing_level
 * 
 *         while len(new_population) < self.__population_size * 10:             # <<<<<<<<<<<<<<
 *             for po in new_population:
 *                 pke = po.keys()
 */
  while (1) {
    __pyx_t_2 = PyList_GET_SIZE(__pyx_v_new_population); if (unlikely(__pyx_t_2 == -1)) __PYX_ERR(0, 316, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_int_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_9) break;

    /* "supplychainpy/demand/_evo_algo.pyx":317
 * 
 *         while len(new_population) < self.__population_size * 10:
 *             for po in new_population:             # <<<<<<<<<<<<<<
 *                 pke = po.keys()
 *                 parent_offspring_population.append(tuple(pke))
 */
    __pyx_t_3 = __pyx_v_new_population; __Pyx_INCREF(__pyx_t_3); __pyx_t_2 = 0;
    for (;;) {
      if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 317, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_po, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":318
 *         while len(new_population) < self.__population_size * 10:
 *             for po in new_population:
 *                 pke = po.keys()             # <<<<<<<<<<<<<<
 *                 parent_offspring_population.append(tuple(pke))
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_po, __pyx_n_s_keys); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (__pyx_t_7) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else {
        __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_pke, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":319
 *             for po in new_population:
 *                 pke = po.keys()
 *                 parent_offspring_population.append(tuple(pke))             # <<<<<<<<<<<<<<
 * 
 *             for genome in parent_offspring_population:
 */
      __pyx_t_4 = PySequence_Tuple(__pyx_v_pke); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_parent_offspring_population, __pyx_t_4); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":317
 * 
 *         while len(new_population) < self.__population_size * 10:
 *             for po in new_population:             # <<<<<<<<<<<<<<
 *                 pke = po.keys()
 *                 parent_offspring_population.append(tuple(pke))
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":321
 *                 parent_offspring_population.append(tuple(pke))
 * 
 *             for genome in parent_offspring_population:             # <<<<<<<<<<<<<<
 *                 new_individual = Individual(overide=True)
 *                 new_individual.genome = genome
 */
    __pyx_t_3 = __pyx_v_parent_offspring_population; __Pyx_INCREF(__pyx_t_3); __pyx_t_2 = 0;
    for (;;) {
      if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 321, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_genome, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":322
 * 
 *             for genome in parent_offspring_population:
 *                 new_individual = Individual(overide=True)             # <<<<<<<<<<<<<<
 *                 new_individual.genome = genome
 *                 new_individuals.append(new_individual)
 */
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_Individual); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_overide, Py_True) < 0) __PYX_ERR(0, 322, __pyx_L1_error)
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_new_individual, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":323
 *             for genome in parent_offspring_population:
 *                 new_individual = Individual(overide=True)
 *                 new_individual.genome = genome             # <<<<<<<<<<<<<<
 *                 new_individuals.append(new_individual)
 * 
 */
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_new_individual, __pyx_n_s_genome_2, __pyx_v_genome) < 0) __PYX_ERR(0, 323, __pyx_L1_error)

      /* "supplychainpy/demand/_evo_algo.pyx":324
 *                 new_individual = Individual(overide=True)
 *                 new_individual.genome = genome
 *                 new_individuals.append(new_individual)             # <<<<<<<<<<<<<<
 * 
 *             # while population allele boundary ie 50 70 95 is less than specified number.
 */
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_new_individuals, __pyx_v_new_individual); if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 324, __pyx_L1_error)

      /* "supplychainpy/demand/_evo_algo.pyx":321
 *                 parent_offspring_population.append(tuple(pke))
 * 
 *             for genome in parent_offspring_population:             # <<<<<<<<<<<<<<
 *                 new_individual = Individual(overide=True)
 *                 new_individual.genome = genome
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":327
 * 
 *             # while population allele boundary ie 50 70 95 is less than specified number.
 *             new_population_genome = [i for i in self.generate_smoothing_level_genome(population=new_individuals,             # <<<<<<<<<<<<<<
 *                                                                                      individual_type=individual_type)]
 * 
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_smoothing_level_genome); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_population, __pyx_v_new_individuals) < 0) __PYX_ERR(0, 327, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":328
 *             # while population allele boundary ie 50 70 95 is less than specified number.
 *             new_population_genome = [i for i in self.generate_smoothing_level_genome(population=new_individuals,
 *                                                                                      individual_type=individual_type)]             # <<<<<<<<<<<<<<
 * 
 *             new_populations_traits = [i for i in
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_individual_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 327, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":327
 * 
 *             # while population allele boundary ie 50 70 95 is less than specified number.
 *             new_population_genome = [i for i in self.generate_smoothing_level_genome(population=new_individuals,             # <<<<<<<<<<<<<<
 *                                                                                      individual_type=individual_type)]
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 327, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 327, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 327, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_6(__pyx_t_1);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 327, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 327, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_population_genome, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":330
 *                                                                                      individual_type=individual_type)]
 * 
 *             new_populations_traits = [i for i in             # <<<<<<<<<<<<<<
 *                                       self.express_smoothing_level_genome(individuals_genome=new_population_genome,
 *                                                                           standard_error=self.__standard_error)]
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "supplychainpy/demand/_evo_algo.pyx":331
 * 
 *             new_populations_traits = [i for i in
 *                                       self.express_smoothing_level_genome(individuals_genome=new_population_genome,             # <<<<<<<<<<<<<<
 *                                                                           standard_error=self.__standard_error)]
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_express_smoothing_level_genome); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_individuals_genome, __pyx_v_new_population_genome) < 0) __PYX_ERR(0, 331, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":332
 *             new_populations_traits = [i for i in
 *                                       self.express_smoothing_level_genome(individuals_genome=new_population_genome,
 *                                                                           standard_error=self.__standard_error)]             # <<<<<<<<<<<<<<
 * 
 *             new_fit_population = [i for i in self._population_fitness(population=new_populations_traits,
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_standard_error, __pyx_t_7) < 0) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":331
 * 
 *             new_populations_traits = [i for i in
 *                                       self.express_smoothing_level_genome(individuals_genome=new_population_genome,             # <<<<<<<<<<<<<<
 *                                                                           standard_error=self.__standard_error)]
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
      __pyx_t_4 = __pyx_t_7; __Pyx_INCREF(__pyx_t_4); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 331, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_2); __Pyx_INCREF(__pyx_t_7); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 331, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_2); __Pyx_INCREF(__pyx_t_7); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 331, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_6(__pyx_t_4);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 331, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":330
 *                                                                                      individual_type=individual_type)]
 * 
 *             new_populations_traits = [i for i in             # <<<<<<<<<<<<<<
 *                                       self.express_smoothing_level_genome(individuals_genome=new_population_genome,
 *                                                                           standard_error=self.__standard_error)]
 */
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 330, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_new_populations_traits, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":334
 *                                                                           standard_error=self.__standard_error)]
 * 
 *             new_fit_population = [i for i in self._population_fitness(population=new_populations_traits,             # <<<<<<<<<<<<<<
 *                                                                       individual_type=individual_type)]
 *             new_population = new_fit_population
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_population_fitness); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_population, __pyx_v_new_populations_traits) < 0) __PYX_ERR(0, 334, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":335
 * 
 *             new_fit_population = [i for i in self._population_fitness(population=new_populations_traits,
 *                                                                       individual_type=individual_type)]             # <<<<<<<<<<<<<<
 *             new_population = new_fit_population
 * 
 */
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_individual_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 334, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":334
 *                                                                           standard_error=self.__standard_error)]
 * 
 *             new_fit_population = [i for i in self._population_fitness(population=new_populations_traits,             # <<<<<<<<<<<<<<
 *                                                                       individual_type=individual_type)]
 *             new_population = new_fit_population
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_7 = __pyx_t_1; __Pyx_INCREF(__pyx_t_7); __pyx_t_2 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 334, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 334, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_7))) {
          if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 334, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 334, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_6(__pyx_t_7);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 334, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
      __pyx_t_1 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 334, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF_SET(__pyx_v_new_fit_population, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":336
 *             new_fit_population = [i for i in self._population_fitness(population=new_populations_traits,
 *                                                                       individual_type=individual_type)]
 *             new_population = new_fit_population             # <<<<<<<<<<<<<<
 * 
 *         new_individuals.clear()
 */
    __Pyx_INCREF(__pyx_v_new_fit_population);
    __Pyx_DECREF_SET(__pyx_v_new_population, __pyx_v_new_fit_population);
  }

  /* "supplychainpy/demand/_evo_algo.pyx":338
 *             new_population = new_fit_population
 * 
 *         new_individuals.clear()             # <<<<<<<<<<<<<<
 * 
 *         new_individuals = [i for i in self.create_individuals(new_population)]
 */
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_new_individuals, __pyx_n_s_clear); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 338, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (__pyx_t_1) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 338, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":340
 *         new_individuals.clear()
 * 
 *         new_individuals = [i for i in self.create_individuals(new_population)]             # <<<<<<<<<<<<<<
 * 
 *         final_error = [i for i in self.generate_smoothing_level_genome(population=new_individuals,
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_create_individuals); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_new_population); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_new_population};
      __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_new_population};
      __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_new_population);
      __Pyx_GIVEREF(__pyx_v_new_population);
      PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_v_new_population);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
    __pyx_t_1 = __pyx_t_7; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 340, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_7); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 340, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_7); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 340, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_1);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 340, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
    __pyx_t_7 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 340, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_new_individuals, ((PyObject*)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":342
 *         new_individuals = [i for i in self.create_individuals(new_population)]
 * 
 *         final_error = [i for i in self.generate_smoothing_level_genome(population=new_individuals,             # <<<<<<<<<<<<<<
 *                                                                        individual_type=individual_type)]
 * 
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_smoothing_level_genome); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_population, __pyx_v_new_individuals) < 0) __PYX_ERR(0, 342, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":343
 * 
 *         final_error = [i for i in self.generate_smoothing_level_genome(population=new_individuals,
 *                                                                        individual_type=individual_type)]             # <<<<<<<<<<<<<<
 * 
 *         minimum_smoothing_level = min(zip(final_error[0].values(), final_error[0].keys()))
 */
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_individual_type, __pyx_v_individual_type) < 0) __PYX_ERR(0, 342, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":342
 *         new_individuals = [i for i in self.create_individuals(new_population)]
 * 
 *         final_error = [i for i in self.generate_smoothing_level_genome(population=new_individuals,             # <<<<<<<<<<<<<<
 *                                                                        individual_type=individual_type)]
 * 
 */
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
    __pyx_t_7 = __pyx_t_10; __Pyx_INCREF(__pyx_t_7); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 342, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_7))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_10 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_2); __Pyx_INCREF(__pyx_t_10); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 342, __pyx_L1_error)
        #else
        __pyx_t_10 = PySequence_ITEM(__pyx_t_7, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 342, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_2); __Pyx_INCREF(__pyx_t_10); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 342, __pyx_L1_error)
        #else
        __pyx_t_10 = PySequence_ITEM(__pyx_t_7, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 342, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
      }
    } else {
      __pyx_t_10 = __pyx_t_6(__pyx_t_7);
      if (unlikely(!__pyx_t_10)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 342, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_10);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_10);
    __pyx_t_10 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 342, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF_SET(__pyx_v_final_error, ((PyObject*)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":345
 *                                                                        individual_type=individual_type)]
 * 
 *         minimum_smoothing_level = min(zip(final_error[0].values(), final_error[0].keys()))             # <<<<<<<<<<<<<<
 * 
 *         return minimum_smoothing_level
 */
  __pyx_t_7 = __Pyx_GetItemInt_List(__pyx_v_final_error, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_values); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  if (__pyx_t_7) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_7 = __Pyx_GetItemInt_List(__pyx_v_final_error, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_keys); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (__pyx_t_7) {
    __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else {
    __pyx_t_10 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 345, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_10);
  __pyx_t_3 = 0;
  __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_10);
  __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_min, __pyx_t_1, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyTuple_CheckExact(__pyx_t_10))||((__pyx_t_10) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_10)->tp_name), 0))) __PYX_ERR(0, 345, __pyx_L1_error)
  __pyx_v_minimum_smoothing_level = ((PyObject*)__pyx_t_10);
  __pyx_t_10 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":347
 *         minimum_smoothing_level = min(zip(final_error[0].values(), final_error[0].keys()))
 * 
 *         return minimum_smoothing_level             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_minimum_smoothing_level);
  __pyx_r = __pyx_v_minimum_smoothing_level;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":275
 *         self.__population_size = population_size
 * 
 *     def _initialise_smoothing_level_evolutionary_algorithm_population(self, individual_type: str):             # <<<<<<<<<<<<<<
 *         """ Starts the process for creating the population. The number of parents is specified during the
 *          initialisation of the class. """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._initialise_smoothing_level_evolutionary_algorithm_population", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parents);
  __Pyx_XDECREF(__pyx_v_parents_population);
  __Pyx_XDECREF(__pyx_v_populations_genome);
  __Pyx_XDECREF(__pyx_v_populations_traits);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_parent);
  __Pyx_XDECREF(__pyx_v_fit_population);
  __Pyx_XDECREF(__pyx_v_create_offspring);
  __Pyx_XDECREF(__pyx_v_new_population);
  __Pyx_XDECREF(__pyx_v_parent_offspring_population);
  __Pyx_XDECREF(__pyx_v_new_individuals);
  __Pyx_XDECREF(__pyx_v_new_populations_traits);
  __Pyx_XDECREF(__pyx_v_new_fit_population);
  __Pyx_XDECREF(__pyx_v_final_error);
  __Pyx_XDECREF(__pyx_v_minimum_smoothing_level);
  __Pyx_XDECREF(__pyx_v_po);
  __Pyx_XDECREF(__pyx_v_pke);
  __Pyx_XDECREF(__pyx_v_genome);
  __Pyx_XDECREF(__pyx_v_new_individual);
  __Pyx_XDECREF(__pyx_v_new_population_genome);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_12generator3(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":350
 * 
 *     @staticmethod
 *     def create_individuals(list new_population):             # <<<<<<<<<<<<<<
 *         """Create individuals using class from genomes striped during processing fitness.
 *         Args:
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_11create_individuals(PyObject *__pyx_self, PyObject *__pyx_v_new_population); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_10create_individuals[] = "Create individuals using class from genomes striped during processing fitness.\n        Args:\n            new_population (list):  new population of individual genomes.\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_11create_individuals = {"create_individuals", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_11create_individuals, METH_O, __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_10create_individuals};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_11create_individuals(PyObject *__pyx_self, PyObject *__pyx_v_new_population) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_individuals (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_new_population), (&PyList_Type), 1, "new_population", 1))) __PYX_ERR(0, 350, __pyx_L1_error)
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_10create_individuals(__pyx_self, ((PyObject*)__pyx_v_new_population));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_10create_individuals(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_new_population) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_individuals", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 350, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_new_population = __pyx_v_new_population;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_new_population);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_new_population);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_12generator3, (PyObject *) __pyx_cur_scope, __pyx_n_s_create_individuals, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.create_individuals", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_12generator3(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L8_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 350, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":355
 *             new_population (list):  new population of individual genomes.
 *         """
 *         cdef list parent_offspring_population = []             # <<<<<<<<<<<<<<
 * 
 *         for po in new_population:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_parent_offspring_population = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":357
 *         cdef list parent_offspring_population = []
 * 
 *         for po in new_population:             # <<<<<<<<<<<<<<
 *             pke = po.keys()
 *             parent_offspring_population.append(tuple(pke))
 */
  if (unlikely(__pyx_cur_scope->__pyx_v_new_population == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 357, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_new_population; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 357, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_po);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_po, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":358
 * 
 *         for po in new_population:
 *             pke = po.keys()             # <<<<<<<<<<<<<<
 *             parent_offspring_population.append(tuple(pke))
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_po, __pyx_n_s_keys); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 358, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 358, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_pke);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_pke, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":359
 *         for po in new_population:
 *             pke = po.keys()
 *             parent_offspring_population.append(tuple(pke))             # <<<<<<<<<<<<<<
 * 
 *         for genome in parent_offspring_population:
 */
    __pyx_t_3 = PySequence_Tuple(__pyx_cur_scope->__pyx_v_pke); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 359, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_parent_offspring_population, __pyx_t_3); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 359, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":357
 *         cdef list parent_offspring_population = []
 * 
 *         for po in new_population:             # <<<<<<<<<<<<<<
 *             pke = po.keys()
 *             parent_offspring_population.append(tuple(pke))
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":361
 *             parent_offspring_population.append(tuple(pke))
 * 
 *         for genome in parent_offspring_population:             # <<<<<<<<<<<<<<
 *             new_individual = Individual(overide=True)
 *             new_individual.genome = genome
 */
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_parent_offspring_population; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 361, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_genome);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_genome, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":362
 * 
 *         for genome in parent_offspring_population:
 *             new_individual = Individual(overide=True)             # <<<<<<<<<<<<<<
 *             new_individual.genome = genome
 *             yield new_individual
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_Individual); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_overide, Py_True) < 0) __PYX_ERR(0, 362, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_new_individual);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_new_individual, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    __pyx_t_5 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":363
 *         for genome in parent_offspring_population:
 *             new_individual = Individual(overide=True)
 *             new_individual.genome = genome             # <<<<<<<<<<<<<<
 *             yield new_individual
 * 
 */
    if (__Pyx_PyObject_SetAttrStr(__pyx_cur_scope->__pyx_v_new_individual, __pyx_n_s_genome_2, __pyx_cur_scope->__pyx_v_genome) < 0) __PYX_ERR(0, 363, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":364
 *             new_individual = Individual(overide=True)
 *             new_individual.genome = genome
 *             yield new_individual             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_new_individual);
    __pyx_r = __pyx_cur_scope->__pyx_v_new_individual;
    __Pyx_XGIVEREF(__pyx_t_1);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L8_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 364, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":361
 *             parent_offspring_population.append(tuple(pke))
 * 
 *         for genome in parent_offspring_population:             # <<<<<<<<<<<<<<
 *             new_individual = Individual(overide=True)
 *             new_individual.genome = genome
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":350
 * 
 *     @staticmethod
 *     def create_individuals(list new_population):             # <<<<<<<<<<<<<<
 *         """Create individuals using class from genomes striped during processing fitness.
 *         Args:
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("create_individuals", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_15generator4(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":367
 * 
 *     @staticmethod
 *     def _population_fitness(list population, individual_type='ses'):             # <<<<<<<<<<<<<<
 *         """ Assess the population for fitness before crossover and creating next generation. Positive traits
 *         should be reflected by more than 70% of the genes in the genome.
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_14_population_fitness(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_13_population_fitness[] = " Assess the population for fitness before crossover and creating next generation. Positive traits\n        should be reflected by more than 70% of the genes in the genome.\n\n        Args:\n            population (list): A population with expressed traits (phenotypes). The standard_errors representing\n            the genome of the individual have been calculated and expressed as one or zero if below or above\n            the original standard error respectively.\n\n        Returns:\n            fit_population (list):  A population of individuals with a probability of procreating above 70% for simple\n                                    exponential smoothing and 30% for holts trend correcting. ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_14_population_fitness = {"_population_fitness", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_14_population_fitness, METH_VARARGS|METH_KEYWORDS, __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_13_population_fitness};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_14_population_fitness(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_population = 0;
  PyObject *__pyx_v_individual_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_population_fitness (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_population,&__pyx_n_s_individual_type,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)((PyObject*)__pyx_n_s_ses));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_population)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individual_type);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_population_fitness") < 0)) __PYX_ERR(0, 367, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_population = ((PyObject*)values[0]);
    __pyx_v_individual_type = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_population_fitness", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 367, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._population_fitness", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_population), (&PyList_Type), 1, "population", 1))) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_13_population_fitness(__pyx_self, __pyx_v_population, __pyx_v_individual_type);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_13_population_fitness(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_population, PyObject *__pyx_v_individual_type) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_population_fitness", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 367, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_population = __pyx_v_population;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_population);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_population);
  __pyx_cur_scope->__pyx_v_individual_type = __pyx_v_individual_type;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individual_type);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_individual_type);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_15generator4, (PyObject *) __pyx_cur_scope, __pyx_n_s_population_fitness, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_2, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._population_fitness", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_15generator4(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  double __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L8_resume_from_yield;
    case 2: goto __pyx_L10_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 367, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":380
 *                                     exponential smoothing and 30% for holts trend correcting. """
 *         cdef double procreation_probability
 *         for individual in population:             # <<<<<<<<<<<<<<
 *             procreation_probability = sum(individual.values()) / len(individual.values())
 *             if individual_type == 'ses':
 */
  if (unlikely(__pyx_cur_scope->__pyx_v_population == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 380, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_population; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 380, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_individual);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_individual, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":381
 *         cdef double procreation_probability
 *         for individual in population:
 *             procreation_probability = sum(individual.values()) / len(individual.values())             # <<<<<<<<<<<<<<
 *             if individual_type == 'ses':
 *                 if procreation_probability >= 0.4:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_individual, __pyx_n_s_values); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 381, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_sum, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_individual, __pyx_n_s_values); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_7 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_cur_scope->__pyx_v_procreation_probability = __pyx_t_8;

    /* "supplychainpy/demand/_evo_algo.pyx":382
 *         for individual in population:
 *             procreation_probability = sum(individual.values()) / len(individual.values())
 *             if individual_type == 'ses':             # <<<<<<<<<<<<<<
 *                 if procreation_probability >= 0.4:
 *                     yield individual
 */
    __pyx_t_9 = (__Pyx_PyString_Equals(__pyx_cur_scope->__pyx_v_individual_type, __pyx_n_s_ses, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 382, __pyx_L1_error)
    if (__pyx_t_9) {

      /* "supplychainpy/demand/_evo_algo.pyx":383
 *             procreation_probability = sum(individual.values()) / len(individual.values())
 *             if individual_type == 'ses':
 *                 if procreation_probability >= 0.4:             # <<<<<<<<<<<<<<
 *                     yield individual
 *             else:
 */
      __pyx_t_9 = ((__pyx_cur_scope->__pyx_v_procreation_probability >= 0.4) != 0);
      if (__pyx_t_9) {

        /* "supplychainpy/demand/_evo_algo.pyx":384
 *             if individual_type == 'ses':
 *                 if procreation_probability >= 0.4:
 *                     yield individual             # <<<<<<<<<<<<<<
 *             else:
 *                 if procreation_probability >= 0.3:
 */
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individual);
        __pyx_r = __pyx_cur_scope->__pyx_v_individual;
        __Pyx_XGIVEREF(__pyx_t_1);
        __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
        __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
        __Pyx_XGIVEREF(__pyx_r);
        __Pyx_RefNannyFinishContext();
        /* return from generator, yielding value */
        __pyx_generator->resume_label = 1;
        return __pyx_r;
        __pyx_L8_resume_from_yield:;
        __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
        __pyx_cur_scope->__pyx_t_0 = 0;
        __Pyx_XGOTREF(__pyx_t_1);
        __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
        if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 384, __pyx_L1_error)

        /* "supplychainpy/demand/_evo_algo.pyx":383
 *             procreation_probability = sum(individual.values()) / len(individual.values())
 *             if individual_type == 'ses':
 *                 if procreation_probability >= 0.4:             # <<<<<<<<<<<<<<
 *                     yield individual
 *             else:
 */
      }

      /* "supplychainpy/demand/_evo_algo.pyx":382
 *         for individual in population:
 *             procreation_probability = sum(individual.values()) / len(individual.values())
 *             if individual_type == 'ses':             # <<<<<<<<<<<<<<
 *                 if procreation_probability >= 0.4:
 *                     yield individual
 */
      goto __pyx_L6;
    }

    /* "supplychainpy/demand/_evo_algo.pyx":386
 *                     yield individual
 *             else:
 *                 if procreation_probability >= 0.3:             # <<<<<<<<<<<<<<
 *                     yield individual
 * 
 */
    /*else*/ {
      __pyx_t_9 = ((__pyx_cur_scope->__pyx_v_procreation_probability >= 0.3) != 0);
      if (__pyx_t_9) {

        /* "supplychainpy/demand/_evo_algo.pyx":387
 *             else:
 *                 if procreation_probability >= 0.3:
 *                     yield individual             # <<<<<<<<<<<<<<
 * 
 *     def generate_smoothing_level_genome(self, list population, individual_type = 'ses'):
 */
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individual);
        __pyx_r = __pyx_cur_scope->__pyx_v_individual;
        __Pyx_XGIVEREF(__pyx_t_1);
        __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
        __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
        __Pyx_XGIVEREF(__pyx_r);
        __Pyx_RefNannyFinishContext();
        /* return from generator, yielding value */
        __pyx_generator->resume_label = 2;
        return __pyx_r;
        __pyx_L10_resume_from_yield:;
        __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
        __pyx_cur_scope->__pyx_t_0 = 0;
        __Pyx_XGOTREF(__pyx_t_1);
        __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
        if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 387, __pyx_L1_error)

        /* "supplychainpy/demand/_evo_algo.pyx":386
 *                     yield individual
 *             else:
 *                 if procreation_probability >= 0.3:             # <<<<<<<<<<<<<<
 *                     yield individual
 * 
 */
      }
    }
    __pyx_L6:;

    /* "supplychainpy/demand/_evo_algo.pyx":380
 *                                     exponential smoothing and 30% for holts trend correcting. """
 *         cdef double procreation_probability
 *         for individual in population:             # <<<<<<<<<<<<<<
 *             procreation_probability = sum(individual.values()) / len(individual.values())
 *             if individual_type == 'ses':
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":367
 * 
 *     @staticmethod
 *     def _population_fitness(list population, individual_type='ses'):             # <<<<<<<<<<<<<<
 *         """ Assess the population for fitness before crossover and creating next generation. Positive traits
 *         should be reflected by more than 70% of the genes in the genome.
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_population_fitness", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_18generator5(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":389
 *                     yield individual
 * 
 *     def generate_smoothing_level_genome(self, list population, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_17generate_smoothing_level_genome(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_17generate_smoothing_level_genome = {"generate_smoothing_level_genome", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_17generate_smoothing_level_genome, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_17generate_smoothing_level_genome(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_population = 0;
  PyObject *__pyx_v_individual_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate_smoothing_level_genome (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_population,&__pyx_n_s_individual_type,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)((PyObject*)__pyx_n_s_ses));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_population)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("generate_smoothing_level_genome", 0, 2, 3, 1); __PYX_ERR(0, 389, __pyx_L3_error)
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individual_type);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "generate_smoothing_level_genome") < 0)) __PYX_ERR(0, 389, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_population = ((PyObject*)values[1]);
    __pyx_v_individual_type = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("generate_smoothing_level_genome", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 389, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.generate_smoothing_level_genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_population), (&PyList_Type), 1, "population", 1))) __PYX_ERR(0, 389, __pyx_L1_error)
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_16generate_smoothing_level_genome(__pyx_self, __pyx_v_self, __pyx_v_population, __pyx_v_individual_type);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_16generate_smoothing_level_genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_population, PyObject *__pyx_v_individual_type) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate_smoothing_level_genome", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 389, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_population = __pyx_v_population;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_population);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_population);
  __pyx_cur_scope->__pyx_v_individual_type = __pyx_v_individual_type;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individual_type);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_individual_type);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_18generator5, (PyObject *) __pyx_cur_scope, __pyx_n_s_generate_smoothing_level_genome, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_3, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.generate_smoothing_level_genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_18generator5(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    case 2: goto __pyx_L10_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 389, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":392
 * 
 *         # stack parents and offspring into a list for next steps
 *         if individual_type == 'ses':             # <<<<<<<<<<<<<<
 *             for parent in population:
 *                 individuals_genome = self._run_exponential_smoothing_forecast(parent.genome)
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_cur_scope->__pyx_v_individual_type, __pyx_n_s_ses, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 392, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "supplychainpy/demand/_evo_algo.pyx":393
 *         # stack parents and offspring into a list for next steps
 *         if individual_type == 'ses':
 *             for parent in population:             # <<<<<<<<<<<<<<
 *                 individuals_genome = self._run_exponential_smoothing_forecast(parent.genome)
 *                 # print(individuals_genome)
 */
    if (unlikely(__pyx_cur_scope->__pyx_v_population == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 393, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_cur_scope->__pyx_v_population; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    for (;;) {
      if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 393, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_parent);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_parent, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":394
 *         if individual_type == 'ses':
 *             for parent in population:
 *                 individuals_genome = self._run_exponential_smoothing_forecast(parent.genome)             # <<<<<<<<<<<<<<
 *                 # print(individuals_genome)
 *                 # individuals_traits = self._express_trait(standard_error, smoothing_level, individuals_genome)
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_run_exponential_smoothing_forec); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 394, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_parent, __pyx_n_s_genome_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 394, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_individuals_genome);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_individuals_genome, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":398
 *                 # individuals_traits = self._express_trait(standard_error, smoothing_level, individuals_genome)
 * 
 *                 yield individuals_genome             # <<<<<<<<<<<<<<
 *         else:
 *             for parent in population:
 */
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individuals_genome);
      __pyx_r = __pyx_cur_scope->__pyx_v_individuals_genome;
      __Pyx_XGIVEREF(__pyx_t_2);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L7_resume_from_yield:;
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 398, __pyx_L1_error)

      /* "supplychainpy/demand/_evo_algo.pyx":393
 *         # stack parents and offspring into a list for next steps
 *         if individual_type == 'ses':
 *             for parent in population:             # <<<<<<<<<<<<<<
 *                 individuals_genome = self._run_exponential_smoothing_forecast(parent.genome)
 *                 # print(individuals_genome)
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":392
 * 
 *         # stack parents and offspring into a list for next steps
 *         if individual_type == 'ses':             # <<<<<<<<<<<<<<
 *             for parent in population:
 *                 individuals_genome = self._run_exponential_smoothing_forecast(parent.genome)
 */
    goto __pyx_L4;
  }

  /* "supplychainpy/demand/_evo_algo.pyx":400
 *                 yield individuals_genome
 *         else:
 *             for parent in population:             # <<<<<<<<<<<<<<
 *                 individuals_genome = self._run_holts_trend_corrected_exponential_smoothing(parent.genome)
 *                 # print(individuals_genome)
 */
  /*else*/ {
    if (unlikely(__pyx_cur_scope->__pyx_v_population == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 400, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_cur_scope->__pyx_v_population; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    for (;;) {
      if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 400, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_parent);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_parent, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":401
 *         else:
 *             for parent in population:
 *                 individuals_genome = self._run_holts_trend_corrected_exponential_smoothing(parent.genome)             # <<<<<<<<<<<<<<
 *                 # print(individuals_genome)
 *                 # individuals_traits = self._express_trait(standard_error, smoothing_level, individuals_genome)
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_run_holts_trend_corrected_expon); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_parent, __pyx_n_s_genome_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_8};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_8};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 401, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_8);
          __pyx_t_8 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_individuals_genome);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_individuals_genome, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "supplychainpy/demand/_evo_algo.pyx":405
 *                 # individuals_traits = self._express_trait(standard_error, smoothing_level, individuals_genome)
 * 
 *                 yield individuals_genome             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individuals_genome);
      __pyx_r = __pyx_cur_scope->__pyx_v_individuals_genome;
      __Pyx_XGIVEREF(__pyx_t_2);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 2;
      return __pyx_r;
      __pyx_L10_resume_from_yield:;
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 405, __pyx_L1_error)

      /* "supplychainpy/demand/_evo_algo.pyx":400
 *                 yield individuals_genome
 *         else:
 *             for parent in population:             # <<<<<<<<<<<<<<
 *                 individuals_genome = self._run_holts_trend_corrected_exponential_smoothing(parent.genome)
 *                 # print(individuals_genome)
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L4:;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":389
 *                     yield individual
 * 
 *     def generate_smoothing_level_genome(self, list population, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("generate_smoothing_level_genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":408
 * 
 *     @staticmethod
 *     def htce_para(forecast, alpha, gamma, intercept, slope):             # <<<<<<<<<<<<<<
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 *                                                                                 intercept=intercept,
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_20htce_para(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_20htce_para = {"htce_para", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_20htce_para, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_20htce_para(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_forecast = 0;
  PyObject *__pyx_v_alpha = 0;
  PyObject *__pyx_v_gamma = 0;
  PyObject *__pyx_v_intercept = 0;
  PyObject *__pyx_v_slope = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("htce_para (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_forecast,&__pyx_n_s_alpha,&__pyx_n_s_gamma,&__pyx_n_s_intercept,&__pyx_n_s_slope,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_forecast)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("htce_para", 1, 5, 5, 1); __PYX_ERR(0, 408, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_gamma)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("htce_para", 1, 5, 5, 2); __PYX_ERR(0, 408, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_intercept)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("htce_para", 1, 5, 5, 3); __PYX_ERR(0, 408, __pyx_L3_error)
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_slope)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("htce_para", 1, 5, 5, 4); __PYX_ERR(0, 408, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "htce_para") < 0)) __PYX_ERR(0, 408, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_forecast = values[0];
    __pyx_v_alpha = values[1];
    __pyx_v_gamma = values[2];
    __pyx_v_intercept = values[3];
    __pyx_v_slope = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("htce_para", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 408, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.htce_para", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_19htce_para(__pyx_self, __pyx_v_forecast, __pyx_v_alpha, __pyx_v_gamma, __pyx_v_intercept, __pyx_v_slope);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_19htce_para(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_forecast, PyObject *__pyx_v_alpha, PyObject *__pyx_v_gamma, PyObject *__pyx_v_intercept, PyObject *__pyx_v_slope) {
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  __Pyx_RefNannySetupContext("htce_para", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":409
 *     @staticmethod
 *     def htce_para(forecast, alpha, gamma, intercept, slope):
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,             # <<<<<<<<<<<<<<
 *                                                                                 intercept=intercept,
 *                                                                                 slope=slope)]
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast, __pyx_n_s_holts_trend_corrected_exponentia); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_alpha, __pyx_v_alpha) < 0) __PYX_ERR(0, 409, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_gamma, __pyx_v_gamma) < 0) __PYX_ERR(0, 409, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":410
 *     def htce_para(forecast, alpha, gamma, intercept, slope):
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 *                                                                                 intercept=intercept,             # <<<<<<<<<<<<<<
 *                                                                                 slope=slope)]
 * 
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_intercept, __pyx_v_intercept) < 0) __PYX_ERR(0, 409, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":411
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 *                                                                                 intercept=intercept,
 *                                                                                 slope=slope)]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_slope, __pyx_v_slope) < 0) __PYX_ERR(0, 409, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":409
 *     @staticmethod
 *     def htce_para(forecast, alpha, gamma, intercept, slope):
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,             # <<<<<<<<<<<<<<
 *                                                                                 intercept=intercept,
 *                                                                                 slope=slope)]
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
    __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 409, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 409, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 409, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_6(__pyx_t_3);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 409, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 409, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":408
 * 
 *     @staticmethod
 *     def htce_para(forecast, alpha, gamma, intercept, slope):             # <<<<<<<<<<<<<<
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 *                                                                                 intercept=intercept,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.htce_para", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":414
 * 
 * 
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):             # <<<<<<<<<<<<<<
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_22sum_squared_error_para(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_22sum_squared_error_para = {"sum_squared_error_para", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_22sum_squared_error_para, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_22sum_squared_error_para(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_forecast = 0;
  PyObject *__pyx_v_holts_trend_corrected_smoothing = 0;
  PyObject *__pyx_v_alpha = 0;
  PyObject *__pyx_v_gamma = 0;
  PyObject *__pyx_v_smoothing_level = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sum_squared_error_para (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_forecast,&__pyx_n_s_holts_trend_corrected_smoothing,&__pyx_n_s_alpha,&__pyx_n_s_gamma,&__pyx_n_s_smoothing_level,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_forecast)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sum_squared_error_para", 1, 6, 6, 1); __PYX_ERR(0, 414, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_holts_trend_corrected_smoothing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sum_squared_error_para", 1, 6, 6, 2); __PYX_ERR(0, 414, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sum_squared_error_para", 1, 6, 6, 3); __PYX_ERR(0, 414, __pyx_L3_error)
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_gamma)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sum_squared_error_para", 1, 6, 6, 4); __PYX_ERR(0, 414, __pyx_L3_error)
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_smoothing_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sum_squared_error_para", 1, 6, 6, 5); __PYX_ERR(0, 414, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sum_squared_error_para") < 0)) __PYX_ERR(0, 414, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_self = values[0];
    __pyx_v_forecast = values[1];
    __pyx_v_holts_trend_corrected_smoothing = values[2];
    __pyx_v_alpha = values[3];
    __pyx_v_gamma = values[4];
    __pyx_v_smoothing_level = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sum_squared_error_para", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 414, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.sum_squared_error_para", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_21sum_squared_error_para(__pyx_self, __pyx_v_self, __pyx_v_forecast, __pyx_v_holts_trend_corrected_smoothing, __pyx_v_alpha, __pyx_v_gamma, __pyx_v_smoothing_level);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_21sum_squared_error_para(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_forecast, PyObject *__pyx_v_holts_trend_corrected_smoothing, PyObject *__pyx_v_alpha, PyObject *__pyx_v_gamma, PyObject *__pyx_v_smoothing_level) {
  PyObject *__pyx_v_appraised_individual = NULL;
  PyObject *__pyx_v_sum_squared_error = NULL;
  PyObject *__pyx_v_standard_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("sum_squared_error_para", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":415
 * 
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):
 *         appraised_individual = {}             # <<<<<<<<<<<<<<
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                             alpha=alpha, gamma=gamma)
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_appraised_individual = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":416
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,             # <<<<<<<<<<<<<<
 *                                                             alpha=alpha, gamma=gamma)
 *         standard_error = forecast.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast, __pyx_n_s_sum_squared_errors_indi_htces); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_squared_error, __pyx_v_holts_trend_corrected_smoothing) < 0) __PYX_ERR(0, 416, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":417
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                             alpha=alpha, gamma=gamma)             # <<<<<<<<<<<<<<
 *         standard_error = forecast.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *         appraised_individual.update({smoothing_level: standard_error})
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_alpha, __pyx_v_alpha) < 0) __PYX_ERR(0, 416, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_gamma, __pyx_v_gamma) < 0) __PYX_ERR(0, 416, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":416
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,             # <<<<<<<<<<<<<<
 *                                                             alpha=alpha, gamma=gamma)
 *         standard_error = forecast.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_sum_squared_error = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":418
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                             alpha=alpha, gamma=gamma)
 *         standard_error = forecast.standard_error(sum_squared_error, len(self.__orders), smoothing_level)             # <<<<<<<<<<<<<<
 *         appraised_individual.update({smoothing_level: standard_error})
 *         return appraised_individual
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast, __pyx_n_s_standard_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_4 == -1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_sum_squared_error, __pyx_t_1, __pyx_v_smoothing_level};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_sum_squared_error, __pyx_t_1, __pyx_v_smoothing_level};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_sum_squared_error);
    __Pyx_GIVEREF(__pyx_v_sum_squared_error);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_sum_squared_error);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_smoothing_level);
    __Pyx_GIVEREF(__pyx_v_smoothing_level);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_v_smoothing_level);
    __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_standard_error = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":419
 *                                                             alpha=alpha, gamma=gamma)
 *         standard_error = forecast.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *         appraised_individual.update({smoothing_level: standard_error})             # <<<<<<<<<<<<<<
 *         return appraised_individual
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_appraised_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_v_smoothing_level, __pyx_v_standard_error) < 0) __PYX_ERR(0, 419, __pyx_L1_error)
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_t_7};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_t_7};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 419, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":420
 *         standard_error = forecast.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *         appraised_individual.update({smoothing_level: standard_error})
 *         return appraised_individual             # <<<<<<<<<<<<<<
 * 
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_appraised_individual);
  __pyx_r = __pyx_v_appraised_individual;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":414
 * 
 * 
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):             # <<<<<<<<<<<<<<
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.sum_squared_error_para", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_appraised_individual);
  __Pyx_XDECREF(__pyx_v_sum_squared_error);
  __Pyx_XDECREF(__pyx_v_standard_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":422
 *         return appraised_individual
 * 
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):             # <<<<<<<<<<<<<<
 *         f = Forecast(self.__orders, self.__average_order)
 *         holts_trend_corrected_smoothing = []
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_24_run_holts_trend_corrected_exponential_smoothing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_24_run_holts_trend_corrected_exponential_smoothing = {"_run_holts_trend_corrected_exponential_smoothing", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_24_run_holts_trend_corrected_exponential_smoothing, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_24_run_holts_trend_corrected_exponential_smoothing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_individual = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_run_holts_trend_corrected_exponential_smoothing (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individual,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individual)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_run_holts_trend_corrected_exponential_smoothing", 1, 2, 2, 1); __PYX_ERR(0, 422, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_run_holts_trend_corrected_exponential_smoothing") < 0)) __PYX_ERR(0, 422, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_individual = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_run_holts_trend_corrected_exponential_smoothing", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 422, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._run_holts_trend_corrected_exponential_smoothing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_23_run_holts_trend_corrected_exponential_smoothing(__pyx_self, __pyx_v_self, __pyx_v_individual);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_23_run_holts_trend_corrected_exponential_smoothing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual) {
  PyObject *__pyx_v_f = NULL;
  PyObject *__pyx_v_holts_trend_corrected_smoothing = NULL;
  PyObject *__pyx_v_demand = NULL;
  PyObject *__pyx_v_stats = NULL;
  PyObject *__pyx_v_log_stats = NULL;
  PyObject *__pyx_v_sm_lvl = NULL;
  PyObject *__pyx_v_p = NULL;
  PyObject *__pyx_v_appraised_individual = NULL;
  PyObject *__pyx_v_smoothing_level = NULL;
  PyObject *__pyx_v_sum_squared_error = NULL;
  PyObject *__pyx_v_standard_error = NULL;
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_v_order = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("_run_holts_trend_corrected_exponential_smoothing", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":423
 * 
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):
 *         f = Forecast(self.__orders, self.__average_order)             # <<<<<<<<<<<<<<
 *         holts_trend_corrected_smoothing = []
 *         demand = [{'t': index, 'demand': order} for index, order in enumerate(self.__orders, 1)]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Forecast); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 423, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 423, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 423, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 423, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_f = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":424
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):
 *         f = Forecast(self.__orders, self.__average_order)
 *         holts_trend_corrected_smoothing = []             # <<<<<<<<<<<<<<
 *         demand = [{'t': index, 'demand': order} for index, order in enumerate(self.__orders, 1)]
 *         stats = LinearRegression(demand)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_holts_trend_corrected_smoothing = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":425
 *         f = Forecast(self.__orders, self.__average_order)
 *         holts_trend_corrected_smoothing = []
 *         demand = [{'t': index, 'demand': order} for index, order in enumerate(self.__orders, 1)]             # <<<<<<<<<<<<<<
 *         stats = LinearRegression(demand)
 *         log_stats = stats.least_squared_error(slice_end=6)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __pyx_t_2 = __pyx_int_1;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
    __pyx_t_4 = __pyx_t_7; __Pyx_INCREF(__pyx_t_4); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 425, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_8); __Pyx_INCREF(__pyx_t_7); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 425, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 425, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_8); __Pyx_INCREF(__pyx_t_7); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 425, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 425, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_9(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 425, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_order, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_2);
    __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_2, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_7;
    __pyx_t_7 = 0;
    __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_t, __pyx_v_index) < 0) __PYX_ERR(0, 425, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_demand, __pyx_v_order) < 0) __PYX_ERR(0, 425, __pyx_L1_error)
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_demand = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":426
 *         holts_trend_corrected_smoothing = []
 *         demand = [{'t': index, 'demand': order} for index, order in enumerate(self.__orders, 1)]
 *         stats = LinearRegression(demand)             # <<<<<<<<<<<<<<
 *         log_stats = stats.least_squared_error(slice_end=6)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_LinearRegression); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_demand); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_demand};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_demand};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_demand);
      __Pyx_GIVEREF(__pyx_v_demand);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_demand);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_stats = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":427
 *         demand = [{'t': index, 'demand': order} for index, order in enumerate(self.__orders, 1)]
 *         stats = LinearRegression(demand)
 *         log_stats = stats.least_squared_error(slice_end=6)             # <<<<<<<<<<<<<<
 * 
 *         for sm_lvl in individual:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_stats, __pyx_n_s_least_squared_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_slice_end, __pyx_int_6) < 0) __PYX_ERR(0, 427, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_log_stats = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":429
 *         log_stats = stats.least_squared_error(slice_end=6)
 * 
 *         for sm_lvl in individual:             # <<<<<<<<<<<<<<
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],
 *                                                                           intercept=log_stats.get('intercept'),
 */
  if (likely(PyList_CheckExact(__pyx_v_individual)) || PyTuple_CheckExact(__pyx_v_individual)) {
    __pyx_t_7 = __pyx_v_individual; __Pyx_INCREF(__pyx_t_7); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_v_individual); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 429, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_7))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 429, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 429, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_9(__pyx_t_7);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 429, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_sm_lvl, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":430
 * 
 *         for sm_lvl in individual:
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],             # <<<<<<<<<<<<<<
 *                                                                           intercept=log_stats.get('intercept'),
 *                                                                           slope=log_stats.get('slope'))]
 */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_holts_trend_corrected_exponentia); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_sm_lvl, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_alpha, __pyx_t_3) < 0) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_sm_lvl, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_gamma, __pyx_t_3) < 0) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":431
 *         for sm_lvl in individual:
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],
 *                                                                           intercept=log_stats.get('intercept'),             # <<<<<<<<<<<<<<
 *                                                                           slope=log_stats.get('slope'))]
 *             # print(p)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_log_stats, __pyx_n_s_get); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_intercept, __pyx_t_5) < 0) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":432
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],
 *                                                                           intercept=log_stats.get('intercept'),
 *                                                                           slope=log_stats.get('slope'))]             # <<<<<<<<<<<<<<
 *             # print(p)
 *             holts_trend_corrected_smoothing.append(p)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_log_stats, __pyx_n_s_get); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_slope, __pyx_t_3) < 0) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":430
 * 
 *         for sm_lvl in individual:
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],             # <<<<<<<<<<<<<<
 *                                                                           intercept=log_stats.get('intercept'),
 *                                                                           slope=log_stats.get('slope'))]
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_10 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_11 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 430, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_10); __Pyx_INCREF(__pyx_t_3); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 430, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_10); __Pyx_INCREF(__pyx_t_3); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 430, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_11(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 430, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
      __pyx_t_3 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 430, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_p, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":434
 *                                                                           slope=log_stats.get('slope'))]
 *             # print(p)
 *             holts_trend_corrected_smoothing.append(p)             # <<<<<<<<<<<<<<
 * 
 *         appraised_individual = {}
 */
    __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_holts_trend_corrected_smoothing, __pyx_v_p); if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(0, 434, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":429
 *         log_stats = stats.least_squared_error(slice_end=6)
 * 
 *         for sm_lvl in individual:             # <<<<<<<<<<<<<<
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],
 *                                                                           intercept=log_stats.get('intercept'),
 */
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":436
 *             holts_trend_corrected_smoothing.append(p)
 * 
 *         appraised_individual = {}             # <<<<<<<<<<<<<<
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 */
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_v_appraised_individual = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":437
 * 
 *         appraised_individual = {}
 *         for smoothing_level in individual:             # <<<<<<<<<<<<<<
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])
 */
  if (likely(PyList_CheckExact(__pyx_v_individual)) || PyTuple_CheckExact(__pyx_v_individual)) {
    __pyx_t_7 = __pyx_v_individual; __Pyx_INCREF(__pyx_t_7); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_v_individual); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 437, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_7))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 437, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 437, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_9(__pyx_t_7);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 437, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_smoothing_level, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":438
 *         appraised_individual = {}
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,             # <<<<<<<<<<<<<<
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_sum_squared_errors_indi_htces); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_squared_error, __pyx_v_holts_trend_corrected_smoothing) < 0) __PYX_ERR(0, 438, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":439
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])             # <<<<<<<<<<<<<<
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *             appraised_individual.update({smoothing_level: standard_error})
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_smoothing_level, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_alpha, __pyx_t_3) < 0) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_smoothing_level, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_gamma, __pyx_t_3) < 0) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":438
 *         appraised_individual = {}
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,             # <<<<<<<<<<<<<<
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_sum_squared_error, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":440
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)             # <<<<<<<<<<<<<<
 *             appraised_individual.update({smoothing_level: standard_error})
 *         # print('The standard error as a trait has been calculated {}'.format(appraised_individual))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_standard_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_sum_squared_error, __pyx_t_2, __pyx_v_smoothing_level};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_sum_squared_error, __pyx_t_2, __pyx_v_smoothing_level};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_sum_squared_error);
      __Pyx_GIVEREF(__pyx_v_sum_squared_error);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_6, __pyx_v_sum_squared_error);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_6, __pyx_t_2);
      __Pyx_INCREF(__pyx_v_smoothing_level);
      __Pyx_GIVEREF(__pyx_v_smoothing_level);
      PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_6, __pyx_v_smoothing_level);
      __pyx_t_2 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_standard_error, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":441
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *             appraised_individual.update({smoothing_level: standard_error})             # <<<<<<<<<<<<<<
 *         # print('The standard error as a trait has been calculated {}'.format(appraised_individual))
 *         return appraised_individual
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_appraised_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_v_smoothing_level, __pyx_v_standard_error) < 0) __PYX_ERR(0, 441, __pyx_L1_error)
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_2) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":437
 * 
 *         appraised_individual = {}
 *         for smoothing_level in individual:             # <<<<<<<<<<<<<<
 *             sum_squared_error = f.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 *                                                                 alpha=smoothing_level[0], gamma=smoothing_level[1])
 */
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":443
 *             appraised_individual.update({smoothing_level: standard_error})
 *         # print('The standard error as a trait has been calculated {}'.format(appraised_individual))
 *         return appraised_individual             # <<<<<<<<<<<<<<
 * 
 *     def express_smoothing_level_genome(self, individuals_genome: list, standard_error):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_appraised_individual);
  __pyx_r = __pyx_v_appraised_individual;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":422
 *         return appraised_individual
 * 
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):             # <<<<<<<<<<<<<<
 *         f = Forecast(self.__orders, self.__average_order)
 *         holts_trend_corrected_smoothing = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._run_holts_trend_corrected_exponential_smoothing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_holts_trend_corrected_smoothing);
  __Pyx_XDECREF(__pyx_v_demand);
  __Pyx_XDECREF(__pyx_v_stats);
  __Pyx_XDECREF(__pyx_v_log_stats);
  __Pyx_XDECREF(__pyx_v_sm_lvl);
  __Pyx_XDECREF(__pyx_v_p);
  __Pyx_XDECREF(__pyx_v_appraised_individual);
  __Pyx_XDECREF(__pyx_v_smoothing_level);
  __Pyx_XDECREF(__pyx_v_sum_squared_error);
  __Pyx_XDECREF(__pyx_v_standard_error);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_order);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_27generator6(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "supplychainpy/demand/_evo_algo.pyx":445
 *         return appraised_individual
 * 
 *     def express_smoothing_level_genome(self, individuals_genome: list, standard_error):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_26express_smoothing_level_genome(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_26express_smoothing_level_genome = {"express_smoothing_level_genome", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_26express_smoothing_level_genome, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_26express_smoothing_level_genome(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_individuals_genome = 0;
  PyObject *__pyx_v_standard_error = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("express_smoothing_level_genome (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individuals_genome,&__pyx_n_s_standard_error,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individuals_genome)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("express_smoothing_level_genome", 1, 3, 3, 1); __PYX_ERR(0, 445, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_standard_error)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("express_smoothing_level_genome", 1, 3, 3, 2); __PYX_ERR(0, 445, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "express_smoothing_level_genome") < 0)) __PYX_ERR(0, 445, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_individuals_genome = values[1];
    __pyx_v_standard_error = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("express_smoothing_level_genome", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 445, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.express_smoothing_level_genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_25express_smoothing_level_genome(__pyx_self, __pyx_v_self, __pyx_v_individuals_genome, __pyx_v_standard_error);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_25express_smoothing_level_genome(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individuals_genome, PyObject *__pyx_v_standard_error) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("express_smoothing_level_genome", 0);
  __pyx_cur_scope = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome(__pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 445, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_individuals_genome = __pyx_v_individuals_genome;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individuals_genome);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_individuals_genome);
  __pyx_cur_scope->__pyx_v_standard_error = __pyx_v_standard_error;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_standard_error);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_standard_error);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_27generator6, (PyObject *) __pyx_cur_scope, __pyx_n_s_express_smoothing_level_genome, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_4, __pyx_n_s_supplychainpy_demand__evo_algo); if (unlikely(!gen)) __PYX_ERR(0, 445, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.express_smoothing_level_genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_27generator6(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *__pyx_cur_scope = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 445, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":448
 * 
 *         # stack parents and offspring into a list for next steps
 *         for genome in individuals_genome:             # <<<<<<<<<<<<<<
 *             individuals_traits = self._express_trait(standard_error, genome)
 *             yield individuals_traits
 */
  if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_individuals_genome)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_individuals_genome)) {
    __pyx_t_1 = __pyx_cur_scope->__pyx_v_individuals_genome; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_individuals_genome); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 448, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 448, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 448, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 448, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 448, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 448, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_genome);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_genome, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":449
 *         # stack parents and offspring into a list for next steps
 *         for genome in individuals_genome:
 *             individuals_traits = self._express_trait(standard_error, genome)             # <<<<<<<<<<<<<<
 *             yield individuals_traits
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self, __pyx_n_s_express_trait); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_standard_error, __pyx_cur_scope->__pyx_v_genome};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 449, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_standard_error, __pyx_cur_scope->__pyx_v_genome};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 449, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 449, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_standard_error);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_standard_error);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_cur_scope->__pyx_v_standard_error);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_genome);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_genome);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_cur_scope->__pyx_v_genome);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 449, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_individuals_traits);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_individuals_traits, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":450
 *         for genome in individuals_genome:
 *             individuals_traits = self._express_trait(standard_error, genome)
 *             yield individuals_traits             # <<<<<<<<<<<<<<
 * 
 *     def _run_exponential_smoothing_forecast(self, individual ):
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_individuals_traits);
    __pyx_r = __pyx_cur_scope->__pyx_v_individuals_traits;
    __Pyx_XGIVEREF(__pyx_t_1);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 450, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":448
 * 
 *         # stack parents and offspring into a list for next steps
 *         for genome in individuals_genome:             # <<<<<<<<<<<<<<
 *             individuals_traits = self._express_trait(standard_error, genome)
 *             yield individuals_traits
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supplychainpy/demand/_evo_algo.pyx":445
 *         return appraised_individual
 * 
 *     def express_smoothing_level_genome(self, individuals_genome: list, standard_error):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("express_smoothing_level_genome", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":452
 *             yield individuals_traits
 * 
 *     def _run_exponential_smoothing_forecast(self, individual ):             # <<<<<<<<<<<<<<
 * 
 *         f = Forecast(self.__orders, self.__average_order)
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_29_run_exponential_smoothing_forecast(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_29_run_exponential_smoothing_forecast = {"_run_exponential_smoothing_forecast", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_29_run_exponential_smoothing_forecast, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_29_run_exponential_smoothing_forecast(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_individual = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_run_exponential_smoothing_forecast (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individual,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individual)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_run_exponential_smoothing_forecast", 1, 2, 2, 1); __PYX_ERR(0, 452, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_run_exponential_smoothing_forecast") < 0)) __PYX_ERR(0, 452, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_individual = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_run_exponential_smoothing_forecast", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 452, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._run_exponential_smoothing_forecast", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_28_run_exponential_smoothing_forecast(__pyx_self, __pyx_v_self, __pyx_v_individual);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_28_run_exponential_smoothing_forecast(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_individual) {
  PyObject *__pyx_v_f = NULL;
  PyObject *__pyx_v_simple_expo_smoothing = NULL;
  PyObject *__pyx_v_sm_lvl = NULL;
  PyObject *__pyx_v_p = NULL;
  PyObject *__pyx_v_appraised_individual = NULL;
  PyObject *__pyx_v_smoothing_level = NULL;
  PyObject *__pyx_v_sum_squared_error = NULL;
  PyObject *__pyx_v_standard_error = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("_run_exponential_smoothing_forecast", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":454
 *     def _run_exponential_smoothing_forecast(self, individual ):
 * 
 *         f = Forecast(self.__orders, self.__average_order)             # <<<<<<<<<<<<<<
 *         simple_expo_smoothing = []
 *         for sm_lvl in individual:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Forecast); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_f = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":455
 * 
 *         f = Forecast(self.__orders, self.__average_order)
 *         simple_expo_smoothing = []             # <<<<<<<<<<<<<<
 *         for sm_lvl in individual:
 *             p = [i for i in f.simple_exponential_smoothing(sm_lvl)]
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_simple_expo_smoothing = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":456
 *         f = Forecast(self.__orders, self.__average_order)
 *         simple_expo_smoothing = []
 *         for sm_lvl in individual:             # <<<<<<<<<<<<<<
 *             p = [i for i in f.simple_exponential_smoothing(sm_lvl)]
 *             # print(p)
 */
  if (likely(PyList_CheckExact(__pyx_v_individual)) || PyTuple_CheckExact(__pyx_v_individual)) {
    __pyx_t_1 = __pyx_v_individual; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_individual); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 456, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 456, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 456, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 456, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 456, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_9(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 456, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_sm_lvl, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":457
 *         simple_expo_smoothing = []
 *         for sm_lvl in individual:
 *             p = [i for i in f.simple_exponential_smoothing(sm_lvl)]             # <<<<<<<<<<<<<<
 *             # print(p)
 *             simple_expo_smoothing.append(p)
 */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_simple_exponential_smoothing); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_sm_lvl); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 457, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_sm_lvl};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_sm_lvl};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_v_sm_lvl);
        __Pyx_GIVEREF(__pyx_v_sm_lvl);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_sm_lvl);
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 457, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
      __pyx_t_4 = __pyx_t_7; __Pyx_INCREF(__pyx_t_4); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_10 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 457, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_11 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 457, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_10); __Pyx_INCREF(__pyx_t_7); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 457, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 457, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_10); __Pyx_INCREF(__pyx_t_7); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 457, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 457, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_11(__pyx_t_4);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 457, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
      __pyx_t_7 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 457, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_p, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":459
 *             p = [i for i in f.simple_exponential_smoothing(sm_lvl)]
 *             # print(p)
 *             simple_expo_smoothing.append(p)             # <<<<<<<<<<<<<<
 * 
 *         appraised_individual = {}
 */
    __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_simple_expo_smoothing, __pyx_v_p); if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(0, 459, __pyx_L1_error)

    /* "supplychainpy/demand/_evo_algo.pyx":456
 *         f = Forecast(self.__orders, self.__average_order)
 *         simple_expo_smoothing = []
 *         for sm_lvl in individual:             # <<<<<<<<<<<<<<
 *             p = [i for i in f.simple_exponential_smoothing(sm_lvl)]
 *             # print(p)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":461
 *             simple_expo_smoothing.append(p)
 * 
 *         appraised_individual = {}             # <<<<<<<<<<<<<<
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi(simple_expo_smoothing, smoothing_level)
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_appraised_individual = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":462
 * 
 *         appraised_individual = {}
 *         for smoothing_level in individual:             # <<<<<<<<<<<<<<
 *             sum_squared_error = f.sum_squared_errors_indi(simple_expo_smoothing, smoothing_level)
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 */
  if (likely(PyList_CheckExact(__pyx_v_individual)) || PyTuple_CheckExact(__pyx_v_individual)) {
    __pyx_t_1 = __pyx_v_individual; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_individual); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 462, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 462, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 462, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 462, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 462, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_9(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 462, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_smoothing_level, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":463
 *         appraised_individual = {}
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi(simple_expo_smoothing, smoothing_level)             # <<<<<<<<<<<<<<
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *             appraised_individual.update({smoothing_level: standard_error})
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_sum_squared_errors_indi); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_simple_expo_smoothing, __pyx_v_smoothing_level};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_simple_expo_smoothing, __pyx_v_smoothing_level};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_simple_expo_smoothing);
      __Pyx_GIVEREF(__pyx_v_simple_expo_smoothing);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_6, __pyx_v_simple_expo_smoothing);
      __Pyx_INCREF(__pyx_v_smoothing_level);
      __Pyx_GIVEREF(__pyx_v_smoothing_level);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_6, __pyx_v_smoothing_level);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_sum_squared_error, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":464
 *         for smoothing_level in individual:
 *             sum_squared_error = f.sum_squared_errors_indi(simple_expo_smoothing, smoothing_level)
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)             # <<<<<<<<<<<<<<
 *             appraised_individual.update({smoothing_level: standard_error})
 *         # print('The standard error as a trait has been calculated {}'.format(appraised_individual))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_standard_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = PyObject_Length(__pyx_t_5); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_sum_squared_error, __pyx_t_5, __pyx_v_smoothing_level};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_sum_squared_error, __pyx_t_5, __pyx_v_smoothing_level};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_sum_squared_error);
      __Pyx_GIVEREF(__pyx_v_sum_squared_error);
      PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_sum_squared_error);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_t_5);
      __Pyx_INCREF(__pyx_v_smoothing_level);
      __Pyx_GIVEREF(__pyx_v_smoothing_level);
      PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_v_smoothing_level);
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_standard_error, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":465
 *             sum_squared_error = f.sum_squared_errors_indi(simple_expo_smoothing, smoothing_level)
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 *             appraised_individual.update({smoothing_level: standard_error})             # <<<<<<<<<<<<<<
 *         # print('The standard error as a trait has been calculated {}'.format(appraised_individual))
 *         return appraised_individual
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_appraised_individual, __pyx_n_s_update); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_v_smoothing_level, __pyx_v_standard_error) < 0) __PYX_ERR(0, 465, __pyx_L1_error)
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":462
 * 
 *         appraised_individual = {}
 *         for smoothing_level in individual:             # <<<<<<<<<<<<<<
 *             sum_squared_error = f.sum_squared_errors_indi(simple_expo_smoothing, smoothing_level)
 *             standard_error = f.standard_error(sum_squared_error, len(self.__orders), smoothing_level)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":467
 *             appraised_individual.update({smoothing_level: standard_error})
 *         # print('The standard error as a trait has been calculated {}'.format(appraised_individual))
 *         return appraised_individual             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_appraised_individual);
  __pyx_r = __pyx_v_appraised_individual;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":452
 *             yield individuals_traits
 * 
 *     def _run_exponential_smoothing_forecast(self, individual ):             # <<<<<<<<<<<<<<
 * 
 *         f = Forecast(self.__orders, self.__average_order)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._run_exponential_smoothing_forecast", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_simple_expo_smoothing);
  __Pyx_XDECREF(__pyx_v_sm_lvl);
  __Pyx_XDECREF(__pyx_v_p);
  __Pyx_XDECREF(__pyx_v_appraised_individual);
  __Pyx_XDECREF(__pyx_v_smoothing_level);
  __Pyx_XDECREF(__pyx_v_sum_squared_error);
  __Pyx_XDECREF(__pyx_v_standard_error);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":470
 * 
 *     @staticmethod
 *     def _express_trait(original_standard_error: float, appraised_individual: dict):             # <<<<<<<<<<<<<<
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 *         # print(original_standard_error)
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_31_express_trait(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_31_express_trait = {"_express_trait", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_31_express_trait, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_31_express_trait(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_original_standard_error = 0;
  PyObject *__pyx_v_appraised_individual = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_express_trait (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_original_standard_error,&__pyx_n_s_appraised_individual,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_original_standard_error)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_appraised_individual)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_express_trait", 1, 2, 2, 1); __PYX_ERR(0, 470, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_express_trait") < 0)) __PYX_ERR(0, 470, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_original_standard_error = values[0];
    __pyx_v_appraised_individual = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_express_trait", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 470, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._express_trait", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_30_express_trait(__pyx_self, __pyx_v_original_standard_error, __pyx_v_appraised_individual);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_30_express_trait(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_original_standard_error, PyObject *__pyx_v_appraised_individual) {
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("_express_trait", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":473
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 *         # print(original_standard_error)
 *         for key in appraised_individual:             # <<<<<<<<<<<<<<
 *             if appraised_individual[key] < original_standard_error:
 *                 appraised_individual[key] = 1
 */
  if (likely(PyList_CheckExact(__pyx_v_appraised_individual)) || PyTuple_CheckExact(__pyx_v_appraised_individual)) {
    __pyx_t_1 = __pyx_v_appraised_individual; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_appraised_individual); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 473, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 473, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 473, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 473, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 473, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 473, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":474
 *         # print(original_standard_error)
 *         for key in appraised_individual:
 *             if appraised_individual[key] < original_standard_error:             # <<<<<<<<<<<<<<
 *                 appraised_individual[key] = 1
 *             else:
 */
    __pyx_t_4 = PyObject_GetItem(__pyx_v_appraised_individual, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 474, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_v_original_standard_error, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 474, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 474, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_6) {

      /* "supplychainpy/demand/_evo_algo.pyx":475
 *         for key in appraised_individual:
 *             if appraised_individual[key] < original_standard_error:
 *                 appraised_individual[key] = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 appraised_individual[key] = 0
 */
      if (unlikely(PyObject_SetItem(__pyx_v_appraised_individual, __pyx_v_key, __pyx_int_1) < 0)) __PYX_ERR(0, 475, __pyx_L1_error)

      /* "supplychainpy/demand/_evo_algo.pyx":474
 *         # print(original_standard_error)
 *         for key in appraised_individual:
 *             if appraised_individual[key] < original_standard_error:             # <<<<<<<<<<<<<<
 *                 appraised_individual[key] = 1
 *             else:
 */
      goto __pyx_L5;
    }

    /* "supplychainpy/demand/_evo_algo.pyx":477
 *                 appraised_individual[key] = 1
 *             else:
 *                 appraised_individual[key] = 0             # <<<<<<<<<<<<<<
 *         # print('The traits have been verified {}'.format(appraised_individual))
 *         # check fitness of individual to procreate. 50% of traits need to be better smoothing_levels than the original
 */
    /*else*/ {
      if (unlikely(PyObject_SetItem(__pyx_v_appraised_individual, __pyx_v_key, __pyx_int_0) < 0)) __PYX_ERR(0, 477, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "supplychainpy/demand/_evo_algo.pyx":473
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 *         # print(original_standard_error)
 *         for key in appraised_individual:             # <<<<<<<<<<<<<<
 *             if appraised_individual[key] < original_standard_error:
 *                 appraised_individual[key] = 1
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":480
 *         # print('The traits have been verified {}'.format(appraised_individual))
 *         # check fitness of individual to procreate. 50% of traits need to be better smoothing_levels than the original
 *         return appraised_individual             # <<<<<<<<<<<<<<
 * 
 *     def _selection_population(self, individuals_fitness: dict, appraised_individual: list):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_appraised_individual);
  __pyx_r = __pyx_v_appraised_individual;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":470
 * 
 *     @staticmethod
 *     def _express_trait(original_standard_error: float, appraised_individual: dict):             # <<<<<<<<<<<<<<
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 *         # print(original_standard_error)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._express_trait", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":482
 *         return appraised_individual
 * 
 *     def _selection_population(self, individuals_fitness: dict, appraised_individual: list):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_33_selection_population(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_33_selection_population = {"_selection_population", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_33_selection_population, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_33_selection_population(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_individuals_fitness = 0;
  CYTHON_UNUSED PyObject *__pyx_v_appraised_individual = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_selection_population (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_individuals_fitness,&__pyx_n_s_appraised_individual,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_individuals_fitness)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_selection_population", 1, 3, 3, 1); __PYX_ERR(0, 482, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_appraised_individual)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_selection_population", 1, 3, 3, 2); __PYX_ERR(0, 482, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_selection_population") < 0)) __PYX_ERR(0, 482, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_individuals_fitness = values[1];
    __pyx_v_appraised_individual = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_selection_population", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 482, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm._selection_population", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_32_selection_population(__pyx_self, __pyx_v_self, __pyx_v_individuals_fitness, __pyx_v_appraised_individual);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_32_selection_population(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_individuals_fitness, CYTHON_UNUSED PyObject *__pyx_v_appraised_individual) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_selection_population", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supplychainpy/demand/_evo_algo.pyx":485
 *         pass
 * 
 *     def simple_exponential_smoothing_evo(self, smoothing_level_constant, initial_estimate_period,             # <<<<<<<<<<<<<<
 *                                          recombination_type = 'single_point', int population_size = 10,
 *                                          int forecast_length = 5):
 */

/* Python wrapper */
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_35simple_exponential_smoothing_evo(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_34simple_exponential_smoothing_evo[] = " Simple exponential smoothing using evolutionary algorithm for optimising smoothing level constant (alpha value)\n\n            Args:\n                initial_estimate_period (int):      The number of previous data points required for initial level estimate.\n                smoothing_level_constant (float):   Best guess at smoothing level constant appropriate for forecast.\n\n           Returns:\n               dict:\n\n            Example:\n\n        ";
static PyMethodDef __pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_35simple_exponential_smoothing_evo = {"simple_exponential_smoothing_evo", (PyCFunction)__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_35simple_exponential_smoothing_evo, METH_VARARGS|METH_KEYWORDS, __pyx_doc_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_34simple_exponential_smoothing_evo};
static PyObject *__pyx_pw_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_35simple_exponential_smoothing_evo(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_smoothing_level_constant = 0;
  PyObject *__pyx_v_initial_estimate_period = 0;
  PyObject *__pyx_v_recombination_type = 0;
  int __pyx_v_population_size;
  int __pyx_v_forecast_length;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("simple_exponential_smoothing_evo (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_smoothing_level_constant,&__pyx_n_s_initial_estimate_period,&__pyx_n_s_recombination_type,&__pyx_n_s_population_size,&__pyx_n_s_forecast_length,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[3] = ((PyObject *)((PyObject*)__pyx_n_s_single_point));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_smoothing_level_constant)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simple_exponential_smoothing_evo", 0, 3, 6, 1); __PYX_ERR(0, 485, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_initial_estimate_period)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simple_exponential_smoothing_evo", 0, 3, 6, 2); __PYX_ERR(0, 485, __pyx_L3_error)
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_recombination_type);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_population_size);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_forecast_length);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "simple_exponential_smoothing_evo") < 0)) __PYX_ERR(0, 485, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_smoothing_level_constant = values[1];
    __pyx_v_initial_estimate_period = values[2];
    __pyx_v_recombination_type = values[3];
    if (values[4]) {
      __pyx_v_population_size = __Pyx_PyInt_As_int(values[4]); if (unlikely((__pyx_v_population_size == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 486, __pyx_L3_error)
    } else {
      __pyx_v_population_size = ((int)((int)10));
    }
    if (values[5]) {
      __pyx_v_forecast_length = __Pyx_PyInt_As_int(values[5]); if (unlikely((__pyx_v_forecast_length == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L3_error)
    } else {
      __pyx_v_forecast_length = ((int)((int)5));
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("simple_exponential_smoothing_evo", 0, 3, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 485, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.simple_exponential_smoothing_evo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_34simple_exponential_smoothing_evo(__pyx_self, __pyx_v_self, __pyx_v_smoothing_level_constant, __pyx_v_initial_estimate_period, __pyx_v_recombination_type, __pyx_v_population_size, __pyx_v_forecast_length);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_34simple_exponential_smoothing_evo(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_smoothing_level_constant, PyObject *__pyx_v_initial_estimate_period, PyObject *__pyx_v_recombination_type, int __pyx_v_population_size, int __pyx_v_forecast_length) {
  PyObject *__pyx_v_sum_orders = NULL;
  PyObject *__pyx_v_demand = NULL;
  PyObject *__pyx_v_avg_orders = NULL;
  PyObject *__pyx_v_forecast_demand = NULL;
  PyObject *__pyx_v_ses_forecast = NULL;
  PyObject *__pyx_v_sum_squared_error = NULL;
  PyObject *__pyx_v_standard_error = NULL;
  PyObject *__pyx_v_evo_mod = NULL;
  PyObject *__pyx_v_optimal_alpha = NULL;
  PyObject *__pyx_v_optimal_ses_forecast = NULL;
  PyObject *__pyx_v_ape = NULL;
  PyObject *__pyx_v_mape = NULL;
  PyObject *__pyx_v_stats = NULL;
  PyObject *__pyx_v_simple_forecast = NULL;
  PyObject *__pyx_v_regression = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("simple_exponential_smoothing_evo", 0);
  __Pyx_INCREF(__pyx_v_recombination_type);

  /* "supplychainpy/demand/_evo_algo.pyx":500
 * 
 *         """
 *         if None != self.__recombination_type:             # <<<<<<<<<<<<<<
 *             recombination_type = self.__recombination_type
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(Py_None, __pyx_t_1, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "supplychainpy/demand/_evo_algo.pyx":501
 *         """
 *         if None != self.__recombination_type:
 *             recombination_type = self.__recombination_type             # <<<<<<<<<<<<<<
 * 
 *         sum_orders = 0
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 501, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_recombination_type, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":500
 * 
 *         """
 *         if None != self.__recombination_type:             # <<<<<<<<<<<<<<
 *             recombination_type = self.__recombination_type
 * 
 */
  }

  /* "supplychainpy/demand/_evo_algo.pyx":503
 *             recombination_type = self.__recombination_type
 * 
 *         sum_orders = 0             # <<<<<<<<<<<<<<
 * 
 *         for demand in self.__orders[:initial_estimate_period]:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_sum_orders = __pyx_int_0;

  /* "supplychainpy/demand/_evo_algo.pyx":505
 *         sum_orders = 0
 * 
 *         for demand in self.__orders[:initial_estimate_period]:             # <<<<<<<<<<<<<<
 *             sum_orders += demand
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_t_2, 0, 0, NULL, &__pyx_v_initial_estimate_period, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 505, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 505, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 505, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_demand, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":506
 * 
 *         for demand in self.__orders[:initial_estimate_period]:
 *             sum_orders += demand             # <<<<<<<<<<<<<<
 * 
 *         avg_orders = sum_orders / initial_estimate_period
 */
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_sum_orders, __pyx_v_demand); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_sum_orders, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "supplychainpy/demand/_evo_algo.pyx":505
 *         sum_orders = 0
 * 
 *         for demand in self.__orders[:initial_estimate_period]:             # <<<<<<<<<<<<<<
 *             sum_orders += demand
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":508
 *             sum_orders += demand
 * 
 *         avg_orders = sum_orders / initial_estimate_period             # <<<<<<<<<<<<<<
 * 
 *         forecast_demand = Forecast(self.__orders, avg_orders)
 */
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_sum_orders, __pyx_v_initial_estimate_period); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_avg_orders = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":510
 *         avg_orders = sum_orders / initial_estimate_period
 * 
 *         forecast_demand = Forecast(self.__orders, avg_orders)             # <<<<<<<<<<<<<<
 * 
 *         ses_forecast = [i for i in forecast_demand.simple_exponential_smoothing(*(smoothing_level_constant,))]
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_Forecast); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_avg_orders};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_avg_orders};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_avg_orders);
    __Pyx_GIVEREF(__pyx_v_avg_orders);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_avg_orders);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_forecast_demand = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":512
 *         forecast_demand = Forecast(self.__orders, avg_orders)
 * 
 *         ses_forecast = [i for i in forecast_demand.simple_exponential_smoothing(*(smoothing_level_constant,))]             # <<<<<<<<<<<<<<
 * 
 *         sum_squared_error = forecast_demand.sum_squared_errors(ses_forecast, smoothing_level_constant)
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast_demand, __pyx_n_s_simple_exponential_smoothing); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_v_smoothing_level_constant);
  __Pyx_GIVEREF(__pyx_v_smoothing_level_constant);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_smoothing_level_constant);
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
    __pyx_t_9 = __pyx_t_6; __Pyx_INCREF(__pyx_t_9); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_5 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 512, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_9))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_9)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 512, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_9, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 512, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 512, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_9, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 512, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_5(__pyx_t_9);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 512, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_6);
    __pyx_t_6 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 512, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_ses_forecast = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":514
 *         ses_forecast = [i for i in forecast_demand.simple_exponential_smoothing(*(smoothing_level_constant,))]
 * 
 *         sum_squared_error = forecast_demand.sum_squared_errors(ses_forecast, smoothing_level_constant)             # <<<<<<<<<<<<<<
 * 
 *         standard_error = forecast_demand.standard_error(sum_squared_error, len(self.__orders),
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast_demand, __pyx_n_s_sum_squared_errors); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_6 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_ses_forecast, __pyx_v_smoothing_level_constant};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_ses_forecast, __pyx_v_smoothing_level_constant};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_ses_forecast);
    __Pyx_GIVEREF(__pyx_v_ses_forecast);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_8, __pyx_v_ses_forecast);
    __Pyx_INCREF(__pyx_v_smoothing_level_constant);
    __Pyx_GIVEREF(__pyx_v_smoothing_level_constant);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_8, __pyx_v_smoothing_level_constant);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_sum_squared_error = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":516
 *         sum_squared_error = forecast_demand.sum_squared_errors(ses_forecast, smoothing_level_constant)
 * 
 *         standard_error = forecast_demand.standard_error(sum_squared_error, len(self.__orders),             # <<<<<<<<<<<<<<
 *                                                         smoothing_level_constant)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast_demand, __pyx_n_s_standard_error); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_4 == -1)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":517
 * 
 *         standard_error = forecast_demand.standard_error(sum_squared_error, len(self.__orders),
 *                                                         smoothing_level_constant)             # <<<<<<<<<<<<<<
 * 
 *         evo_mod = OptimiseSmoothingLevelGeneticAlgorithm(orders=self.__orders,
 */
  __pyx_t_6 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_sum_squared_error, __pyx_t_1, __pyx_v_smoothing_level_constant};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_sum_squared_error, __pyx_t_1, __pyx_v_smoothing_level_constant};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_sum_squared_error);
    __Pyx_GIVEREF(__pyx_v_sum_squared_error);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_8, __pyx_v_sum_squared_error);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_8, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_smoothing_level_constant);
    __Pyx_GIVEREF(__pyx_v_smoothing_level_constant);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_8, __pyx_v_smoothing_level_constant);
    __pyx_t_1 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_standard_error = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":519
 *                                                         smoothing_level_constant)
 * 
 *         evo_mod = OptimiseSmoothingLevelGeneticAlgorithm(orders=self.__orders,             # <<<<<<<<<<<<<<
 *                                                          average_order=avg_orders,
 *                                                          smoothing_level=smoothing_level_constant,
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = PyDict_New(); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_OptimiseSmoothingLevelGeneticAl); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_orders, __pyx_t_7) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":520
 * 
 *         evo_mod = OptimiseSmoothingLevelGeneticAlgorithm(orders=self.__orders,
 *                                                          average_order=avg_orders,             # <<<<<<<<<<<<<<
 *                                                          smoothing_level=smoothing_level_constant,
 *                                                          population_size=population_size,
 */
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_average_order, __pyx_v_avg_orders) < 0) __PYX_ERR(0, 519, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":521
 *         evo_mod = OptimiseSmoothingLevelGeneticAlgorithm(orders=self.__orders,
 *                                                          average_order=avg_orders,
 *                                                          smoothing_level=smoothing_level_constant,             # <<<<<<<<<<<<<<
 *                                                          population_size=population_size,
 *                                                          standard_error=standard_error,
 */
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_smoothing_level, __pyx_v_smoothing_level_constant) < 0) __PYX_ERR(0, 519, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":522
 *                                                          average_order=avg_orders,
 *                                                          smoothing_level=smoothing_level_constant,
 *                                                          population_size=population_size,             # <<<<<<<<<<<<<<
 *                                                          standard_error=standard_error,
 *                                                          recombination_type=recombination_type)
 */
  __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_population_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_population_size, __pyx_t_7) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":523
 *                                                          smoothing_level=smoothing_level_constant,
 *                                                          population_size=population_size,
 *                                                          standard_error=standard_error,             # <<<<<<<<<<<<<<
 *                                                          recombination_type=recombination_type)
 * 
 */
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_standard_error, __pyx_v_standard_error) < 0) __PYX_ERR(0, 519, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":524
 *                                                          population_size=population_size,
 *                                                          standard_error=standard_error,
 *                                                          recombination_type=recombination_type)             # <<<<<<<<<<<<<<
 * 
 *         optimal_alpha = evo_mod.initial_population()
 */
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_recombination_type, __pyx_v_recombination_type) < 0) __PYX_ERR(0, 519, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":519
 *                                                         smoothing_level_constant)
 * 
 *         evo_mod = OptimiseSmoothingLevelGeneticAlgorithm(orders=self.__orders,             # <<<<<<<<<<<<<<
 *                                                          average_order=avg_orders,
 *                                                          smoothing_level=smoothing_level_constant,
 */
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_evo_mod = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":526
 *                                                          recombination_type=recombination_type)
 * 
 *         optimal_alpha = evo_mod.initial_population()             # <<<<<<<<<<<<<<
 * 
 *         optimal_ses_forecast = [i for i in forecast_demand.simple_exponential_smoothing(optimal_alpha[1])]
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_evo_mod, __pyx_n_s_initial_population); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 526, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 526, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_optimal_alpha = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":528
 *         optimal_alpha = evo_mod.initial_population()
 * 
 *         optimal_ses_forecast = [i for i in forecast_demand.simple_exponential_smoothing(optimal_alpha[1])]             # <<<<<<<<<<<<<<
 * 
 *         ape = LinearRegression(optimal_ses_forecast)
 */
  __pyx_t_7 = PyList_New(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast_demand, __pyx_n_s_simple_exponential_smoothing); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_optimal_alpha, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_9);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_1};
      __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_1};
      __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_9)) || PyTuple_CheckExact(__pyx_t_9)) {
    __pyx_t_2 = __pyx_t_9; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 528, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_9 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_9); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 528, __pyx_L1_error)
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_9); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 528, __pyx_L1_error)
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
      }
    } else {
      __pyx_t_9 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_9)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 528, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_9);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_9);
    __pyx_t_9 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_7, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 528, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_optimal_ses_forecast = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":530
 *         optimal_ses_forecast = [i for i in forecast_demand.simple_exponential_smoothing(optimal_alpha[1])]
 * 
 *         ape = LinearRegression(optimal_ses_forecast)             # <<<<<<<<<<<<<<
 *         mape = forecast_demand.mean_aboslute_percentage_error_opt(optimal_ses_forecast)
 *         stats = ape.least_squared_error()
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_LinearRegression); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_optimal_ses_forecast); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 530, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_v_optimal_ses_forecast};
      __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 530, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_v_optimal_ses_forecast};
      __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 530, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 530, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
      __Pyx_INCREF(__pyx_v_optimal_ses_forecast);
      __Pyx_GIVEREF(__pyx_v_optimal_ses_forecast);
      PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_v_optimal_ses_forecast);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 530, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ape = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":531
 * 
 *         ape = LinearRegression(optimal_ses_forecast)
 *         mape = forecast_demand.mean_aboslute_percentage_error_opt(optimal_ses_forecast)             # <<<<<<<<<<<<<<
 *         stats = ape.least_squared_error()
 *         simple_forecast = forecast_demand.simple_exponential_smoothing_forecast(forecast=optimal_ses_forecast,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast_demand, __pyx_n_s_mean_aboslute_percentage_error_o); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_optimal_ses_forecast); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 531, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_v_optimal_ses_forecast};
      __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 531, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_v_optimal_ses_forecast};
      __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 531, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 531, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_10); __pyx_t_10 = NULL;
      __Pyx_INCREF(__pyx_v_optimal_ses_forecast);
      __Pyx_GIVEREF(__pyx_v_optimal_ses_forecast);
      PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_v_optimal_ses_forecast);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 531, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_mape = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":532
 *         ape = LinearRegression(optimal_ses_forecast)
 *         mape = forecast_demand.mean_aboslute_percentage_error_opt(optimal_ses_forecast)
 *         stats = ape.least_squared_error()             # <<<<<<<<<<<<<<
 *         simple_forecast = forecast_demand.simple_exponential_smoothing_forecast(forecast=optimal_ses_forecast,
 *                                                                                 forecast_length=forecast_length)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_ape, __pyx_n_s_least_squared_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_9) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else {
    __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 532, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_stats = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":533
 *         mape = forecast_demand.mean_aboslute_percentage_error_opt(optimal_ses_forecast)
 *         stats = ape.least_squared_error()
 *         simple_forecast = forecast_demand.simple_exponential_smoothing_forecast(forecast=optimal_ses_forecast,             # <<<<<<<<<<<<<<
 *                                                                                 forecast_length=forecast_length)
 *         regression = {
 */
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_forecast_demand, __pyx_n_s_simple_exponential_smoothing_for); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_forecast, __pyx_v_optimal_ses_forecast) < 0) __PYX_ERR(0, 533, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":534
 *         stats = ape.least_squared_error()
 *         simple_forecast = forecast_demand.simple_exponential_smoothing_forecast(forecast=optimal_ses_forecast,
 *                                                                                 forecast_length=forecast_length)             # <<<<<<<<<<<<<<
 *         regression = {
 *             'regression': [(stats.get('slope') * i) + stats.get('intercept') for i in range(0, 12)]}
 */
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_forecast_length); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_forecast_length, __pyx_t_9) < 0) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":533
 *         mape = forecast_demand.mean_aboslute_percentage_error_opt(optimal_ses_forecast)
 *         stats = ape.least_squared_error()
 *         simple_forecast = forecast_demand.simple_exponential_smoothing_forecast(forecast=optimal_ses_forecast,             # <<<<<<<<<<<<<<
 *                                                                                 forecast_length=forecast_length)
 *         regression = {
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_simple_forecast = __pyx_t_9;
  __pyx_t_9 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":536
 *                                                                                 forecast_length=forecast_length)
 *         regression = {
 *             'regression': [(stats.get('slope') * i) + stats.get('intercept') for i in range(0, 12)]}             # <<<<<<<<<<<<<<
 *         return {'forecast_breakdown': optimal_ses_forecast, 'mape': mape, 'statistics': stats,
 *                 'forecast': simple_forecast, 'optimal_alpha': optimal_alpha[1], 'standard_error': standard_error,
 */
  __pyx_t_9 = PyDict_New(); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
    __pyx_t_10 = __pyx_t_7; __Pyx_INCREF(__pyx_t_10); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_5 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 536, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_10))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_10)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_4); __Pyx_INCREF(__pyx_t_7); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 536, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_10, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_4); __Pyx_INCREF(__pyx_t_7); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 536, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_10, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_5(__pyx_t_10);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 536, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_stats, __pyx_n_s_get); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyNumber_Multiply(__pyx_t_1, __pyx_v_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_stats, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Add(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_1))) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_regression, __pyx_t_2) < 0) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_regression = ((PyObject*)__pyx_t_9);
  __pyx_t_9 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":537
 *         regression = {
 *             'regression': [(stats.get('slope') * i) + stats.get('intercept') for i in range(0, 12)]}
 *         return {'forecast_breakdown': optimal_ses_forecast, 'mape': mape, 'statistics': stats,             # <<<<<<<<<<<<<<
 *                 'forecast': simple_forecast, 'optimal_alpha': optimal_alpha[1], 'standard_error': standard_error,
 *                 'regression': [i for i in regression.get('regression')]}
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = PyDict_New(); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_forecast_breakdown, __pyx_v_optimal_ses_forecast) < 0) __PYX_ERR(0, 537, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_mape, __pyx_v_mape) < 0) __PYX_ERR(0, 537, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_statistics, __pyx_v_stats) < 0) __PYX_ERR(0, 537, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":538
 *             'regression': [(stats.get('slope') * i) + stats.get('intercept') for i in range(0, 12)]}
 *         return {'forecast_breakdown': optimal_ses_forecast, 'mape': mape, 'statistics': stats,
 *                 'forecast': simple_forecast, 'optimal_alpha': optimal_alpha[1], 'standard_error': standard_error,             # <<<<<<<<<<<<<<
 *                 'regression': [i for i in regression.get('regression')]}
 */
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_forecast, __pyx_v_simple_forecast) < 0) __PYX_ERR(0, 537, __pyx_L1_error)
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_optimal_alpha, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_optimal_alpha, __pyx_t_2) < 0) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_standard_error, __pyx_v_standard_error) < 0) __PYX_ERR(0, 537, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":539
 *         return {'forecast_breakdown': optimal_ses_forecast, 'mape': mape, 'statistics': stats,
 *                 'forecast': simple_forecast, 'optimal_alpha': optimal_alpha[1], 'standard_error': standard_error,
 *                 'regression': [i for i in regression.get('regression')]}             # <<<<<<<<<<<<<<
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyDict_GetItemDefault(__pyx_v_regression, __pyx_n_s_regression, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
    __pyx_t_1 = __pyx_t_10; __Pyx_INCREF(__pyx_t_1); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 539, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_10); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 539, __pyx_L1_error)
        #else
        __pyx_t_10 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_10); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 539, __pyx_L1_error)
        #else
        __pyx_t_10 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
      }
    } else {
      __pyx_t_10 = __pyx_t_5(__pyx_t_1);
      if (unlikely(!__pyx_t_10)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 539, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_10);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_10);
    __pyx_t_10 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_v_i))) __PYX_ERR(0, 539, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_regression, __pyx_t_2) < 0) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "supplychainpy/demand/_evo_algo.pyx":485
 *         pass
 * 
 *     def simple_exponential_smoothing_evo(self, smoothing_level_constant, initial_estimate_period,             # <<<<<<<<<<<<<<
 *                                          recombination_type = 'single_point', int population_size = 10,
 *                                          int forecast_length = 5):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("supplychainpy.demand._evo_algo.OptimiseSmoothingLevelGeneticAlgorithm.simple_exponential_smoothing_evo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sum_orders);
  __Pyx_XDECREF(__pyx_v_demand);
  __Pyx_XDECREF(__pyx_v_avg_orders);
  __Pyx_XDECREF(__pyx_v_forecast_demand);
  __Pyx_XDECREF(__pyx_v_ses_forecast);
  __Pyx_XDECREF(__pyx_v_sum_squared_error);
  __Pyx_XDECREF(__pyx_v_standard_error);
  __Pyx_XDECREF(__pyx_v_evo_mod);
  __Pyx_XDECREF(__pyx_v_optimal_alpha);
  __Pyx_XDECREF(__pyx_v_optimal_ses_forecast);
  __Pyx_XDECREF(__pyx_v_ape);
  __Pyx_XDECREF(__pyx_v_mape);
  __Pyx_XDECREF(__pyx_v_stats);
  __Pyx_XDECREF(__pyx_v_simple_forecast);
  __Pyx_XDECREF(__pyx_v_regression);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_recombination_type);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_i);
  Py_CLEAR(p->__pyx_v_recombination_type);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)o;
  if (p->__pyx_v_i) {
    e = (*v)(p->__pyx_v_i, a); if (e) return e;
  }
  if (p->__pyx_v_recombination_type) {
    e = (*v)(p->__pyx_v_recombination_type, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(p->__pyx_v_self, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce *)o;
  tmp = ((PyObject*)p->__pyx_v_i);
  p->__pyx_v_i = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_recombination_type);
  p->__pyx_v_recombination_type = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct__reproduce", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_index);
  Py_CLEAR(p->__pyx_v_individual);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_new_individual);
  Py_CLEAR(p->__pyx_v_new_individual_two);
  Py_CLEAR(p->__pyx_v_number_of_mutations_allowed);
  Py_CLEAR(p->__pyx_v_population_allele_count);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_value);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  Py_CLEAR(p->__pyx_t_3);
  Py_CLEAR(p->__pyx_t_5);
  Py_CLEAR(p->__pyx_t_7);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)o;
  if (p->__pyx_v_index) {
    e = (*v)(p->__pyx_v_index, a); if (e) return e;
  }
  if (p->__pyx_v_individual) {
    e = (*v)(p->__pyx_v_individual, a); if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_new_individual) {
    e = (*v)(p->__pyx_v_new_individual, a); if (e) return e;
  }
  if (p->__pyx_v_new_individual_two) {
    e = (*v)(p->__pyx_v_new_individual_two, a); if (e) return e;
  }
  if (p->__pyx_v_number_of_mutations_allowed) {
    e = (*v)(p->__pyx_v_number_of_mutations_allowed, a); if (e) return e;
  }
  if (p->__pyx_v_population_allele_count) {
    e = (*v)(p->__pyx_v_population_allele_count, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(p->__pyx_v_self, a); if (e) return e;
  }
  if (p->__pyx_v_value) {
    e = (*v)(p->__pyx_v_value, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  if (p->__pyx_t_3) {
    e = (*v)(p->__pyx_t_3, a); if (e) return e;
  }
  if (p->__pyx_t_5) {
    e = (*v)(p->__pyx_t_5, a); if (e) return e;
  }
  if (p->__pyx_t_7) {
    e = (*v)(p->__pyx_t_7, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination *)o;
  tmp = ((PyObject*)p->__pyx_v_index);
  p->__pyx_v_index = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_individual);
  p->__pyx_v_individual = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_key);
  p->__pyx_v_key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_new_individual);
  p->__pyx_v_new_individual = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_new_individual_two);
  p->__pyx_v_new_individual_two = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_number_of_mutations_allowed);
  p->__pyx_v_number_of_mutations_allowed = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_population_allele_count);
  p->__pyx_v_population_allele_count = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_value);
  p->__pyx_v_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_1);
  p->__pyx_t_1 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_2);
  p->__pyx_t_2 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_3);
  p->__pyx_t_3 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_5);
  p->__pyx_t_5 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_7);
  p->__pyx_t_7 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_1__single_point_crossover_recombination", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_e);
  Py_CLEAR(p->__pyx_v_index);
  Py_CLEAR(p->__pyx_v_individual);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_new_individual);
  Py_CLEAR(p->__pyx_v_new_individual_two);
  Py_CLEAR(p->__pyx_v_number_of_mutations_allowed);
  Py_CLEAR(p->__pyx_v_population_allele_count);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_value);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  Py_CLEAR(p->__pyx_t_3);
  Py_CLEAR(p->__pyx_t_5);
  Py_CLEAR(p->__pyx_t_7);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)o;
  if (p->__pyx_v_e) {
    e = (*v)(p->__pyx_v_e, a); if (e) return e;
  }
  if (p->__pyx_v_index) {
    e = (*v)(p->__pyx_v_index, a); if (e) return e;
  }
  if (p->__pyx_v_individual) {
    e = (*v)(p->__pyx_v_individual, a); if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_new_individual) {
    e = (*v)(p->__pyx_v_new_individual, a); if (e) return e;
  }
  if (p->__pyx_v_new_individual_two) {
    e = (*v)(p->__pyx_v_new_individual_two, a); if (e) return e;
  }
  if (p->__pyx_v_number_of_mutations_allowed) {
    e = (*v)(p->__pyx_v_number_of_mutations_allowed, a); if (e) return e;
  }
  if (p->__pyx_v_population_allele_count) {
    e = (*v)(p->__pyx_v_population_allele_count, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(p->__pyx_v_self, a); if (e) return e;
  }
  if (p->__pyx_v_value) {
    e = (*v)(p->__pyx_v_value, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  if (p->__pyx_t_3) {
    e = (*v)(p->__pyx_t_3, a); if (e) return e;
  }
  if (p->__pyx_t_5) {
    e = (*v)(p->__pyx_t_5, a); if (e) return e;
  }
  if (p->__pyx_t_7) {
    e = (*v)(p->__pyx_t_7, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination *)o;
  tmp = ((PyObject*)p->__pyx_v_e);
  p->__pyx_v_e = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_index);
  p->__pyx_v_index = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_individual);
  p->__pyx_v_individual = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_key);
  p->__pyx_v_key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_new_individual);
  p->__pyx_v_new_individual = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_new_individual_two);
  p->__pyx_v_new_individual_two = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_number_of_mutations_allowed);
  p->__pyx_v_number_of_mutations_allowed = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_population_allele_count);
  p->__pyx_v_population_allele_count = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_value);
  p->__pyx_v_value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_1);
  p->__pyx_t_1 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_2);
  p->__pyx_t_2 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_3);
  p->__pyx_t_3 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_5);
  p->__pyx_t_5 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_7);
  p->__pyx_t_7 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_2__two_point_crossover_recombination", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_original_gene);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *)o;
  if (p->__pyx_v_original_gene) {
    e = (*v)(p->__pyx_v_original_gene, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation *)o;
  tmp = ((PyObject*)p->__pyx_v_original_gene);
  p->__pyx_v_original_gene = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_3__mutation", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_genome);
  Py_CLEAR(p->__pyx_v_new_individual);
  Py_CLEAR(p->__pyx_v_new_population);
  Py_CLEAR(p->__pyx_v_parent_offspring_population);
  Py_CLEAR(p->__pyx_v_pke);
  Py_CLEAR(p->__pyx_v_po);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)o;
  if (p->__pyx_v_genome) {
    e = (*v)(p->__pyx_v_genome, a); if (e) return e;
  }
  if (p->__pyx_v_new_individual) {
    e = (*v)(p->__pyx_v_new_individual, a); if (e) return e;
  }
  if (p->__pyx_v_new_population) {
    e = (*v)(p->__pyx_v_new_population, a); if (e) return e;
  }
  if (p->__pyx_v_parent_offspring_population) {
    e = (*v)(p->__pyx_v_parent_offspring_population, a); if (e) return e;
  }
  if (p->__pyx_v_pke) {
    e = (*v)(p->__pyx_v_pke, a); if (e) return e;
  }
  if (p->__pyx_v_po) {
    e = (*v)(p->__pyx_v_po, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals *)o;
  tmp = ((PyObject*)p->__pyx_v_genome);
  p->__pyx_v_genome = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_new_individual);
  p->__pyx_v_new_individual = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_new_population);
  p->__pyx_v_new_population = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_parent_offspring_population);
  p->__pyx_v_parent_offspring_population = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_pke);
  p->__pyx_v_pke = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_po);
  p->__pyx_v_po = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_4_create_individuals", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_individual);
  Py_CLEAR(p->__pyx_v_individual_type);
  Py_CLEAR(p->__pyx_v_population);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)o;
  if (p->__pyx_v_individual) {
    e = (*v)(p->__pyx_v_individual, a); if (e) return e;
  }
  if (p->__pyx_v_individual_type) {
    e = (*v)(p->__pyx_v_individual_type, a); if (e) return e;
  }
  if (p->__pyx_v_population) {
    e = (*v)(p->__pyx_v_population, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness *)o;
  tmp = ((PyObject*)p->__pyx_v_individual);
  p->__pyx_v_individual = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_individual_type);
  p->__pyx_v_individual_type = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_population);
  p->__pyx_v_population = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_5__population_fitness", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_individual_type);
  Py_CLEAR(p->__pyx_v_individuals_genome);
  Py_CLEAR(p->__pyx_v_parent);
  Py_CLEAR(p->__pyx_v_population);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)o;
  if (p->__pyx_v_individual_type) {
    e = (*v)(p->__pyx_v_individual_type, a); if (e) return e;
  }
  if (p->__pyx_v_individuals_genome) {
    e = (*v)(p->__pyx_v_individuals_genome, a); if (e) return e;
  }
  if (p->__pyx_v_parent) {
    e = (*v)(p->__pyx_v_parent, a); if (e) return e;
  }
  if (p->__pyx_v_population) {
    e = (*v)(p->__pyx_v_population, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(p->__pyx_v_self, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome *)o;
  tmp = ((PyObject*)p->__pyx_v_individual_type);
  p->__pyx_v_individual_type = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_individuals_genome);
  p->__pyx_v_individuals_genome = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_parent);
  p->__pyx_v_parent = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_population);
  p->__pyx_v_population = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_6_generate_smoothing_level_genome", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome[8];
static int __pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome = 0;

static PyObject *__pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome)))) {
    o = (PyObject*)__pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome[--__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome];
    memset(o, 0, sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome(PyObject *o) {
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_genome);
  Py_CLEAR(p->__pyx_v_individuals_genome);
  Py_CLEAR(p->__pyx_v_individuals_traits);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_standard_error);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome)))) {
    __pyx_freelist_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome[__pyx_freecount_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome++] = ((struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)o;
  if (p->__pyx_v_genome) {
    e = (*v)(p->__pyx_v_genome, a); if (e) return e;
  }
  if (p->__pyx_v_individuals_genome) {
    e = (*v)(p->__pyx_v_individuals_genome, a); if (e) return e;
  }
  if (p->__pyx_v_individuals_traits) {
    e = (*v)(p->__pyx_v_individuals_traits, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(p->__pyx_v_self, a); if (e) return e;
  }
  if (p->__pyx_v_standard_error) {
    e = (*v)(p->__pyx_v_standard_error, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *p = (struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome *)o;
  tmp = ((PyObject*)p->__pyx_v_genome);
  p->__pyx_v_genome = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_individuals_genome);
  p->__pyx_v_individuals_genome = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_individuals_traits);
  p->__pyx_v_individuals_traits = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_standard_error);
  p->__pyx_v_standard_error = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome = {
  PyVarObject_HEAD_INIT(0, 0)
  "supplychainpy.demand._evo_algo.__pyx_scope_struct_7_express_smoothing_level_genome", /*tp_name*/
  sizeof(struct __pyx_obj_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome, /*tp_traverse*/
  __pyx_tp_clear_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_evo_algo",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_Create_a_population_of_individu, __pyx_k_Create_a_population_of_individu, sizeof(__pyx_k_Create_a_population_of_individu), 0, 0, 1, 0},
  {&__pyx_n_s_Forecast, __pyx_k_Forecast, sizeof(__pyx_k_Forecast), 0, 0, 1, 1},
  {&__pyx_n_s_Individual, __pyx_k_Individual, sizeof(__pyx_k_Individual), 0, 0, 1, 1},
  {&__pyx_n_s_Individual___genome_generator, __pyx_k_Individual___genome_generator, sizeof(__pyx_k_Individual___genome_generator), 0, 0, 1, 1},
  {&__pyx_n_s_Individual___init, __pyx_k_Individual___init, sizeof(__pyx_k_Individual___init), 0, 0, 1, 1},
  {&__pyx_n_s_Individual___repr, __pyx_k_Individual___repr, sizeof(__pyx_k_Individual___repr), 0, 0, 1, 1},
  {&__pyx_n_s_Individual__genome_generator, __pyx_k_Individual__genome_generator, sizeof(__pyx_k_Individual__genome_generator), 0, 0, 1, 1},
  {&__pyx_n_s_Individual_gene_count, __pyx_k_Individual_gene_count, sizeof(__pyx_k_Individual_gene_count), 0, 0, 1, 1},
  {&__pyx_n_s_Individual_genome, __pyx_k_Individual_genome, sizeof(__pyx_k_Individual_genome), 0, 0, 1, 1},
  {&__pyx_n_s_Individual_name, __pyx_k_Individual_name, sizeof(__pyx_k_Individual_name), 0, 0, 1, 1},
  {&__pyx_n_s_LinearRegression, __pyx_k_LinearRegression, sizeof(__pyx_k_LinearRegression), 0, 0, 1, 1},
  {&__pyx_n_s_OSError, __pyx_k_OSError, sizeof(__pyx_k_OSError), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAl, __pyx_k_OptimiseSmoothingLevelGeneticAl, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAl), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAl_2, __pyx_k_OptimiseSmoothingLevelGeneticAl_2, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAl_2), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAl_3, __pyx_k_OptimiseSmoothingLevelGeneticAl_3, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAl_3), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAl_4, __pyx_k_OptimiseSmoothingLevelGeneticAl_4, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAl_4), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAl_5, __pyx_k_OptimiseSmoothingLevelGeneticAl_5, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAl_5), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAl_6, __pyx_k_OptimiseSmoothingLevelGeneticAl_6, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAl_6), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg, __pyx_k_OptimiseSmoothingLevelGeneticAlg, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_10, __pyx_k_OptimiseSmoothingLevelGeneticAlg_10, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_10), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_11, __pyx_k_OptimiseSmoothingLevelGeneticAlg_11, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_11), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_12, __pyx_k_OptimiseSmoothingLevelGeneticAlg_12, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_12), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_13, __pyx_k_OptimiseSmoothingLevelGeneticAlg_13, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_13), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_14, __pyx_k_OptimiseSmoothingLevelGeneticAlg_14, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_14), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_15, __pyx_k_OptimiseSmoothingLevelGeneticAlg_15, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_15), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_16, __pyx_k_OptimiseSmoothingLevelGeneticAlg_16, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_16), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_2, __pyx_k_OptimiseSmoothingLevelGeneticAlg_2, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_2), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_3, __pyx_k_OptimiseSmoothingLevelGeneticAlg_3, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_3), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_4, __pyx_k_OptimiseSmoothingLevelGeneticAlg_4, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_4), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5, __pyx_k_OptimiseSmoothingLevelGeneticAlg_5, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_5), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_6, __pyx_k_OptimiseSmoothingLevelGeneticAlg_6, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_6), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_7, __pyx_k_OptimiseSmoothingLevelGeneticAlg_7, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_7), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_8, __pyx_k_OptimiseSmoothingLevelGeneticAlg_8, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_8), 0, 0, 1, 1},
  {&__pyx_n_s_OptimiseSmoothingLevelGeneticAlg_9, __pyx_k_OptimiseSmoothingLevelGeneticAlg_9, sizeof(__pyx_k_OptimiseSmoothingLevelGeneticAlg_9), 0, 0, 1, 1},
  {&__pyx_n_s_Population, __pyx_k_Population, sizeof(__pyx_k_Population), 0, 0, 1, 1},
  {&__pyx_kp_s_Population_Size_is_too_small_for, __pyx_k_Population_Size_is_too_small_for, sizeof(__pyx_k_Population_Size_is_too_small_for), 0, 0, 1, 0},
  {&__pyx_n_s_Population___init, __pyx_k_Population___init, sizeof(__pyx_k_Population___init), 0, 0, 1, 1},
  {&__pyx_n_s_Population__mutation, __pyx_k_Population__mutation, sizeof(__pyx_k_Population__mutation), 0, 0, 1, 1},
  {&__pyx_n_s_Population__mutation_locals_muta, __pyx_k_Population__mutation_locals_muta, sizeof(__pyx_k_Population__mutation_locals_muta), 0, 0, 1, 1},
  {&__pyx_n_s_Population__single_point_crossov, __pyx_k_Population__single_point_crossov, sizeof(__pyx_k_Population__single_point_crossov), 0, 0, 1, 1},
  {&__pyx_n_s_Population__two_point_crossover, __pyx_k_Population__two_point_crossover, sizeof(__pyx_k_Population__two_point_crossover), 0, 0, 1, 1},
  {&__pyx_n_s_Population__uniform_crossover_re, __pyx_k_Population__uniform_crossover_re, sizeof(__pyx_k_Population__uniform_crossover_re), 0, 0, 1, 1},
  {&__pyx_n_s_Population_reproduce, __pyx_k_Population_reproduce, sizeof(__pyx_k_Population_reproduce), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_alpha, __pyx_k_alpha, sizeof(__pyx_k_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_ape, __pyx_k_ape, sizeof(__pyx_k_ape), 0, 0, 1, 1},
  {&__pyx_n_s_appraised_individual, __pyx_k_appraised_individual, sizeof(__pyx_k_appraised_individual), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_average_order, __pyx_k_average_order, sizeof(__pyx_k_average_order), 0, 0, 1, 1},
  {&__pyx_n_s_avg_orders, __pyx_k_avg_orders, sizeof(__pyx_k_avg_orders), 0, 0, 1, 1},
  {&__pyx_n_s_clear, __pyx_k_clear, sizeof(__pyx_k_clear), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_create_individuals, __pyx_k_create_individuals, sizeof(__pyx_k_create_individuals), 0, 0, 1, 1},
  {&__pyx_n_s_create_offspring, __pyx_k_create_offspring, sizeof(__pyx_k_create_offspring), 0, 0, 1, 1},
  {&__pyx_n_s_demand, __pyx_k_demand, sizeof(__pyx_k_demand), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_e, __pyx_k_e, sizeof(__pyx_k_e), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_evo_mod, __pyx_k_evo_mod, sizeof(__pyx_k_evo_mod), 0, 0, 1, 1},
  {&__pyx_n_s_express_smoothing_level_genome, __pyx_k_express_smoothing_level_genome, sizeof(__pyx_k_express_smoothing_level_genome), 0, 0, 1, 1},
  {&__pyx_n_s_express_trait, __pyx_k_express_trait, sizeof(__pyx_k_express_trait), 0, 0, 1, 1},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_final_error, __pyx_k_final_error, sizeof(__pyx_k_final_error), 0, 0, 1, 1},
  {&__pyx_n_s_fit_population, __pyx_k_fit_population, sizeof(__pyx_k_fit_population), 0, 0, 1, 1},
  {&__pyx_n_s_forecast, __pyx_k_forecast, sizeof(__pyx_k_forecast), 0, 0, 1, 1},
  {&__pyx_n_s_forecast_breakdown, __pyx_k_forecast_breakdown, sizeof(__pyx_k_forecast_breakdown), 0, 0, 1, 1},
  {&__pyx_n_s_forecast_demand, __pyx_k_forecast_demand, sizeof(__pyx_k_forecast_demand), 0, 0, 1, 1},
  {&__pyx_n_s_forecast_length, __pyx_k_forecast_length, sizeof(__pyx_k_forecast_length), 0, 0, 1, 1},
  {&__pyx_n_s_forecast_type, __pyx_k_forecast_type, sizeof(__pyx_k_forecast_type), 0, 0, 1, 1},
  {&__pyx_n_s_forecast_type_2, __pyx_k_forecast_type_2, sizeof(__pyx_k_forecast_type_2), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_gamma, __pyx_k_gamma, sizeof(__pyx_k_gamma), 0, 0, 1, 1},
  {&__pyx_n_s_gene, __pyx_k_gene, sizeof(__pyx_k_gene), 0, 0, 1, 1},
  {&__pyx_n_s_gene_count, __pyx_k_gene_count, sizeof(__pyx_k_gene_count), 0, 0, 1, 1},
  {&__pyx_n_s_gene_count_2, __pyx_k_gene_count_2, sizeof(__pyx_k_gene_count_2), 0, 0, 1, 1},
  {&__pyx_n_s_generate_smoothing_level_genome, __pyx_k_generate_smoothing_level_genome, sizeof(__pyx_k_generate_smoothing_level_genome), 0, 0, 1, 1},
  {&__pyx_n_s_genome, __pyx_k_genome, sizeof(__pyx_k_genome), 0, 0, 1, 1},
  {&__pyx_n_s_genome_2, __pyx_k_genome_2, sizeof(__pyx_k_genome_2), 0, 0, 1, 1},
  {&__pyx_n_s_genome_count, __pyx_k_genome_count, sizeof(__pyx_k_genome_count), 0, 0, 1, 1},
  {&__pyx_n_s_genome_generator, __pyx_k_genome_generator, sizeof(__pyx_k_genome_generator), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_holts_trend_corrected_exponentia, __pyx_k_holts_trend_corrected_exponentia, sizeof(__pyx_k_holts_trend_corrected_exponentia), 0, 0, 1, 1},
  {&__pyx_n_s_holts_trend_corrected_smoothing, __pyx_k_holts_trend_corrected_smoothing, sizeof(__pyx_k_holts_trend_corrected_smoothing), 0, 0, 1, 1},
  {&__pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_k_home_kevin_Projects_Python_supp, sizeof(__pyx_k_home_kevin_Projects_Python_supp), 0, 0, 1, 0},
  {&__pyx_n_s_htce_para, __pyx_k_htce_para, sizeof(__pyx_k_htce_para), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_individual, __pyx_k_individual, sizeof(__pyx_k_individual), 0, 0, 1, 1},
  {&__pyx_n_s_individual_type, __pyx_k_individual_type, sizeof(__pyx_k_individual_type), 0, 0, 1, 1},
  {&__pyx_n_s_individuals, __pyx_k_individuals, sizeof(__pyx_k_individuals), 0, 0, 1, 1},
  {&__pyx_n_s_individuals_fitness, __pyx_k_individuals_fitness, sizeof(__pyx_k_individuals_fitness), 0, 0, 1, 1},
  {&__pyx_n_s_individuals_genome, __pyx_k_individuals_genome, sizeof(__pyx_k_individuals_genome), 0, 0, 1, 1},
  {&__pyx_n_s_individuals_traits, __pyx_k_individuals_traits, sizeof(__pyx_k_individuals_traits), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_initial_estimate_period, __pyx_k_initial_estimate_period, sizeof(__pyx_k_initial_estimate_period), 0, 0, 1, 1},
  {&__pyx_n_s_initial_population, __pyx_k_initial_population, sizeof(__pyx_k_initial_population), 0, 0, 1, 1},
  {&__pyx_n_s_initialise_smoothing_level_evol, __pyx_k_initialise_smoothing_level_evol, sizeof(__pyx_k_initialise_smoothing_level_evol), 0, 0, 1, 1},
  {&__pyx_n_s_intercept, __pyx_k_intercept, sizeof(__pyx_k_intercept), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_least_squared_error, __pyx_k_least_squared_error, sizeof(__pyx_k_least_squared_error), 0, 0, 1, 1},
  {&__pyx_n_s_log_stats, __pyx_k_log_stats, sizeof(__pyx_k_log_stats), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mape, __pyx_k_mape, sizeof(__pyx_k_mape), 0, 0, 1, 1},
  {&__pyx_n_s_mean_aboslute_percentage_error_o, __pyx_k_mean_aboslute_percentage_error_o, sizeof(__pyx_k_mean_aboslute_percentage_error_o), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_min, __pyx_k_min, sizeof(__pyx_k_min), 0, 0, 1, 1},
  {&__pyx_n_s_minimum_smoothing_level, __pyx_k_minimum_smoothing_level, sizeof(__pyx_k_minimum_smoothing_level), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_mutate, __pyx_k_mutate, sizeof(__pyx_k_mutate), 0, 0, 1, 1},
  {&__pyx_n_s_mutation, __pyx_k_mutation, sizeof(__pyx_k_mutation), 0, 0, 1, 1},
  {&__pyx_n_s_mutation_count, __pyx_k_mutation_count, sizeof(__pyx_k_mutation_count), 0, 0, 1, 1},
  {&__pyx_n_s_mutation_index, __pyx_k_mutation_index, sizeof(__pyx_k_mutation_index), 0, 0, 1, 1},
  {&__pyx_n_s_mutation_probability, __pyx_k_mutation_probability, sizeof(__pyx_k_mutation_probability), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new_fit_population, __pyx_k_new_fit_population, sizeof(__pyx_k_new_fit_population), 0, 0, 1, 1},
  {&__pyx_n_s_new_individual, __pyx_k_new_individual, sizeof(__pyx_k_new_individual), 0, 0, 1, 1},
  {&__pyx_n_s_new_individual_two, __pyx_k_new_individual_two, sizeof(__pyx_k_new_individual_two), 0, 0, 1, 1},
  {&__pyx_n_s_new_individuals, __pyx_k_new_individuals, sizeof(__pyx_k_new_individuals), 0, 0, 1, 1},
  {&__pyx_n_s_new_population, __pyx_k_new_population, sizeof(__pyx_k_new_population), 0, 0, 1, 1},
  {&__pyx_n_s_new_population_genome, __pyx_k_new_population_genome, sizeof(__pyx_k_new_population_genome), 0, 0, 1, 1},
  {&__pyx_n_s_new_populations_traits, __pyx_k_new_populations_traits, sizeof(__pyx_k_new_populations_traits), 0, 0, 1, 1},
  {&__pyx_n_s_number_of_mutations_allowed, __pyx_k_number_of_mutations_allowed, sizeof(__pyx_k_number_of_mutations_allowed), 0, 0, 1, 1},
  {&__pyx_n_s_offspring, __pyx_k_offspring, sizeof(__pyx_k_offspring), 0, 0, 1, 1},
  {&__pyx_n_s_optimal_alpha, __pyx_k_optimal_alpha, sizeof(__pyx_k_optimal_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_optimal_ses_forecast, __pyx_k_optimal_ses_forecast, sizeof(__pyx_k_optimal_ses_forecast), 0, 0, 1, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_s_orders, __pyx_k_orders, sizeof(__pyx_k_orders), 0, 0, 1, 1},
  {&__pyx_n_s_original_gene, __pyx_k_original_gene, sizeof(__pyx_k_original_gene), 0, 0, 1, 1},
  {&__pyx_n_s_original_standard_error, __pyx_k_original_standard_error, sizeof(__pyx_k_original_standard_error), 0, 0, 1, 1},
  {&__pyx_n_s_overide, __pyx_k_overide, sizeof(__pyx_k_overide), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_parent, __pyx_k_parent, sizeof(__pyx_k_parent), 0, 0, 1, 1},
  {&__pyx_n_s_parent_offspring_population, __pyx_k_parent_offspring_population, sizeof(__pyx_k_parent_offspring_population), 0, 0, 1, 1},
  {&__pyx_n_s_parents, __pyx_k_parents, sizeof(__pyx_k_parents), 0, 0, 1, 1},
  {&__pyx_n_s_parents_population, __pyx_k_parents_population, sizeof(__pyx_k_parents_population), 0, 0, 1, 1},
  {&__pyx_n_s_pke, __pyx_k_pke, sizeof(__pyx_k_pke), 0, 0, 1, 1},
  {&__pyx_n_s_po, __pyx_k_po, sizeof(__pyx_k_po), 0, 0, 1, 1},
  {&__pyx_n_s_population, __pyx_k_population, sizeof(__pyx_k_population), 0, 0, 1, 1},
  {&__pyx_n_s_population_allele_count, __pyx_k_population_allele_count, sizeof(__pyx_k_population_allele_count), 0, 0, 1, 1},
  {&__pyx_n_s_population_fitness, __pyx_k_population_fitness, sizeof(__pyx_k_population_fitness), 0, 0, 1, 1},
  {&__pyx_n_s_population_size, __pyx_k_population_size, sizeof(__pyx_k_population_size), 0, 0, 1, 1},
  {&__pyx_n_s_populations_genome, __pyx_k_populations_genome, sizeof(__pyx_k_populations_genome), 0, 0, 1, 1},
  {&__pyx_n_s_populations_traits, __pyx_k_populations_traits, sizeof(__pyx_k_populations_traits), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_procreation_probability, __pyx_k_procreation_probability, sizeof(__pyx_k_procreation_probability), 0, 0, 1, 1},
  {&__pyx_n_s_property, __pyx_k_property, sizeof(__pyx_k_property), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_random, __pyx_k_random, sizeof(__pyx_k_random), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_recombination_type, __pyx_k_recombination_type, sizeof(__pyx_k_recombination_type), 0, 0, 1, 1},
  {&__pyx_n_s_recombination_type_2, __pyx_k_recombination_type_2, sizeof(__pyx_k_recombination_type_2), 0, 0, 1, 1},
  {&__pyx_n_s_regression, __pyx_k_regression, sizeof(__pyx_k_regression), 0, 0, 1, 1},
  {&__pyx_n_s_repr, __pyx_k_repr, sizeof(__pyx_k_repr), 0, 0, 1, 1},
  {&__pyx_n_s_reproduce, __pyx_k_reproduce, sizeof(__pyx_k_reproduce), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_run_exponential_smoothing_forec, __pyx_k_run_exponential_smoothing_forec, sizeof(__pyx_k_run_exponential_smoothing_forec), 0, 0, 1, 1},
  {&__pyx_n_s_run_holts_trend_corrected_expon, __pyx_k_run_holts_trend_corrected_expon, sizeof(__pyx_k_run_holts_trend_corrected_expon), 0, 0, 1, 1},
  {&__pyx_n_s_selection_population, __pyx_k_selection_population, sizeof(__pyx_k_selection_population), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_ses, __pyx_k_ses, sizeof(__pyx_k_ses), 0, 0, 1, 1},
  {&__pyx_n_s_ses_forecast, __pyx_k_ses_forecast, sizeof(__pyx_k_ses_forecast), 0, 0, 1, 1},
  {&__pyx_n_s_setter, __pyx_k_setter, sizeof(__pyx_k_setter), 0, 0, 1, 1},
  {&__pyx_n_s_simple_expo_smoothing, __pyx_k_simple_expo_smoothing, sizeof(__pyx_k_simple_expo_smoothing), 0, 0, 1, 1},
  {&__pyx_n_s_simple_exponential_smoothing, __pyx_k_simple_exponential_smoothing, sizeof(__pyx_k_simple_exponential_smoothing), 0, 0, 1, 1},
  {&__pyx_n_s_simple_exponential_smoothing_evo, __pyx_k_simple_exponential_smoothing_evo, sizeof(__pyx_k_simple_exponential_smoothing_evo), 0, 0, 1, 1},
  {&__pyx_n_s_simple_exponential_smoothing_for, __pyx_k_simple_exponential_smoothing_for, sizeof(__pyx_k_simple_exponential_smoothing_for), 0, 0, 1, 1},
  {&__pyx_n_s_simple_forecast, __pyx_k_simple_forecast, sizeof(__pyx_k_simple_forecast), 0, 0, 1, 1},
  {&__pyx_n_s_single_point, __pyx_k_single_point, sizeof(__pyx_k_single_point), 0, 0, 1, 1},
  {&__pyx_n_s_single_point_crossover_recombin, __pyx_k_single_point_crossover_recombin, sizeof(__pyx_k_single_point_crossover_recombin), 0, 0, 1, 1},
  {&__pyx_n_s_slice_end, __pyx_k_slice_end, sizeof(__pyx_k_slice_end), 0, 0, 1, 1},
  {&__pyx_n_s_slope, __pyx_k_slope, sizeof(__pyx_k_slope), 0, 0, 1, 1},
  {&__pyx_n_s_slots, __pyx_k_slots, sizeof(__pyx_k_slots), 0, 0, 1, 1},
  {&__pyx_n_s_sm_lvl, __pyx_k_sm_lvl, sizeof(__pyx_k_sm_lvl), 0, 0, 1, 1},
  {&__pyx_n_s_smoothing_level, __pyx_k_smoothing_level, sizeof(__pyx_k_smoothing_level), 0, 0, 1, 1},
  {&__pyx_n_s_smoothing_level_constant, __pyx_k_smoothing_level_constant, sizeof(__pyx_k_smoothing_level_constant), 0, 0, 1, 1},
  {&__pyx_n_s_squared_error, __pyx_k_squared_error, sizeof(__pyx_k_squared_error), 0, 0, 1, 1},
  {&__pyx_n_s_standard_error, __pyx_k_standard_error, sizeof(__pyx_k_standard_error), 0, 0, 1, 1},
  {&__pyx_n_s_staticmethod, __pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 0, 1, 1},
  {&__pyx_n_s_statistics, __pyx_k_statistics, sizeof(__pyx_k_statistics), 0, 0, 1, 1},
  {&__pyx_n_s_stats, __pyx_k_stats, sizeof(__pyx_k_stats), 0, 0, 1, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_sum_orders, __pyx_k_sum_orders, sizeof(__pyx_k_sum_orders), 0, 0, 1, 1},
  {&__pyx_n_s_sum_squared_error, __pyx_k_sum_squared_error, sizeof(__pyx_k_sum_squared_error), 0, 0, 1, 1},
  {&__pyx_n_s_sum_squared_error_para, __pyx_k_sum_squared_error_para, sizeof(__pyx_k_sum_squared_error_para), 0, 0, 1, 1},
  {&__pyx_n_s_sum_squared_errors, __pyx_k_sum_squared_errors, sizeof(__pyx_k_sum_squared_errors), 0, 0, 1, 1},
  {&__pyx_n_s_sum_squared_errors_indi, __pyx_k_sum_squared_errors_indi, sizeof(__pyx_k_sum_squared_errors_indi), 0, 0, 1, 1},
  {&__pyx_n_s_sum_squared_errors_indi_htces, __pyx_k_sum_squared_errors_indi_htces, sizeof(__pyx_k_sum_squared_errors_indi_htces), 0, 0, 1, 1},
  {&__pyx_n_s_supplychainpy_demand__evo_algo, __pyx_k_supplychainpy_demand__evo_algo, sizeof(__pyx_k_supplychainpy_demand__evo_algo), 0, 0, 1, 1},
  {&__pyx_n_s_supplychainpy_demand__forecast_d, __pyx_k_supplychainpy_demand__forecast_d, sizeof(__pyx_k_supplychainpy_demand__forecast_d), 0, 0, 1, 1},
  {&__pyx_n_s_supplychainpy_demand_regression, __pyx_k_supplychainpy_demand_regression, sizeof(__pyx_k_supplychainpy_demand_regression), 0, 0, 1, 1},
  {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_two_point, __pyx_k_two_point, sizeof(__pyx_k_two_point), 0, 0, 1, 1},
  {&__pyx_n_s_two_point_crossover_recombinati, __pyx_k_two_point_crossover_recombinati, sizeof(__pyx_k_two_point_crossover_recombinati), 0, 0, 1, 1},
  {&__pyx_n_s_uniform, __pyx_k_uniform, sizeof(__pyx_k_uniform), 0, 0, 1, 1},
  {&__pyx_n_s_uniform_crossover_recombination, __pyx_k_uniform_crossover_recombination, sizeof(__pyx_k_uniform_crossover_recombination), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {&__pyx_n_s_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_property = __Pyx_GetBuiltinName(__pyx_n_s_property); if (!__pyx_builtin_property) __PYX_ERR(0, 23, __pyx_L1_error)
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_n_s_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 220, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 46, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 105, __pyx_L1_error)
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 108, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 147, __pyx_L1_error)
  __pyx_builtin_OSError = __Pyx_GetBuiltinName(__pyx_n_s_OSError); if (!__pyx_builtin_OSError) __PYX_ERR(0, 212, __pyx_L1_error)
  __pyx_builtin_min = __Pyx_GetBuiltinName(__pyx_n_s_min); if (!__pyx_builtin_min) __PYX_ERR(0, 345, __pyx_L1_error)
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) __PYX_ERR(0, 345, __pyx_L1_error)
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_n_s_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 381, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "supplychainpy/demand/_evo_algo.pyx":46
 *         genome = ()
 *         if self._forecast_type == 'ses':
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])             # <<<<<<<<<<<<<<
 *         else:
 *             genome = ([(uniform(0, 1), uniform(0, 1)) for i in range(0, self._gene_count)])
 */
  __pyx_tuple__2 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_1); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "supplychainpy/demand/_evo_algo.pyx":48
 *             genome = ([uniform(0, 1) for i in range(0, self._gene_count)])
 *         else:
 *             genome = ([(uniform(0, 1), uniform(0, 1)) for i in range(0, self._gene_count)])             # <<<<<<<<<<<<<<
 * 
 *         return genome
 */
  __pyx_tuple__3 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_1); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);
  __pyx_tuple__4 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_1); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "supplychainpy/demand/_evo_algo.pyx":235
 *         original_gene = [[k, v] for k, v in gene.items()]
 * 
 *         def mutate():             # <<<<<<<<<<<<<<
 * 
 *             if original_gene[0][1] == 0:
 */
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_mutate, 235, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 235, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":431
 *         for sm_lvl in individual:
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],
 *                                                                           intercept=log_stats.get('intercept'),             # <<<<<<<<<<<<<<
 *                                                                           slope=log_stats.get('slope'))]
 *             # print(p)
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_n_s_intercept); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "supplychainpy/demand/_evo_algo.pyx":432
 *             p = [i for i in f.holts_trend_corrected_exponential_smoothing(alpha=sm_lvl[0], gamma=sm_lvl[1],
 *                                                                           intercept=log_stats.get('intercept'),
 *                                                                           slope=log_stats.get('slope'))]             # <<<<<<<<<<<<<<
 *             # print(p)
 *             holts_trend_corrected_smoothing.append(p)
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_s_slope); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "supplychainpy/demand/_evo_algo.pyx":536
 *                                                                                 forecast_length=forecast_length)
 *         regression = {
 *             'regression': [(stats.get('slope') * i) + stats.get('intercept') for i in range(0, 12)]}             # <<<<<<<<<<<<<<
 *         return {'forecast_breakdown': optimal_ses_forecast, 'mape': mape, 'statistics': stats,
 *                 'forecast': simple_forecast, 'optimal_alpha': optimal_alpha[1], 'standard_error': standard_error,
 */
  __pyx_tuple__8 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_12); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_slope); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_n_s_intercept); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "supplychainpy/demand/_evo_algo.pyx":9
 *     _genome = 0
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, name ='offspring', overide=False, int gene_count =12,
 *             forecast_type ='ses'
 */
  __pyx_tuple__11 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_name, __pyx_n_s_overide, __pyx_n_s_gene_count, __pyx_n_s_forecast_type); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_init, 9, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 9, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":20
 *             self._genome = self.__genome_generator()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '{}: {}'.format(self._name, self._genome)
 * 
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_repr, 20, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 20, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":24
 * 
 *     @property
 *     def gene_count(self):             # <<<<<<<<<<<<<<
 *         return self.gene_count
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_gene_count, 24, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 24, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":28
 * 
 *     @gene_count.setter
 *     def gene_count(self, int value):             # <<<<<<<<<<<<<<
 *         self._gene_count = value
 * 
 */
  __pyx_tuple__17 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_value); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_gene_count, 28, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 28, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":32
 * 
 *     @property
 *     def genome(self):             # <<<<<<<<<<<<<<
 *         return self._genome
 * 
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_genome_2, 32, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 32, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":36
 * 
 *     @genome.setter
 *     def genome(self, tuple val):             # <<<<<<<<<<<<<<
 *         self._genome = val
 * 
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_val); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_genome_2, 36, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 36, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":40
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_name, 40, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 40, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":43
 *         return self._name
 * 
 *     def __genome_generator(self):             # <<<<<<<<<<<<<<
 *         genome = ()
 *         if self._forecast_type == 'ses':
 */
  __pyx_tuple__25 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_genome_2, __pyx_n_s_i); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_genome_generator, 43, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 43, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":57
 * 
 *     __slots__ = ['individuals', 'mutation_probability']
 *     _recombination_type = ('single_point', 'two_point', 'uniform')             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):
 */
  __pyx_tuple__27 = PyTuple_Pack(3, __pyx_n_s_single_point, __pyx_n_s_two_point, __pyx_n_s_uniform); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "supplychainpy/demand/_evo_algo.pyx":59
 *     _recombination_type = ('single_point', 'two_point', 'uniform')
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):             # <<<<<<<<<<<<<<
 *         self.individuals = individuals
 *         self.mutation_probability = mutation_probability
 */
  __pyx_tuple__28 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_individuals, __pyx_n_s_mutation_probability); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_init, 59, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 59, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":64
 *         #print("initialising {}".format(id(self)))
 * 
 *     def reproduce(self, recombination_type = 'single_point'):             # <<<<<<<<<<<<<<
 *         """ Coordinates the reproduction of two individuals, using one of three recombination methods 'single_point,
 *         two_point or uniform'.
 */
  __pyx_tuple__30 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_recombination_type, __pyx_n_s_i); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_reproduce, 64, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(0, 64, __pyx_L1_error)
  __pyx_tuple__32 = PyTuple_Pack(1, ((PyObject*)__pyx_n_s_single_point)); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "supplychainpy/demand/_evo_algo.pyx":85
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 *     def _single_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on one point along the chromosome.
 * 
 */
  __pyx_tuple__33 = PyTuple_Pack(12, __pyx_n_s_self, __pyx_n_s_genome_count, __pyx_n_s_new_individual, __pyx_n_s_new_individual_two, __pyx_n_s_mutation_count, __pyx_n_s_mutation_index, __pyx_n_s_index, __pyx_n_s_individual, __pyx_n_s_key, __pyx_n_s_value, __pyx_n_s_population_allele_count, __pyx_n_s_number_of_mutations_allowed); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(1, 0, 12, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_single_point_crossover_recombin, 85, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 85, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":152
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')
 * 
 *     def _two_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on two points along the chromosome.
 * 
 */
  __pyx_tuple__35 = PyTuple_Pack(13, __pyx_n_s_self, __pyx_n_s_genome_count, __pyx_n_s_new_individual, __pyx_n_s_mutation_count, __pyx_n_s_mutation_index, __pyx_n_s_new_individual_two, __pyx_n_s_index, __pyx_n_s_individual, __pyx_n_s_key, __pyx_n_s_value, __pyx_n_s_population_allele_count, __pyx_n_s_number_of_mutations_allowed, __pyx_n_s_e); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(1, 0, 13, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_two_point_crossover_recombinati, 152, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 152, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":217
 * 
 *     # TODO-feature uniform crossover recombination for reproduction
 *     def _uniform_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_uniform_crossover_recombination, 217, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 217, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":221
 * 
 *     @staticmethod
 *     def _mutation(gene: dict):             # <<<<<<<<<<<<<<
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 *         by having the bit representing it flipped.
 */
  __pyx_tuple__39 = PyTuple_Pack(6, __pyx_n_s_gene, __pyx_n_s_original_gene, __pyx_n_s_mutate, __pyx_n_s_mutate, __pyx_n_s_k, __pyx_n_s_v); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(1, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_mutation, 221, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(0, 221, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":251
 *     __generation = 0
 * 
 *     def __init__(self, orders: list, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__orders = orders
 *         if len(kwargs) != 0:
 */
  __pyx_tuple__41 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_orders, __pyx_n_s_kwargs); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_VARKEYWORDS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_init, 251, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(0, 251, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":259
 *             self.__recombination_type = kwargs['recombination_type']
 * 
 *     def initial_population(self, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 *         """Initialises population and initiates the optimisation of the standard error by searching for an optimum
 *         alpha value.
 */
  __pyx_tuple__43 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_individual_type); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_initial_population, 259, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(0, 259, __pyx_L1_error)
  __pyx_tuple__45 = PyTuple_Pack(1, ((PyObject*)__pyx_n_s_ses)); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "supplychainpy/demand/_evo_algo.pyx":268
 * 
 *     @property
 *     def population_size(self):             # <<<<<<<<<<<<<<
 *         return self.__population_size
 * 
 */
  __pyx_tuple__46 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_codeobj__47 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_population_size, 268, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__47)) __PYX_ERR(0, 268, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":272
 * 
 *     @population_size.setter
 *     def population_size(self, population_size):             # <<<<<<<<<<<<<<
 *         self.__population_size = population_size
 * 
 */
  __pyx_tuple__48 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_population_size); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);
  __pyx_codeobj__49 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__48, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_population_size, 272, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__49)) __PYX_ERR(0, 272, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":275
 *         self.__population_size = population_size
 * 
 *     def _initialise_smoothing_level_evolutionary_algorithm_population(self, individual_type: str):             # <<<<<<<<<<<<<<
 *         """ Starts the process for creating the population. The number of parents is specified during the
 *          initialisation of the class. """
 */
  __pyx_tuple__50 = PyTuple_Pack(22, __pyx_n_s_self, __pyx_n_s_individual_type, __pyx_n_s_parents, __pyx_n_s_parents_population, __pyx_n_s_populations_genome, __pyx_n_s_populations_traits, __pyx_n_s_i, __pyx_n_s_parent, __pyx_n_s_fit_population, __pyx_n_s_create_offspring, __pyx_n_s_new_population, __pyx_n_s_parent_offspring_population, __pyx_n_s_new_individuals, __pyx_n_s_new_populations_traits, __pyx_n_s_new_fit_population, __pyx_n_s_final_error, __pyx_n_s_minimum_smoothing_level, __pyx_n_s_po, __pyx_n_s_pke, __pyx_n_s_genome_2, __pyx_n_s_new_individual, __pyx_n_s_new_population_genome); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(2, 0, 22, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_initialise_smoothing_level_evol, 275, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(0, 275, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":350
 * 
 *     @staticmethod
 *     def create_individuals(list new_population):             # <<<<<<<<<<<<<<
 *         """Create individuals using class from genomes striped during processing fitness.
 *         Args:
 */
  __pyx_tuple__52 = PyTuple_Pack(6, __pyx_n_s_new_population, __pyx_n_s_parent_offspring_population, __pyx_n_s_po, __pyx_n_s_pke, __pyx_n_s_genome_2, __pyx_n_s_new_individual); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(1, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_create_individuals, 350, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(0, 350, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":367
 * 
 *     @staticmethod
 *     def _population_fitness(list population, individual_type='ses'):             # <<<<<<<<<<<<<<
 *         """ Assess the population for fitness before crossover and creating next generation. Positive traits
 *         should be reflected by more than 70% of the genes in the genome.
 */
  __pyx_tuple__54 = PyTuple_Pack(4, __pyx_n_s_population, __pyx_n_s_individual_type, __pyx_n_s_procreation_probability, __pyx_n_s_individual); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);
  __pyx_codeobj__55 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__54, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_population_fitness, 367, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__55)) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_tuple__56 = PyTuple_Pack(1, ((PyObject*)__pyx_n_s_ses)); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "supplychainpy/demand/_evo_algo.pyx":389
 *                     yield individual
 * 
 *     def generate_smoothing_level_genome(self, list population, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */
  __pyx_tuple__57 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_population, __pyx_n_s_individual_type, __pyx_n_s_parent, __pyx_n_s_individuals_genome); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  __pyx_codeobj__58 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__57, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_generate_smoothing_level_genome, 389, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__58)) __PYX_ERR(0, 389, __pyx_L1_error)
  __pyx_tuple__59 = PyTuple_Pack(1, ((PyObject*)__pyx_n_s_ses)); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "supplychainpy/demand/_evo_algo.pyx":408
 * 
 *     @staticmethod
 *     def htce_para(forecast, alpha, gamma, intercept, slope):             # <<<<<<<<<<<<<<
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 *                                                                                 intercept=intercept,
 */
  __pyx_tuple__60 = PyTuple_Pack(6, __pyx_n_s_forecast, __pyx_n_s_alpha, __pyx_n_s_gamma, __pyx_n_s_intercept, __pyx_n_s_slope, __pyx_n_s_i); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(5, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_htce_para, 408, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(0, 408, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":414
 * 
 * 
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):             # <<<<<<<<<<<<<<
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 */
  __pyx_tuple__62 = PyTuple_Pack(9, __pyx_n_s_self, __pyx_n_s_forecast, __pyx_n_s_holts_trend_corrected_smoothing, __pyx_n_s_alpha, __pyx_n_s_gamma, __pyx_n_s_smoothing_level, __pyx_n_s_appraised_individual, __pyx_n_s_sum_squared_error, __pyx_n_s_standard_error); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(6, 0, 9, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__62, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_sum_squared_error_para, 414, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(0, 414, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":422
 *         return appraised_individual
 * 
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):             # <<<<<<<<<<<<<<
 *         f = Forecast(self.__orders, self.__average_order)
 *         holts_trend_corrected_smoothing = []
 */
  __pyx_tuple__64 = PyTuple_Pack(16, __pyx_n_s_self, __pyx_n_s_individual, __pyx_n_s_f, __pyx_n_s_holts_trend_corrected_smoothing, __pyx_n_s_demand, __pyx_n_s_stats, __pyx_n_s_log_stats, __pyx_n_s_sm_lvl, __pyx_n_s_p, __pyx_n_s_appraised_individual, __pyx_n_s_smoothing_level, __pyx_n_s_sum_squared_error, __pyx_n_s_standard_error, __pyx_n_s_index, __pyx_n_s_order, __pyx_n_s_i); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  __pyx_codeobj__65 = (PyObject*)__Pyx_PyCode_New(2, 0, 16, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__64, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_run_holts_trend_corrected_expon, 422, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__65)) __PYX_ERR(0, 422, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":445
 *         return appraised_individual
 * 
 *     def express_smoothing_level_genome(self, individuals_genome: list, standard_error):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */
  __pyx_tuple__66 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_individuals_genome, __pyx_n_s_standard_error, __pyx_n_s_genome_2, __pyx_n_s_individuals_traits); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);
  __pyx_codeobj__67 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__66, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_express_smoothing_level_genome, 445, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__67)) __PYX_ERR(0, 445, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":452
 *             yield individuals_traits
 * 
 *     def _run_exponential_smoothing_forecast(self, individual ):             # <<<<<<<<<<<<<<
 * 
 *         f = Forecast(self.__orders, self.__average_order)
 */
  __pyx_tuple__68 = PyTuple_Pack(11, __pyx_n_s_self, __pyx_n_s_individual, __pyx_n_s_f, __pyx_n_s_simple_expo_smoothing, __pyx_n_s_sm_lvl, __pyx_n_s_p, __pyx_n_s_appraised_individual, __pyx_n_s_smoothing_level, __pyx_n_s_sum_squared_error, __pyx_n_s_standard_error, __pyx_n_s_i); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_codeobj__69 = (PyObject*)__Pyx_PyCode_New(2, 0, 11, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__68, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_run_exponential_smoothing_forec, 452, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__69)) __PYX_ERR(0, 452, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":470
 * 
 *     @staticmethod
 *     def _express_trait(original_standard_error: float, appraised_individual: dict):             # <<<<<<<<<<<<<<
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 *         # print(original_standard_error)
 */
  __pyx_tuple__70 = PyTuple_Pack(3, __pyx_n_s_original_standard_error, __pyx_n_s_appraised_individual, __pyx_n_s_key); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  __pyx_codeobj__71 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__70, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_express_trait, 470, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__71)) __PYX_ERR(0, 470, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":482
 *         return appraised_individual
 * 
 *     def _selection_population(self, individuals_fitness: dict, appraised_individual: list):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */
  __pyx_tuple__72 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_individuals_fitness, __pyx_n_s_appraised_individual); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);
  __pyx_codeobj__73 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__72, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_selection_population, 482, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__73)) __PYX_ERR(0, 482, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":485
 *         pass
 * 
 *     def simple_exponential_smoothing_evo(self, smoothing_level_constant, initial_estimate_period,             # <<<<<<<<<<<<<<
 *                                          recombination_type = 'single_point', int population_size = 10,
 *                                          int forecast_length = 5):
 */
  __pyx_tuple__74 = PyTuple_Pack(22, __pyx_n_s_self, __pyx_n_s_smoothing_level_constant, __pyx_n_s_initial_estimate_period, __pyx_n_s_recombination_type, __pyx_n_s_population_size, __pyx_n_s_forecast_length, __pyx_n_s_sum_orders, __pyx_n_s_demand, __pyx_n_s_avg_orders, __pyx_n_s_forecast_demand, __pyx_n_s_ses_forecast, __pyx_n_s_sum_squared_error, __pyx_n_s_standard_error, __pyx_n_s_evo_mod, __pyx_n_s_optimal_alpha, __pyx_n_s_optimal_ses_forecast, __pyx_n_s_ape, __pyx_n_s_mape, __pyx_n_s_stats, __pyx_n_s_simple_forecast, __pyx_n_s_regression, __pyx_n_s_i); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);
  __pyx_codeobj__75 = (PyObject*)__Pyx_PyCode_New(6, 0, 22, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__74, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_kevin_Projects_Python_supp, __pyx_n_s_simple_exponential_smoothing_evo, 485, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__75)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_10 = PyInt_FromLong(10); if (unlikely(!__pyx_int_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_12 = PyInt_FromLong(12); if (unlikely(!__pyx_int_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_evo_algo(void); /*proto*/
PyMODINIT_FUNC init_evo_algo(void)
#else
PyMODINIT_FUNC PyInit__evo_algo(void); /*proto*/
PyMODINIT_FUNC PyInit__evo_algo(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__evo_algo(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_evo_algo", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_supplychainpy__demand___evo_algo) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "supplychainpy.demand._evo_algo")) {
      if (unlikely(PyDict_SetItemString(modules, "supplychainpy.demand._evo_algo", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce) < 0) __PYX_ERR(0, 64, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct__reproduce;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_1__single_point_crossover_recombination;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_2__two_point_crossover_recombination;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_3__mutation;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals) < 0) __PYX_ERR(0, 350, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_4_create_individuals;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_5__population_fitness;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome) < 0) __PYX_ERR(0, 389, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_6_generate_smoothing_level_genome;
  if (PyType_Ready(&__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome) < 0) __PYX_ERR(0, 445, __pyx_L1_error)
  __pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome.tp_print = 0;
  __pyx_ptype_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome = &__pyx_type_13supplychainpy_6demand_9_evo_algo___pyx_scope_struct_7_express_smoothing_level_genome;
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "supplychainpy/demand/_evo_algo.pyx":1
 * from random import uniform             # <<<<<<<<<<<<<<
 * 
 * from supplychainpy.demand._forecast_demand import Forecast
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_uniform);
  __Pyx_GIVEREF(__pyx_n_s_uniform);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_uniform);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_random, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_uniform); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_uniform, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":3
 * from random import uniform
 * 
 * from supplychainpy.demand._forecast_demand import Forecast             # <<<<<<<<<<<<<<
 * from supplychainpy.demand.regression import LinearRegression
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_Forecast);
  __Pyx_GIVEREF(__pyx_n_s_Forecast);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_Forecast);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_supplychainpy_demand__forecast_d, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_Forecast); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Forecast, __pyx_t_2) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":4
 * 
 * from supplychainpy.demand._forecast_demand import Forecast
 * from supplychainpy.demand.regression import LinearRegression             # <<<<<<<<<<<<<<
 * 
 * class Individual:
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_LinearRegression);
  __Pyx_GIVEREF(__pyx_n_s_LinearRegression);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_LinearRegression);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_supplychainpy_demand_regression, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_LinearRegression); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LinearRegression, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":6
 * from supplychainpy.demand.regression import LinearRegression
 * 
 * class Individual:             # <<<<<<<<<<<<<<
 *     _genome = 0
 * 
 */
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_Individual, __pyx_n_s_Individual, (PyObject *) NULL, __pyx_n_s_supplychainpy_demand__evo_algo, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "supplychainpy/demand/_evo_algo.pyx":7
 * 
 * class Individual:
 *     _genome = 0             # <<<<<<<<<<<<<<
 * 
 *     def __init__(
 */
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_genome, __pyx_int_0) < 0) __PYX_ERR(0, 7, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":10
 * 
 *     def __init__(
 *             self, name ='offspring', overide=False, int gene_count =12,             # <<<<<<<<<<<<<<
 *             forecast_type ='ses'
 *         ):
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(((int)12)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":9
 *     _genome = 0
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *             self, name ='offspring', overide=False, int gene_count =12,
 *             forecast_type ='ses'
 */
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject*)__pyx_n_s_offspring));
  __Pyx_GIVEREF(((PyObject*)__pyx_n_s_offspring));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject*)__pyx_n_s_offspring));
  __Pyx_INCREF(((PyObject *)Py_False));
  __Pyx_GIVEREF(((PyObject *)Py_False));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)Py_False));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_INCREF(((PyObject*)__pyx_n_s_ses));
  __Pyx_GIVEREF(((PyObject*)__pyx_n_s_ses));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject*)__pyx_n_s_ses));
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_1__init__, 0, __pyx_n_s_Individual___init, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_init, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":20
 *             self._genome = self.__genome_generator()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '{}: {}'.format(self._name, self._genome)
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_3__repr__, 0, __pyx_n_s_Individual___repr, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_repr, __pyx_t_1) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":24
 * 
 *     @property
 *     def gene_count(self):             # <<<<<<<<<<<<<<
 *         return self.gene_count
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_5gene_count, 0, __pyx_n_s_Individual_gene_count, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":23
 *         return '{}: {}'.format(self._name, self._genome)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def gene_count(self):
 *         return self.gene_count
 */
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_gene_count, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":27
 *         return self.gene_count
 * 
 *     @gene_count.setter             # <<<<<<<<<<<<<<
 *     def gene_count(self, int value):
 *         self._gene_count = value
 */
  __pyx_t_3 = PyObject_GetItem(__pyx_t_2, __pyx_n_s_gene_count);
  if (unlikely(!__pyx_t_3)) {
    PyErr_Clear();
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_gene_count);
  }
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_setter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":28
 * 
 *     @gene_count.setter
 *     def gene_count(self, int value):             # <<<<<<<<<<<<<<
 *         self._gene_count = value
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_7gene_count, 0, __pyx_n_s_Individual_gene_count, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_gene_count, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":32
 * 
 *     @property
 *     def genome(self):             # <<<<<<<<<<<<<<
 *         return self._genome
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_9genome, 0, __pyx_n_s_Individual_genome, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":31
 *         self._gene_count = value
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def genome(self):
 *         return self._genome
 */
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_genome_2, __pyx_t_1) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":35
 *         return self._genome
 * 
 *     @genome.setter             # <<<<<<<<<<<<<<
 *     def genome(self, tuple val):
 *         self._genome = val
 */
  __pyx_t_4 = PyObject_GetItem(__pyx_t_2, __pyx_n_s_genome_2);
  if (unlikely(!__pyx_t_4)) {
    PyErr_Clear();
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_genome_2);
  }
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_setter); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":36
 * 
 *     @genome.setter
 *     def genome(self, tuple val):             # <<<<<<<<<<<<<<
 *         self._genome = val
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_11genome, 0, __pyx_n_s_Individual_genome, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__22)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 35, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_genome_2, __pyx_t_1) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":40
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_13name, 0, __pyx_n_s_Individual_name, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__24)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":39
 *         self._genome = val
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def name(self):
 *         return self._name
 */
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_name, __pyx_t_1) < 0) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":43
 *         return self._name
 * 
 *     def __genome_generator(self):             # <<<<<<<<<<<<<<
 *         genome = ()
 *         if self._forecast_type == 'ses':
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Individual_15__genome_generator, 0, __pyx_n_s_Individual___genome_generator, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__26)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_Individual__genome_generator, __pyx_t_1) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":6
 * from supplychainpy.demand.regression import LinearRegression
 * 
 * class Individual:             # <<<<<<<<<<<<<<
 *     _genome = 0
 * 
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_Individual, __pyx_empty_tuple, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Individual, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":53
 * 
 * 
 * class Population:             # <<<<<<<<<<<<<<
 *     """ Create a population of individuals to reproduce. """
 * 
 */
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_Population, __pyx_n_s_Population, (PyObject *) NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_kp_s_Create_a_population_of_individu); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "supplychainpy/demand/_evo_algo.pyx":56
 *     """ Create a population of individuals to reproduce. """
 * 
 *     __slots__ = ['individuals', 'mutation_probability']             # <<<<<<<<<<<<<<
 *     _recombination_type = ('single_point', 'two_point', 'uniform')
 * 
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_individuals);
  __Pyx_GIVEREF(__pyx_n_s_individuals);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_individuals);
  __Pyx_INCREF(__pyx_n_s_mutation_probability);
  __Pyx_GIVEREF(__pyx_n_s_mutation_probability);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_mutation_probability);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_slots, __pyx_t_1) < 0) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":57
 * 
 *     __slots__ = ['individuals', 'mutation_probability']
 *     _recombination_type = ('single_point', 'two_point', 'uniform')             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):
 */
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_recombination_type_2, __pyx_tuple__27) < 0) __PYX_ERR(0, 57, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":59
 *     _recombination_type = ('single_point', 'two_point', 'uniform')
 * 
 *     def __init__(self, list individuals, double mutation_probability = 0.2):             # <<<<<<<<<<<<<<
 *         self.individuals = individuals
 *         self.mutation_probability = mutation_probability
 */
  __pyx_t_1 = PyFloat_FromDouble(((double)0.2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_1__init__, 0, __pyx_n_s_Population___init, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__29)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_t_6);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_init, __pyx_t_1) < 0) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":64
 *         #print("initialising {}".format(id(self)))
 * 
 *     def reproduce(self, recombination_type = 'single_point'):             # <<<<<<<<<<<<<<
 *         """ Coordinates the reproduction of two individuals, using one of three recombination methods 'single_point,
 *         two_point or uniform'.
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_3reproduce, 0, __pyx_n_s_Population_reproduce, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__31)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__32);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_reproduce, __pyx_t_1) < 0) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":85
 *             yield [i for i in self._uniform_crossover_recombination()][0]
 * 
 *     def _single_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on one point along the chromosome.
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_6_single_point_crossover_recombination, 0, __pyx_n_s_Population__single_point_crossov, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_single_point_crossover_recombin, __pyx_t_1) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":152
 *             print('Population Size is too small for effective genetic recombination and reproduction of offspring')
 * 
 *     def _two_point_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         """ Selects genes from the parent to crossover based on two points along the chromosome.
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_9_two_point_crossover_recombination, 0, __pyx_n_s_Population__two_point_crossover, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_two_point_crossover_recombinati, __pyx_t_1) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":217
 * 
 *     # TODO-feature uniform crossover recombination for reproduction
 *     def _uniform_crossover_recombination(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_12_uniform_crossover_recombination, 0, __pyx_n_s_Population__uniform_crossover_re, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_uniform_crossover_recombination, __pyx_t_1) < 0) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":221
 * 
 *     @staticmethod
 *     def _mutation(gene: dict):             # <<<<<<<<<<<<<<
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 *         by having the bit representing it flipped.
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_gene, ((PyObject *)(&PyDict_Type))) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  __pyx_t_6 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_10Population_14_mutation, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_n_s_Population__mutation, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__40)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_6, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":220
 *         pass
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _mutation(gene: dict):
 *         """Mutates a gene and reinserts it into the chromosome. A phenotype expressed as a standard error is altered
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_mutation, __pyx_t_6) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":53
 * 
 * 
 * class Population:             # <<<<<<<<<<<<<<
 *     """ Create a population of individuals to reproduce. """
 * 
 */
  __pyx_t_6 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_Population, __pyx_empty_tuple, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Population, __pyx_t_6) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":248
 * 
 * 
 * class OptimiseSmoothingLevelGeneticAlgorithm:             # <<<<<<<<<<<<<<
 *     __generation = 0
 * 
 */
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5, (PyObject *) NULL, __pyx_n_s_supplychainpy_demand__evo_algo, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "supplychainpy/demand/_evo_algo.pyx":249
 * 
 * class OptimiseSmoothingLevelGeneticAlgorithm:
 *     __generation = 0             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, orders: list, **kwargs):
 */
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_OptimiseSmoothingLevelGeneticAl_6, __pyx_int_0) < 0) __PYX_ERR(0, 249, __pyx_L1_error)

  /* "supplychainpy/demand/_evo_algo.pyx":251
 *     __generation = 0
 * 
 *     def __init__(self, orders: list, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__orders = orders
 *         if len(kwargs) != 0:
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_orders, ((PyObject *)(&PyList_Type))) < 0) __PYX_ERR(0, 251, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_1__init__, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_6, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__42)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_6);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_init, __pyx_t_1) < 0) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":259
 *             self.__recombination_type = kwargs['recombination_type']
 * 
 *     def initial_population(self, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 *         """Initialises population and initiates the optimisation of the standard error by searching for an optimum
 *         alpha value.
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_3initial_population, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_7, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__44)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__45);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_initial_population, __pyx_t_1) < 0) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":268
 * 
 *     @property
 *     def population_size(self):             # <<<<<<<<<<<<<<
 *         return self.__population_size
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_5population_size, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_8, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__47)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":267
 *         return self._initialise_smoothing_level_evolutionary_algorithm_population(individual_type=individual_type)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def population_size(self):
 *         return self.__population_size
 */
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_property, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_population_size, __pyx_t_1) < 0) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":271
 *         return self.__population_size
 * 
 *     @population_size.setter             # <<<<<<<<<<<<<<
 *     def population_size(self, population_size):
 *         self.__population_size = population_size
 */
  __pyx_t_6 = PyObject_GetItem(__pyx_t_2, __pyx_n_s_population_size);
  if (unlikely(!__pyx_t_6)) {
    PyErr_Clear();
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_population_size);
  }
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_setter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":272
 * 
 *     @population_size.setter
 *     def population_size(self, population_size):             # <<<<<<<<<<<<<<
 *         self.__population_size = population_size
 * 
 */
  __pyx_t_6 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_7population_size, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_8, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__49)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_population_size, __pyx_t_1) < 0) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":275
 *         self.__population_size = population_size
 * 
 *     def _initialise_smoothing_level_evolutionary_algorithm_population(self, individual_type: str):             # <<<<<<<<<<<<<<
 *         """ Starts the process for creating the population. The number of parents is specified during the
 *          initialisation of the class. """
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_individual_type, ((PyObject *)(&PyString_Type))) < 0) __PYX_ERR(0, 275, __pyx_L1_error)
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_9_initialise_smoothing_level_evolutionary_algorithm_population, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_9, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__51)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_5, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_initialise_smoothing_level_evol, __pyx_t_5) < 0) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":350
 * 
 *     @staticmethod
 *     def create_individuals(list new_population):             # <<<<<<<<<<<<<<
 *         """Create individuals using class from genomes striped during processing fitness.
 *         Args:
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_11create_individuals, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__53)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "supplychainpy/demand/_evo_algo.pyx":349
 *         return minimum_smoothing_level
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def create_individuals(list new_population):
 *         """Create individuals using class from genomes striped during processing fitness.
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_create_individuals, __pyx_t_5) < 0) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":367
 * 
 *     @staticmethod
 *     def _population_fitness(list population, individual_type='ses'):             # <<<<<<<<<<<<<<
 *         """ Assess the population for fitness before crossover and creating next generation. Positive traits
 *         should be reflected by more than 70% of the genes in the genome.
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_14_population_fitness, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_2, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__55)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_5, __pyx_tuple__56);

  /* "supplychainpy/demand/_evo_algo.pyx":366
 *             yield new_individual
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _population_fitness(list population, individual_type='ses'):
 *         """ Assess the population for fitness before crossover and creating next generation. Positive traits
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_population_fitness, __pyx_t_5) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":389
 *                     yield individual
 * 
 *     def generate_smoothing_level_genome(self, list population, individual_type = 'ses'):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_17generate_smoothing_level_genome, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_3, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__58)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_5, __pyx_tuple__59);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_generate_smoothing_level_genome, __pyx_t_5) < 0) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":408
 * 
 *     @staticmethod
 *     def htce_para(forecast, alpha, gamma, intercept, slope):             # <<<<<<<<<<<<<<
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 *                                                                                 intercept=intercept,
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_20htce_para, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_10, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__61)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "supplychainpy/demand/_evo_algo.pyx":407
 *                 yield individuals_genome
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def htce_para(forecast, alpha, gamma, intercept, slope):
 *         return [i for i in forecast.holts_trend_corrected_exponential_smoothing(alpha=alpha, gamma=gamma,
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_htce_para, __pyx_t_5) < 0) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":414
 * 
 * 
 *     def sum_squared_error_para(self, forecast, holts_trend_corrected_smoothing, alpha, gamma, smoothing_level):             # <<<<<<<<<<<<<<
 *         appraised_individual = {}
 *         sum_squared_error = forecast.sum_squared_errors_indi_htces(squared_error=holts_trend_corrected_smoothing,
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_22sum_squared_error_para, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_11, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__63)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_sum_squared_error_para, __pyx_t_5) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":422
 *         return appraised_individual
 * 
 *     def _run_holts_trend_corrected_exponential_smoothing(self, individual: tuple):             # <<<<<<<<<<<<<<
 *         f = Forecast(self.__orders, self.__average_order)
 *         holts_trend_corrected_smoothing = []
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_individual, ((PyObject *)(&PyTuple_Type))) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_24_run_holts_trend_corrected_exponential_smoothing, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_12, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__65)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_run_holts_trend_corrected_expon, __pyx_t_1) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":445
 *         return appraised_individual
 * 
 *     def express_smoothing_level_genome(self, individuals_genome: list, standard_error):             # <<<<<<<<<<<<<<
 * 
 *         # stack parents and offspring into a list for next steps
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_individuals_genome, ((PyObject *)(&PyList_Type))) < 0) __PYX_ERR(0, 445, __pyx_L1_error)
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_26express_smoothing_level_genome, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_4, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__67)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_5, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_express_smoothing_level_genome, __pyx_t_5) < 0) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":452
 *             yield individuals_traits
 * 
 *     def _run_exponential_smoothing_forecast(self, individual ):             # <<<<<<<<<<<<<<
 * 
 *         f = Forecast(self.__orders, self.__average_order)
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_29_run_exponential_smoothing_forecast, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_13, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__69)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_run_exponential_smoothing_forec, __pyx_t_5) < 0) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":470
 * 
 *     @staticmethod
 *     def _express_trait(original_standard_error: float, appraised_individual: dict):             # <<<<<<<<<<<<<<
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 *         # print(original_standard_error)
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_original_standard_error, ((PyObject *)(&PyFloat_Type))) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_appraised_individual, ((PyObject *)(&PyDict_Type))) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_31_express_trait, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_14, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__71)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":469
 *         return appraised_individual
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _express_trait(original_standard_error: float, appraised_individual: dict):
 *         # fitness test? over 50% of the alleles must be positive traits. give percentage score
 */
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_express_trait, __pyx_t_1) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":482
 *         return appraised_individual
 * 
 *     def _selection_population(self, individuals_fitness: dict, appraised_individual: list):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_individuals_fitness, ((PyObject *)(&PyDict_Type))) < 0) __PYX_ERR(0, 482, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_appraised_individual, ((PyObject *)(&PyList_Type))) < 0) __PYX_ERR(0, 482, __pyx_L1_error)
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_33_selection_population, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_15, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__73)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_5, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_selection_population, __pyx_t_5) < 0) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":486
 * 
 *     def simple_exponential_smoothing_evo(self, smoothing_level_constant, initial_estimate_period,
 *                                          recombination_type = 'single_point', int population_size = 10,             # <<<<<<<<<<<<<<
 *                                          int forecast_length = 5):
 *         """ Simple exponential smoothing using evolutionary algorithm for optimising smoothing level constant (alpha value)
 */
  __pyx_t_5 = __Pyx_PyInt_From_int(((int)10)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "supplychainpy/demand/_evo_algo.pyx":487
 *     def simple_exponential_smoothing_evo(self, smoothing_level_constant, initial_estimate_period,
 *                                          recombination_type = 'single_point', int population_size = 10,
 *                                          int forecast_length = 5):             # <<<<<<<<<<<<<<
 *         """ Simple exponential smoothing using evolutionary algorithm for optimising smoothing level constant (alpha value)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(((int)5)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supplychainpy/demand/_evo_algo.pyx":485
 *         pass
 * 
 *     def simple_exponential_smoothing_evo(self, smoothing_level_constant, initial_estimate_period,             # <<<<<<<<<<<<<<
 *                                          recombination_type = 'single_point', int population_size = 10,
 *                                          int forecast_length = 5):
 */
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject*)__pyx_n_s_single_point));
  __Pyx_GIVEREF(((PyObject*)__pyx_n_s_single_point));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject*)__pyx_n_s_single_point));
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __pyx_t_5 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_13supplychainpy_6demand_9_evo_algo_38OptimiseSmoothingLevelGeneticAlgorithm_35simple_exponential_smoothing_evo, 0, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_16, NULL, __pyx_n_s_supplychainpy_demand__evo_algo, __pyx_d, ((PyObject *)__pyx_codeobj__75)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_simple_exponential_smoothing_evo, __pyx_t_1) < 0) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":248
 * 
 * 
 * class OptimiseSmoothingLevelGeneticAlgorithm:             # <<<<<<<<<<<<<<
 *     __generation = 0
 * 
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5, __pyx_empty_tuple, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_OptimiseSmoothingLevelGeneticAlg_5, __pyx_t_1) < 0) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supplychainpy/demand/_evo_algo.pyx":1
 * from random import uniform             # <<<<<<<<<<<<<<
 * 
 * from supplychainpy.demand._forecast_demand import Forecast
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init supplychainpy.demand._evo_algo", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init supplychainpy.demand._evo_algo");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs, NULL);
}
#endif  // CYTHON_FAST_PYCCALL

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = PyThreadState_GET();
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif  // CPython < 3.6
#endif  // CYTHON_FAST_PYCALL

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ArgTypeTest */
    static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

/* BytesEquals */
    static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
    static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* GetModuleGlobalName */
    static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* GetItemInt */
      static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyIntBinop */
      #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* RaiseTooManyValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyErrFetchRestore */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
      #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* SaveResetException */
        #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
        #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    return PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
        #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyIntBinop */
          #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* None */
          static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* FetchCommonType */
          static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
          static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CyFunctionObject, func.m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0 = PySequence_ITEM(arg, 0);
                if (unlikely(!arg0)) return NULL;
                result = (*meth)(self, arg0);
                Py_DECREF(arg0);
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (__pyx_CyFunctionType == NULL) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (!m->defaults)
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* SliceObject */
              static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* dict_getitem_default */
              static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    } else {
        if (default_value == Py_None)
            default_value = NULL;
        value = PyObject_CallMethodObjArgs(
            d, __pyx_n_s_get, key, default_value, NULL);
    }
#endif
    return value;
}

/* Import */
              static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
              static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CalculateMetaclass */
              static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
              static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CodeObjectCache */
              static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
              #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
              #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* Print */
              #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntFromPy */
              static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* PrintOne */
              #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* CIntFromPy */
              static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* SwapException */
              #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectCallMethod1 */
              static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            Py_DECREF(method); method = NULL;
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* CoroutineBase */
              #include <structmember.h>
#include <frameobject.h>
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
#if 1 || PY_VERSION_HEX < 0x030300B0
static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
#endif
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__pyx_CoroutineObject *self) {
    PyObject *exc_type = self->exc_type;
    PyObject *exc_value = self->exc_value;
    PyObject *exc_traceback = self->exc_traceback;
    self->exc_type = NULL;
    self->exc_value = NULL;
    self->exc_traceback = NULL;
    Py_XDECREF(exc_type);
    Py_XDECREF(exc_value);
    Py_XDECREF(exc_traceback);
}
static CYTHON_INLINE
int __Pyx_Coroutine_CheckRunning(__pyx_CoroutineObject *gen) {
    if (unlikely(gen->is_running)) {
        PyErr_SetString(PyExc_ValueError,
                        "generator already executing");
        return 1;
    }
    return 0;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value) {
    PyObject *retval;
    __Pyx_PyThreadState_declare
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            PyErr_SetString(PyExc_TypeError,
                            "can't send non-None value to a "
                            "just-started generator");
            return NULL;
        }
    }
    if (unlikely(self->resume_label == -1)) {
        PyErr_SetNone(PyExc_StopIteration);
        return NULL;
    }
    __Pyx_PyThreadState_assign
    if (value) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        if (self->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(__pyx_tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = __pyx_tstate->frame;
        }
#endif
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
    }
    self->is_running = 1;
    retval = self->body((PyObject *) self, value);
    self->is_running = 0;
    if (retval) {
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        if (self->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_CLEAR(f->f_back);
        }
#endif
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(PyObject *retval) {
    if (unlikely(!retval && !PyErr_Occurred())) {
        PyErr_SetNone(PyExc_StopIteration);
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen_FetchStopIterationValue(&val);
    ret = __Pyx_Coroutine_SendEx(gen, val);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value);
    }
    return __Pyx_Coroutine_MethodReturn(retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL);
    if (retval) {
        Py_DECREF(retval);
        PyErr_SetString(PyExc_RuntimeError,
                        "generator ignored GeneratorExit");
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (!raised_exception
        || raised_exception == PyExc_StopIteration
        || raised_exception == PyExc_GeneratorExit
        || PyErr_GivenExceptionMatches(raised_exception, PyExc_GeneratorExit)
        || PyErr_GivenExceptionMatches(raised_exception, PyExc_StopIteration))
    {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *typ;
    PyObject *tb = NULL;
    PyObject *val = NULL;
    PyObject *yf = gen->yieldfrom;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit)) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(__Pyx_Coroutine_SendEx(gen, NULL));
            goto throw_here;
        }
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Throw(yf, args);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Throw(yf, args);
        } else
        #endif
        {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            ret = PyObject_CallObject(meth, args);
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(__Pyx_Coroutine_SendEx(gen, NULL));
}
static int __Pyx_Coroutine_traverse(PyObject *self, visitproc visit, void *arg) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    Py_VISIT(gen->exc_type);
    Py_VISIT(gen->exc_value);
    Py_VISIT(gen->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    Py_CLEAR(gen->exc_type);
    Py_CLEAR(gen->exc_value);
    Py_CLEAR(gen->exc_traceback);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label > 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (self->ob_refcnt > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *res;
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label <= 0)
        return ;
#if PY_VERSION_HEX < 0x030400a1
    assert(self->ob_refcnt == 0);
    self->ob_refcnt = 1;
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
    res = __Pyx_Coroutine_Close(self);
    if (res == NULL)
        PyErr_WriteUnraisable(self);
    else
        Py_DECREF(res);
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if PY_VERSION_HEX < 0x030400a1
    assert(self->ob_refcnt > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = self->ob_refcnt;
        _Py_NewReference(self);
        self->ob_refcnt = refcnt;
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(self->ob_type) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (gen == NULL)
        return NULL;
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->exc_type = NULL;
    gen->exc_value = NULL;
    gen->exc_traceback = NULL;
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
                  static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
                  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (!abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_VERSION_HEX >= 0x03030000) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_VERSION_HEX >= 0x03030000) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if (0) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
                  static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if PY_VERSION_HEX >= 0x030400a1
    __Pyx_Coroutine_del,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = PyObject_GenericGetAttr;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
                  static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
                  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
  #else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
  #endif
#else
  res = PyNumber_Int(x);
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
